@@ -23,31 +23,60 @@
 #include <stdlib.h>           // for size_t, free, malloc, NULL
 #include <string.h>           // for memset
 
-// these clamp away insane memory requirements.
-// they should reasonably faithfully represent the
-// full precision though, so tiling will help reducing memory footprint
-// and export will look the same as darkroom mode (only 1mpix there).
-#define DT_COMMON_BILATERAL_MAX_RES_S 6000
+// These limits clamp away insane memory requirements.  They should reasonably faithfully represent the full
+// precision though, so tiling will help reduce the memory footprint and export will look the same as darkroom
+// mode (only 1mpix there).
+#define DT_COMMON_BILATERAL_MAX_RES_S 3000
 #define DT_COMMON_BILATERAL_MAX_RES_R 50
 
-#ifndef HAVE_OPENCL
-// function definition on opencl path takes precedence
+void dt_bilateral_grid_size(dt_bilateral_t *b, const int width, const int height, const float L_range,
+                            float sigma_s, const float sigma_r)
+{
+  // Callers adjust sigma_s to account for image scaling to make the bilateral filter scale-invariant.  As a
+  // result, if the user sets a small enough value for sigma, we can get sigma_s substantially below 1.0.
+  // Values < 1 generate a bilateral grid with spatial dimensions larger than the (scaled) image pixel
+  // dimensions; for sigma_s < 0.5, there is at least one unused grid point between any two used points, and
+  // thus the gaussian blur will have little effect.  So we force sigma_s to be at least 0.5 to avoid an
+  // excessively large grid.
+  if (sigma_s < 0.5) sigma_s = 0.5;
+
+  // compute an initial grid size, clamping away insanely large grids
+  float _x = CLAMPS((int)roundf(width / sigma_s), 4, DT_COMMON_BILATERAL_MAX_RES_S);
+  float _y = CLAMPS((int)roundf(height / sigma_s), 4, DT_COMMON_BILATERAL_MAX_RES_S);
+  float _z = CLAMPS((int)roundf(L_range / sigma_r), 4, DT_COMMON_BILATERAL_MAX_RES_R);
+  // If we clamped the X or Y dimensions, the sigma_s for that dimension changes.  Since we need to use the
+  // same value in both dimensions, compute the effective sigma_s for the grid.
+  b->sigma_s = MAX(height / _y, width / _x);
+  b->sigma_r = L_range / _z;
+  // Compute the grid size in light of the actual adjusted values for sigma_s and sigma_r
+  b->size_x = (int)ceilf(width / b->sigma_s) + 1;
+  b->size_y = (int)ceilf(height / b->sigma_s) + 1;
+  b->size_z = (int)ceilf(L_range / b->sigma_r) + 1;
+#if 0
+  if (b->sigma_s != sigma_s) fprintf(stderr, "[bilateral] clamped sigma_s (%g -> %g)!\n",sigma_s,b->sigma_s);
+  if (b->sigma_r != sigma_r) fprintf(stderr, "[bilateral] clamped sigma_r (%g -> %g)!\n",sigma_r,b->sigma_r);
+#endif
+}
+
 size_t dt_bilateral_memory_use(const int width,     // width of input image
                                const int height,    // height of input image
                                const float sigma_s, // spatial sigma (blur pixel coords)
                                const float sigma_r) // range sigma (blur luma values)
 {
-  float _x = roundf(width / sigma_s);
-  float _y = roundf(height / sigma_s);
-  float _z = roundf(100.0f / sigma_r);
-  size_t size_x = CLAMPS((int)_x, 4, DT_COMMON_BILATERAL_MAX_RES_S) + 1;
-  size_t size_y = CLAMPS((int)_y, 4, DT_COMMON_BILATERAL_MAX_RES_S) + 1;
-  size_t size_z = CLAMPS((int)_z, 4, DT_COMMON_BILATERAL_MAX_RES_R) + 1;
-
-  return dt_get_num_threads() * size_x * size_y * size_z * sizeof(float);
+  dt_bilateral_t b;
+  dt_bilateral_grid_size(&b,width,height,100.0f,sigma_s,sigma_r);
+  size_t grid_size = b.size_x * b.size_y * b.size_z;
+#ifdef HAVE_OPENCL
+  // OpenCL path needs two buffers
+  return 2 * grid_size * sizeof(float);
+#else
+  return (grid_size + 3 * darktable.num_openmp_threads * b.size_x * b.size_z) * sizeof(float);
+#endif /* HAVE_OPENCL */
 }
 
+#ifndef HAVE_OPENCL
 // for the CPU path this is just an alias as no additional temp buffer is needed
+// when compiling with OpenCL, version in bilateralcl.c takes precedence
 size_t dt_bilateral_memory_use2(const int width,
                                 const int height,
                                 const float sigma_s,
@@ -55,23 +84,22 @@ size_t dt_bilateral_memory_use2(const int width,
 {
   return dt_bilateral_memory_use(width, height, sigma_s, sigma_r);
 }
+#endif /* !HAVE_OPENCL */
 
 size_t dt_bilateral_singlebuffer_size(const int width,     // width of input image
                                       const int height,    // height of input image
                                       const float sigma_s, // spatial sigma (blur pixel coords)
                                       const float sigma_r) // range sigma (blur luma values)
 {
-  float _x = roundf(width / sigma_s);
-  float _y = roundf(height / sigma_s);
-  float _z = roundf(100.0f / sigma_r);
-  size_t size_x = CLAMPS((int)_x, 4, DT_COMMON_BILATERAL_MAX_RES_S) + 1;
-  size_t size_y = CLAMPS((int)_y, 4, DT_COMMON_BILATERAL_MAX_RES_S) + 1;
-  size_t size_z = CLAMPS((int)_z, 4, DT_COMMON_BILATERAL_MAX_RES_R) + 1;
-
-  return dt_get_num_threads() * size_x * size_y * size_z * sizeof(float);
+  dt_bilateral_t b;
+  dt_bilateral_grid_size(&b,width,height,100.0f,sigma_s,sigma_r);
+  size_t grid_size = b.size_x * b.size_y * b.size_z;
+  return (grid_size + 3 * darktable.num_openmp_threads * b.size_x * b.size_z) * sizeof(float);
 }
 
+#ifndef HAVE_OPENCL
 // for the CPU path this is just an alias as no additional temp buffer is needed
+// when compiling with OpenCL, version in bilateralcl.c takes precedence
 size_t dt_bilateral_singlebuffer_size2(const int width,
                                        const int height,
                                        const float sigma_s,
@@ -79,14 +107,32 @@ size_t dt_bilateral_singlebuffer_size2(const int width,
 {
   return dt_bilateral_singlebuffer_size(width, height, sigma_s, sigma_r);
 }
-#endif
+#endif /* !HAVE_OPENCL */
 
-static void image_to_grid(const dt_bilateral_t *const b, const int i, const int j, const float L, float *x,
-                          float *y, float *z)
+static size_t image_to_grid(const dt_bilateral_t *const b, const int i, const int j, const float L,
+                            float *xf, float *yf, float *zf)
 {
-  *x = CLAMPS(i / b->sigma_s, 0, b->size_x - 1);
-  *y = CLAMPS(j / b->sigma_s, 0, b->size_y - 1);
-  *z = CLAMPS(L / b->sigma_r, 0, b->size_z - 1);
+  float x = CLAMPS(i / b->sigma_s, 0, b->size_x - 1);
+  float y = CLAMPS(j / b->sigma_s, 0, b->size_y - 1);
+  float z = CLAMPS(L / b->sigma_r, 0, b->size_z - 1);
+  const int xi = MIN((int)x, b->size_x - 2);
+  const int yi = MIN((int)y, b->size_y - 2);
+  const int zi = MIN((int)z, b->size_z - 2);
+  *xf = x - xi;
+  *yf = y - yi;
+  *zf = z - zi;
+  return ((xi + yi * b->size_x) * b->size_z) + zi;
+}
+
+static size_t image_to_relgrid(const dt_bilateral_t *const b, const int i, const float L, float *xf, float *zf)
+{
+  float x = CLAMPS(i / b->sigma_s, 0, b->size_x - 1);
+  float z = CLAMPS(L / b->sigma_r, 0, b->size_z - 1);
+  const int xi = MIN((int)x, b->size_x - 2);
+  const int zi = MIN((int)z, b->size_z - 2);
+  *xf = x - xi;
+  *zf = z - zi;
+  return (xi * b->size_z) + zi;
 }
 
 dt_bilateral_t *dt_bilateral_init(const int width,     // width of input image
@@ -96,90 +142,117 @@ dt_bilateral_t *dt_bilateral_init(const int width,     // width of input image
 {
   dt_bilateral_t *b = (dt_bilateral_t *)malloc(sizeof(dt_bilateral_t));
   if(!b) return NULL;
-  // if(width/sigma_s < 4 || width/sigma_s > 1000) fprintf(stderr, "[bilateral] need to clamp sigma_s!\n");
-  // if(height/sigma_s < 4 || height/sigma_s > 1000) fprintf(stderr, "[bilateral] need to clamp sigma_s!\n");
-  // if(100.0/sigma_r < 4 || 100.0/sigma_r > 100) fprintf(stderr, "[bilateral] need to clamp sigma_r!\n");
-  float _x = roundf(width / sigma_s);
-  float _y = roundf(height / sigma_s);
-  float _z = roundf(100.0f / sigma_r);
-  b->size_x = CLAMPS((int)_x, 4, DT_COMMON_BILATERAL_MAX_RES_S) + 1;
-  b->size_y = CLAMPS((int)_y, 4, DT_COMMON_BILATERAL_MAX_RES_S) + 1;
-  b->size_z = CLAMPS((int)_z, 4, DT_COMMON_BILATERAL_MAX_RES_R) + 1;
+  dt_bilateral_grid_size(b,width,height,100.0f,sigma_s,sigma_r);
   b->width = width;
   b->height = height;
-  b->sigma_s = MAX(height / (b->size_y - 1.0f), width / (b->size_x - 1.0f));
-  b->sigma_r = 100.0f / (b->size_z - 1.0f);
-  const int nthreads = /*darktable.num_openmp_threads*/ dt_get_num_threads();
-  b->buf = dt_alloc_align(64, b->size_x * b->size_y * b->size_z * sizeof(float) * nthreads);
-
-  memset(b->buf, 0, b->size_x * b->size_y * b->size_z * sizeof(float) * nthreads);
-#if 0
-  fprintf(stderr, "[bilateral] created grid [%d %d %d]"
-          " with sigma (%f %f) (%f %f)\n", b->size_x, b->size_y, b->size_z,
-          b->sigma_s, sigma_s, b->sigma_r, sigma_r);
-#endif
+  b->numslices = darktable.num_openmp_threads;
+  b->sliceheight = (height + b->numslices - 1) / b->numslices;
+  b->slicerows = (b->size_y + b->numslices - 1) / b->numslices + 2;
+  b->buf = dt_alloc_align(64, b->size_x * b->size_z * b->numslices * b->slicerows * sizeof(float));
+  if (b->buf)
+  {
+    memset(b->buf, 0, b->size_x * b->size_z * b->numslices * b->slicerows * sizeof(float));
+  }
+  else
+  {
+    fprintf(stderr,"[bilateral] unable to allocate buffer for %lux%lux%lu grid\n",b->size_x,b->size_y,b->size_z);
+  }
+  dt_print(DT_DEBUG_DEV, "[bilateral] created grid [%ld %ld %ld] with sigma (%f %f) (%f %f)\n",
+           b->size_x, b->size_y, b->size_z, b->sigma_s, sigma_s, b->sigma_r, sigma_r);
   return b;
 }
 
 #ifdef _OPENMP
 #pragma omp declare simd aligned(in:64)
 #endif
-void dt_bilateral_splat(dt_bilateral_t *b, const float *const in)
+void dt_bilateral_splat(const dt_bilateral_t *b, const float *const in)
 {
-  const int ox = 1;
-  const int oy = b->size_x;
-  const int oz = b->size_y * b->size_x;
+  const int ox = b->size_z;
+  const int oy = b->size_x * b->size_z;
+  const int oz = 1;
   const float sigma_s = b->sigma_s * b->sigma_s;
   float *const buf = b->buf;
-  const int bufsize = b->size_x * b->size_y * b->size_z;
 
-// splat into downsampled grid
+  if (!buf) return;
+  // splat into downsampled grid
+  const int nthreads = darktable.num_openmp_threads;
+  const size_t offsets[8] =
+  {
+    0,
+    ox,
+    oy,
+    ox + oy,
+    oz,
+    oz + ox,
+    oz + oy,
+    oz + oy + ox
+  };
+
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-  dt_omp_firstprivate(in, oy, oz, ox, sigma_s, buf, bufsize)    \
-  shared(b) \
-  collapse(2)
+  dt_omp_firstprivate(in, oy, oz, ox, sigma_s, buf, offsets) \
+  shared(b)
 #endif
-  for(int j = 0; j < b->height; j++)
+  for(int slice = 0; slice < b->numslices; slice++)
   {
-    for(int i = 0; i < b->width; i++)
+    const int firstrow = slice * b->sliceheight;
+    const int lastrow = MIN((slice+1)*b->sliceheight,b->height);
+    // compute the first row of the final grid which this slice splats, and subtract that from the first
+    // row the current thread should use to get an offset
+    const int slice_offset = slice * b->slicerows - (int)(firstrow / b->sigma_s);
+    // now iterate over the rows of the current horizontal slice
+    for(int j = firstrow; j < lastrow; j++)
     {
-      size_t index = 4 * (j * b->width + i);
-      float x, y, z;
-      const float L = in[index];
-      image_to_grid(b, i, j, L, &x, &y, &z);
-      const int xi = MIN((int)x, b->size_x - 2);
+      float y = CLAMPS(j / b->sigma_s, 0, b->size_y - 1);
       const int yi = MIN((int)y, b->size_y - 2);
-      const int zi = MIN((int)z, b->size_z - 2);
-      const float xf = x - xi;
       const float yf = y - yi;
-      const float zf = z - zi;
-      // nearest neighbour splatting:
-      const size_t grid_index = xi + b->size_x * (yi + b->size_y * zi) + bufsize * dt_get_thread_num();
-      // sum up payload here, doesn't have to be same as edge stopping data
-      // for cross bilateral applications.
-      // also note that this is not clipped (as L->z is), so potentially hdr/out of gamut
-      // should not cause clipping here.
+      const size_t base = (yi + slice_offset) * oy;
+      for(int i = 0; i < b->width; i++)
+      {
+        size_t index = 4 * (j * b->width + i);
+        float xf, zf;
+        const float L = in[index];
+        // nearest neighbour splatting:
+        const size_t grid_index = base + image_to_relgrid(b, i, L, &xf, &zf);
+        // sum up payload here
+        const float contrib[4] =
+        {
+          (1.0f - xf) * (1.0f - yf) * 100.0f / sigma_s,	// precompute the contributions along the first two dimensions
+          xf * (1.0f - yf) * 100.0f / sigma_s,
+          (1.0f - xf) * yf * 100.0f / sigma_s,
+          xf * yf * 100.0f / sigma_s
+        };
 #ifdef _OPENMP
 #pragma omp simd aligned(buf:64)
 #endif
-      for(int k = 0; k < 8; k++)
-      {
-        const size_t ii = grid_index + ((k & 1) ? ox : 0) + ((k & 2) ? oy : 0) + ((k & 4) ? oz : 0);
-        const float contrib = ((k & 1) ? xf : (1.0f - xf)) * ((k & 2) ? yf : (1.0f - yf))
-                              * ((k & 4) ? zf : (1.0f - zf)) * 100.0f / sigma_s;
-        buf[ii] += contrib;
+        for(int k = 0; k < 4; k++)
+        {
+          buf[grid_index + offsets[k]] += (contrib[k] * (1.0f - zf));
+          buf[grid_index + offsets[k+4]] += (contrib[k] * zf);
+        }
       }
     }
   }
 
   // merge the per-thread results into the final result
-  const int nthreads = /*darktable.num_openmp_threads*/ dt_get_num_threads();
-  for(int index = 0; index < bufsize; index++)
+  for (int slice = 1 ; slice < nthreads; slice++)
   {
-    for(int i = 1; i < nthreads; i++)
+    // compute the first row of the final grid which this slice splats
+    const int destrow = (int)(slice * b->sliceheight / b->sigma_s);
+    float *dest = buf + destrow * oy;
+    // now iterate over the grid rows splatted for this slice
+    for(int j = slice * b->slicerows; j < (slice+1)*b->slicerows; j++)
     {
-      buf[index] += buf[index + i*bufsize];
+      float *src = buf + j * oy;
+      for(int i = 0; i < oy; i++)
+      {
+        dest[i] += src[i];
+      }
+      dest += oy;
+      // clear elements in the part of the buffer which holds the final result now that we've read the partial result,
+      // since we'll be adding to those locations later
+      if (j < b->size_y)
+        memset(buf + j*oy, '\0', oy*sizeof(float));
     }
   }
 }
@@ -271,14 +344,19 @@ static void blur_line(float *buf, const int offset1, const int offset2, const in
 }
 
 
-void dt_bilateral_blur(dt_bilateral_t *b)
+void dt_bilateral_blur(const dt_bilateral_t *b)
 {
+  if (!b || !b->buf)
+    return;
+  const int ox = b->size_z;
+  const int oy = b->size_x * b->size_z;
+  const int oz = 1;
   // gaussian up to 3 sigma
-  blur_line(b->buf, b->size_x * b->size_y, b->size_x, 1, b->size_z, b->size_y, b->size_x);
+  blur_line(b->buf, oz, oy, ox, b->size_z, b->size_y, b->size_x);
   // gaussian up to 3 sigma
-  blur_line(b->buf, b->size_x * b->size_y, 1, b->size_x, b->size_z, b->size_x, b->size_y);
+  blur_line(b->buf, oz, ox, oy, b->size_z, b->size_x, b->size_y);
   // -2 derivative of the gaussian up to 3 sigma: x*exp(-x*x)
-  blur_line_z(b->buf, 1, b->size_x, b->size_x * b->size_y, b->size_x, b->size_y, b->size_z);
+  blur_line_z(b->buf, ox, oy, oz, b->size_x, b->size_y, b->size_z);
 }
 
 
@@ -289,19 +367,17 @@ void dt_bilateral_slice(const dt_bilateral_t *const b, const float *const in, fl
 {
   // detail: 0 is leave as is, -1 is bilateral filtered, +1 is contrast boost
   const float norm = -detail * b->sigma_r * 0.04f;
-  const int ox = 1;
-  const int oy = b->size_x;
-  const int oz = b->size_y * b->size_x;
+  const int ox = b->size_z;
+  const int oy = b->size_x * b->size_z;
+  const int oz = 1;
   float *const buf = b->buf;
-  const int size_x = b->size_x;
-  const int size_y = b->size_y;
-  const int size_z = b->size_z;
   const int width = b->width;
   const int height = b->height;
 
+  if (!buf) return;
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-  dt_omp_firstprivate(b, in, norm, ox, oy, oz, size_x, size_y, size_z, height, width, buf) \
+  dt_omp_firstprivate(b, in, norm, ox, oy, oz, height, width, buf) \
     shared(out) collapse(2)
 #endif
   for(int j = 0; j < height; j++)
@@ -309,17 +385,10 @@ void dt_bilateral_slice(const dt_bilateral_t *const b, const float *const in, fl
     for(int i = 0; i < width; i++)
     {
       size_t index = 4 * (j * width + i);
-      float x, y, z;
+      float xf, yf, zf;
       const float L = in[index];
-      image_to_grid(b, i, j, L, &x, &y, &z);
       // trilinear lookup:
-      const int xi = MIN((int)x, size_x - 2);
-      const int yi = MIN((int)y, size_y - 2);
-      const int zi = MIN((int)z, size_z - 2);
-      const float xf = x - xi;
-      const float yf = y - yi;
-      const float zf = z - zi;
-      const size_t gi = xi + size_x * (yi + size_y * zi);
+      const size_t gi = image_to_grid(b, i, j, L, &xf, &yf, &zf);
       const float Lout = L
                          + norm * (buf[gi] * (1.0f - xf) * (1.0f - yf) * (1.0f - zf)
                                    + buf[gi + ox] * (xf) * (1.0f - yf) * (1.0f - zf)
@@ -346,19 +415,17 @@ void dt_bilateral_slice_to_output(const dt_bilateral_t *const b, const float *co
 {
   // detail: 0 is leave as is, -1 is bilateral filtered, +1 is contrast boost
   const float norm = -detail * b->sigma_r * 0.04f;
-  const int ox = 1;
-  const int oy = b->size_x;
-  const int oz = b->size_y * b->size_x;
+  const int ox = b->size_z;
+  const int oy = b->size_x * b->size_z;
+  const int oz = 1;
   float *const buf = b->buf;
-  const int size_x = b->size_x;
-  const int size_y = b->size_y;
-  const int size_z = b->size_z;
   const int width = b->width;
   const int height = b->height;
 
+  if (!buf) return;
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-  dt_omp_firstprivate(b, in, norm, oy, oz, ox, buf, size_x, size_y, size_z, width, height) \
+  dt_omp_firstprivate(b, in, norm, oy, oz, ox, buf, width, height) \
   shared(out) collapse(2)
 #endif
   for(int j = 0; j < height; j++)
@@ -366,17 +433,10 @@ void dt_bilateral_slice_to_output(const dt_bilateral_t *const b, const float *co
     for(int i = 0; i < width; i++)
     {
       size_t index = 4 * (j * width + i);
-      float x, y, z;
+      float xf, yf, zf;
       const float L = in[index];
-      image_to_grid(b, i, j, L, &x, &y, &z);
       // trilinear lookup:
-      const int xi = MIN((int)x, size_x - 2);
-      const int yi = MIN((int)y, size_y - 2);
-      const int zi = MIN((int)z, size_z - 2);
-      const float xf = x - xi;
-      const float yf = y - yi;
-      const float zf = z - zi;
-      const size_t gi = xi + size_x * (yi + size_y * zi);
+      const size_t gi = image_to_grid(b, i, j, L, &xf, &yf, &zf);
       const float Lout = norm * (buf[gi] * (1.0f - xf) * (1.0f - yf) * (1.0f - zf)
                                  + buf[gi + ox] * (xf) * (1.0f - yf) * (1.0f - zf)
                                  + buf[gi + oy] * (1.0f - xf) * (yf) * (1.0f - zf)
