@@ -18,6 +18,8 @@
 
 #pragma once
 
+#include "common/iop_profile.h"
+
 struct dt_iop_buffer_dsc_t;
 struct dt_iop_roi_t;
 enum dt_iop_colorspace_type_t;
@@ -25,7 +27,9 @@ enum dt_iop_colorspace_type_t;
 void dt_color_picker_helper(const struct dt_iop_buffer_dsc_t *dsc, const float *const pixel,
                             const struct dt_iop_roi_t *roi, const int *const box, float *const picked_color,
                             float *const picked_color_min, float *const picked_color_max,
-                            const enum dt_iop_colorspace_type_t image_cst, const enum dt_iop_colorspace_type_t picker_cst);
+                            const enum dt_iop_colorspace_type_t image_cst,
+                            const enum dt_iop_colorspace_type_t picker_cst,
+                            const dt_iop_order_iccprofile_info_t *const profile);
 
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
