@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 johannes hanika.
+    Copyright (C) 2009-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,8 +26,12 @@
 extern "C" {
 #endif
 
+/** set the list of available tags from Exvi2 */
+void dt_exif_set_exiv2_taglist();
+
 /** get the list of available tags from Exvi2 */
-GList *dt_get_exiv2_taglist();
+/** must not be freed */
+const GList * const dt_exif_get_exiv2_taglist();
 
 /** read metadata from file with full path name, XMP data trumps IPTC data trumps EXIF data, store to image
  * struct. returns 0 on success. */
@@ -42,7 +46,7 @@ int dt_exif_read_blob(uint8_t **blob, const char *path, const int imgid, const i
                       const int out_height, const int dng_mode);
 
 /** Reads exif DefaultUserCrop */
-void dt_img_check_usercrop(dt_image_t *img, const char *filename);
+void dt_exif_img_check_usercrop(dt_image_t *img, const char *filename);
 
 /** write blob to file exif. merges with existing exif information.*/
 int dt_exif_write_blob(uint8_t *blob, uint32_t size, const char *path, const int compressed);
@@ -59,6 +63,9 @@ char *dt_exif_xmp_read_string(const int imgid);
 /** read xmp sidecar file. */
 int dt_exif_xmp_read(dt_image_t *img, const char *filename, const int history_only);
 
+/** apply default import metadata */
+void dt_exif_apply_default_metadata(dt_image_t *img);
+
 /** fetch largest exif thumbnail jpg bytestream into buffer*/
 int dt_exif_get_thumbnail(const char *path, uint8_t **buffer, size_t *size, char **mime_type);
 
