@@ -24,7 +24,7 @@
 #include "gui/gtk.h"
 
 /** add an image to a group */
-void dt_grouping_add_to_group(const int group_id, const int image_id)
+void dt_grouping_add_to_group(const int group_id, const int32_t image_id)
 {
   // remove from old group
   dt_grouping_remove_from_group(image_id);
@@ -34,11 +34,11 @@ void dt_grouping_add_to_group(const int group_id, const int image_id)
   dt_image_cache_write_release(darktable.image_cache, img, DT_IMAGE_CACHE_SAFE);
   GList *imgs = NULL;
   imgs = g_list_prepend(imgs, GINT_TO_POINTER(image_id));
-  dt_control_signal_raise(darktable.signals, DT_SIGNAL_IMAGE_INFO_CHANGED, imgs);
+  DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_IMAGE_INFO_CHANGED, imgs);
 }
 
 /** remove an image from a group */
-int dt_grouping_remove_from_group(const int image_id)
+int dt_grouping_remove_from_group(const int32_t image_id)
 {
   sqlite3_stmt *stmt;
   int new_group_id = -1;
@@ -85,18 +85,18 @@ int dt_grouping_remove_from_group(const int image_id)
     // refresh also the group leader which may be alone now
     imgs = g_list_prepend(imgs, GINT_TO_POINTER(img_group_id));
   }
-  dt_control_signal_raise(darktable.signals, DT_SIGNAL_IMAGE_INFO_CHANGED, imgs);
+  DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_IMAGE_INFO_CHANGED, imgs);
 
   return new_group_id;
 }
 
 /** make an image the representative of the group it is in */
-int dt_grouping_change_representative(const int image_id)
+int dt_grouping_change_representative(const int32_t image_id)
 {
   sqlite3_stmt *stmt;
 
   dt_image_t *img = dt_image_cache_get(darktable.image_cache, image_id, 'r');
-  int group_id = img->group_id;
+  const int group_id = img->group_id;
   dt_image_cache_read_release(darktable.image_cache, img);
 
   GList *imgs = NULL;
@@ -105,20 +105,20 @@ int dt_grouping_change_representative(const int image_id)
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, group_id);
   while(sqlite3_step(stmt) == SQLITE_ROW)
   {
-    int other_id = sqlite3_column_int(stmt, 0);
+    const int other_id = sqlite3_column_int(stmt, 0);
     dt_image_t *other_img = dt_image_cache_get(darktable.image_cache, other_id, 'w');
     other_img->group_id = image_id;
     dt_image_cache_write_release(darktable.image_cache, other_img, DT_IMAGE_CACHE_SAFE);
     imgs = g_list_prepend(imgs, GINT_TO_POINTER(other_id));
   }
   sqlite3_finalize(stmt);
-  dt_control_signal_raise(darktable.signals, DT_SIGNAL_IMAGE_INFO_CHANGED, imgs);
+  DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_IMAGE_INFO_CHANGED, imgs);
 
   return image_id;
 }
 
 /** get images of the group */
-GList *dt_grouping_get_group_images(const int imgid)
+GList *dt_grouping_get_group_images(const int32_t imgid)
 {
   GList *imgs = NULL;
   const dt_image_t *image = dt_image_cache_get(darktable.image_cache, imgid, 'r');
