@@ -29,13 +29,34 @@
 */
 
 #include "common/guided_filter.h"
-#include "common/darktable.h"
 #include "common/opencl.h"
 #include <assert.h>
 #include <float.h>
 #include <stdlib.h>
 #include <string.h>
 
+// processing is split into tiles of this size (or three times the filter
+// width, if greater) to keep memory use under control.
+#define GF_TILE_SIZE 512
+
+// some shorthand to make code more legible
+// if we have OpenMP simd enabled, declare a vectorizable for loop;
+// otherwise, just leave it a plain for()
+#if defined(_OPENMP) && defined(OPENMP_SIMD_)
+#define SIMD_FOR \
+  _Pragma("omp simd") \
+  for
+#else
+#define SIMD_FOR for
+#endif
+
+// avoid cluttering the scalar codepath with #ifdefs by hiding the dependency on SSE2
+#ifndef __SSE2__
+# define _mm_prefetch(where,hint)
+#endif
+
+// the filter does internal tiling to keep memory requirements reasonable, so this structure
+// defines the position of the tile being processed
 typedef struct tile
 {
   int left, right, lower, upper;
@@ -47,79 +68,63 @@ typedef struct color_image
   int width, height, stride;
 } color_image;
 
-static inline float *get_color_pixel(color_image img, size_t i)
+// allocate space for n-component image of size width x height
+static inline color_image new_color_image(int width, int height, int ch)
 {
-  return img.data + i * img.stride;
+  return (color_image){ dt_alloc_align(64, sizeof(float) * width * height * ch), width, height, ch };
 }
 
-typedef struct gray_image
-{
-  float *data;
-  int width, height;
-} gray_image;
-
-// allocate space for 1-component image of size width x height
-static inline gray_image new_gray_image(int width, int height)
-{
-  return (gray_image){ dt_alloc_align(64, sizeof(float) * width * height), width, height };
-}
-
-// free space for 1-component image
-static inline void free_gray_image(gray_image *img_p)
+// free space for n-component image
+static inline void free_color_image(color_image *img_p)
 {
   dt_free_align(img_p->data);
   img_p->data = NULL;
 }
 
-// minimum of two integers
-static inline int min_i(int a, int b)
+// get a pointer to pixel number 'i' within the image
+static inline float *get_color_pixel(color_image img, size_t i)
 {
-  return a < b ? a : b;
+  return img.data + i * img.stride;
 }
 
-// maximum of two integers
-static inline int max_i(int a, int b)
+// calculate the one-dimensional moving average over a window of size 2*w+1 independently for each of four channels
+// input array x has stride 4, output array y has stride stride_y
+static inline void box_mean_1d_4ch(int N, const float *x, float *y, size_t stride_y, int w)
 {
-  return a > b ? a : b;
-}
-
-// Kahan summation algorithm
-static inline float Kahan_sum(const float m, float *c, const float add)
-{
-   const float t1 = add - (*c);
-   const float t2 = m + t1;
-   *c = (t2 - m) - t1;
-   return t2;
-}
-
-// calculate the one-dimensional moving average over a window of size 2*w+1
-// input array x has stride 1, output array y has stride stride_y
-static inline void box_mean_1d(int N, const float *x, float *y, size_t stride_y, int w)
-{
-  float m = 0.f, n_box = 0.f, c = 0.f;
+  float n_box = 0.f, m[4] = { 0.f, 0.f, 0.f, 0.f }, c[4] = { 0.f, 0.f, 0.f, 0.f };
   if(N > 2 * w)
   {
     for(int i = 0, i_end = w + 1; i < i_end; i++)
     {
-      m = Kahan_sum(m, &c, x[i]);
+      SIMD_FOR (int k = 0; k < 4; k++)
+        m[k] = Kahan_sum(m[k], &c[k], x[4*i+k]);
       n_box++;
     }
     for(int i = 0, i_end = w; i < i_end; i++)
     {
-      y[i * stride_y] = m / n_box;
-      m = Kahan_sum(m, &c, x[i + w + 1]);
+      SIMD_FOR (int k = 0; k < 4; k++)
+      {
+        y[i * stride_y + k] = m[k] / n_box;
+        m[k] = Kahan_sum(m[k], &c[k], x[4*(i + w + 1) + k]);
+      }
       n_box++;
     }
     for(int i = w, i_end = N - w - 1; i < i_end; i++)
     {
-      y[i * stride_y] = m / n_box;
-      m = Kahan_sum(m, &c, x[i + w + 1]),
-      m = Kahan_sum(m, &c, -x[i - w]);
+      SIMD_FOR (int k = 0; k < 4; k++)
+      {
+        y[i * stride_y + k] = m[k] / n_box;
+        m[k] = Kahan_sum(m[k], &c[k], x[4*(i + w + 1) + k]);
+        m[k] = Kahan_sum(m[k], &c[k], -x[4*(i - w) + k]);
+      }
     }
     for(int i = N - w - 1, i_end = N; i < i_end; i++)
     {
-      y[i * stride_y] = m / n_box;
-      m = Kahan_sum(m, &c, -x[i - w]);
+      SIMD_FOR (int k = 0; k < 4; k++)
+      {
+        y[i * stride_y + k] = m[k] / n_box;
+        m[k] = Kahan_sum(m[k], &c[k], -x[4*(i - w) + k]);
+      }
       n_box--;
     }
   }
@@ -127,57 +132,163 @@ static inline void box_mean_1d(int N, const float *x, float *y, size_t stride_y,
   {
     for(int i = 0, i_end = min_i(w + 1, N); i < i_end; i++)
     {
-      m = Kahan_sum(m, &c, x[i]);
+      SIMD_FOR (int k = 0; k < 4; k++)
+      {
+        m[k] = Kahan_sum(m[k], &c[k], x[4*i + k]);
+      }
       n_box++;
     }
     for(int i = 0; i < N; i++)
     {
-      y[i * stride_y] = m / n_box;
+      SIMD_FOR (int k = 0; k < 4; k++)
+        y[i * stride_y + k] = m[k] / n_box;
       if(i - w >= 0)
       {
-        m = Kahan_sum(m, &c, -x[i - w]);
+        SIMD_FOR (int k = 0; k < 4; k++)
+          m[k] = Kahan_sum(m[k], &c[k], -x[4*(i - w)+k]);
         n_box--;
       }
       if(i + w + 1 < N)
       {
-        m = Kahan_sum(m, &c, x[i + w + 1]);
+        SIMD_FOR (int k = 0; k < 4; k++)
+          m[k] = Kahan_sum(m[k], &c[k], x[4*(i + w + 1)+k]);
         n_box++;
       }
     }
   }
 }
 
-// calculate the two-dimensional moving average over a box of size (2*w+1) x (2*w+1)
-// does the calculation in-place if input and output images are identical
-// this function is always called from a OpenMP thread, thus no parallelization
-static void box_mean(gray_image img1, gray_image img2, int w)
+// calculate the one-dimensional moving average over a window of size 2*w+1 independently for each of eight channels
+// input array x has stride 9, output array y has stride stride_y
+static inline void box_mean_1d_9ch(int N, const float *x, float *y, size_t stride_y, int w)
 {
-  gray_image img2_bak;
-  if(img1.data == img2.data)
+  float n_box = 0.f, m[9] = { 0.f, 0.f, 0.f, 0.f, 0.f, 0.f, 0.f, 0.f };
+  float c[9] = { 0.f, 0.f, 0.f, 0.f, 0.f, 0.f, 0.f, 0.f };
+  if(N > 2 * w)
   {
-    img2_bak = new_gray_image(max_i(img2.width, img2.height), 1);
-    for(int i1 = 0; i1 < img2.height; i1++)
+    for(int i = 0, i_end = w + 1; i < i_end; i++)
     {
-      memcpy(img2_bak.data, img2.data + (size_t)i1 * img2.width, sizeof(float) * img2.width);
-      box_mean_1d(img2.width, img2_bak.data, img2.data + (size_t)i1 * img2.width, 1, w);
+      SIMD_FOR (int k = 0; k < 9; k++)
+        m[k] = Kahan_sum(m[k], &c[k], x[9*i+k]);
+      n_box++;
+    }
+    for(int i = 0, i_end = w; i < i_end; i++)
+    {
+      SIMD_FOR (int k = 0; k < 9; k++)
+      {
+        y[i * stride_y + k] = m[k] / n_box;
+        m[k] = Kahan_sum(m[k], &c[k], x[9*(i + w + 1) + k]);
+      }
+      n_box++;
+    }
+    for(int i = w, i_end = N - w - 1; i < i_end; i++)
+    {
+      SIMD_FOR (int k = 0; k < 9; k++)
+      {
+        y[i * stride_y + k] = m[k] / n_box;
+        m[k] = Kahan_sum(m[k], &c[k], x[9*(i + w + 1) + k]);
+        m[k] = Kahan_sum(m[k], &c[k], -x[9*(i - w) + k]);
+      }
+    }
+    for(int i = N - w - 1, i_end = N; i < i_end; i++)
+    {
+      SIMD_FOR (int k = 0; k < 9; k++)
+      {
+        y[i * stride_y + k] = m[k] / n_box;
+        m[k] = Kahan_sum(m[k], &c[k], -x[9*(i - w) + k]);
+      }
+      n_box--;
     }
   }
   else
   {
-    for(int i1 = 0; i1 < img1.height; i1++)
-      box_mean_1d(img1.width, img1.data + (size_t)i1 * img1.width, img2.data + (size_t)i1 * img2.width, 1, w);
-    img2_bak = new_gray_image(1, img2.height);
+    for(int i = 0, i_end = min_i(w + 1, N); i < i_end; i++)
+    {
+      SIMD_FOR (int k = 0; k < 9; k++)
+      {
+        m[k] = Kahan_sum(m[k], &c[k], x[9*i + k]);
+      }
+      n_box++;
+    }
+    for(int i = 0; i < N; i++)
+    {
+      SIMD_FOR (int k = 0; k < 9; k++)
+        y[i * stride_y + k] = m[k] / n_box;
+      if(i - w >= 0)
+      {
+        SIMD_FOR (int k = 0; k < 9; k++)
+          m[k] = Kahan_sum(m[k], &c[k], -x[9*(i - w)+k]);
+        n_box--;
+      }
+      if(i + w + 1 < N)
+      {
+        SIMD_FOR (int k = 0; k < 9; k++)
+          m[k] = Kahan_sum(m[k], &c[k], x[9*(i + w + 1)+k]);
+        n_box++;
+      }
+    }
   }
-  for(int i0 = 0; i0 < img1.width; i0++)
-  {
-    for(int i1 = 0; i1 < img1.height; i1++) img2_bak.data[i1] = img2.data[i0 + (size_t)i1 * img2.width];
-    box_mean_1d(img1.height, img2_bak.data, img2.data + i0, img1.width, w);
-  }
-  free_gray_image(&img2_bak);
 }
 
-// apply guided filter to single-component image img using the 3-components
-// image imgg as a guide
+// in-place calculate the two-dimensional moving average of a four-channel image over a box of size (2*w+1) x (2*w+1)
+static void box_mean_4ch(color_image img, int w)
+{
+  const size_t size = 4 * max_i(img.width, img.height);
+  float *img_bak = dt_alloc_align(64, dt_get_num_threads() * size * sizeof(float));
+  const size_t width = 4 * img.width;
+#ifdef _OPENMP
+#pragma omp parallel for schedule(static) default(none) dt_omp_firstprivate(w, size, width, img_bak) shared(img)
+#endif
+  for(int i1 = 0; i1 < img.height; i1++)
+  {
+    float *buf = img_bak + dt_get_thread_num() * size;
+    memcpy(buf, img.data + (size_t)i1 * width, sizeof(float) * width);
+    box_mean_1d_4ch(img.width, buf, img.data + (size_t)i1 * width, 4, w);
+  }
+#ifdef _OPENMP
+#pragma omp parallel for schedule(static) default(none) dt_omp_firstprivate(w, size, width, img_bak) shared(img)
+#endif
+  for(int i0 = 0; i0 < img.width; i0++)
+  {
+    float *buf = img_bak + dt_get_thread_num() * size;
+    for(int i1 = 0; i1 < img.height; i1++)
+      SIMD_FOR (int k = 0; k < 4; k++)
+        buf[4*i1+k] = img.data[4*(i0 + (size_t)i1 * img.width)+k];
+    box_mean_1d_4ch(img.height, buf, img.data + 4*i0, width, w);
+  }
+  dt_free_align(img_bak);
+}
+
+// in-place calculate the two-dimensional moving average of a four-channel image over a box of size (2*w+1) x (2*w+1)
+static void box_means_vert(float *img_bak, int w, color_image mean, color_image var)
+{
+  const size_t size = 9 * max_i(mean.width, mean.height);
+#ifdef _OPENMP
+#pragma omp parallel for schedule(static) default(none) dt_omp_firstprivate(w, size, img_bak) \
+  shared(mean, var)
+#endif
+  for(int i0 = 0; i0 < mean.width; i0++)
+  {
+    float *buf = img_bak + dt_get_thread_num() * size;
+    for(int i1 = 0; i1 < mean.height; i1++)
+      SIMD_FOR (int k = 0; k < 4; k++)
+        buf[4*i1+k] = mean.data[4*(i0 + (size_t)i1 * mean.width)+k];
+    box_mean_1d_4ch(mean.height, buf, mean.data + 4*i0, 4 * mean.width, w);
+    for(int i1 = 0; i1 < var.height; i1++)
+      SIMD_FOR (int k = 0; k < 9; k++)
+        buf[9*i1+k] = var.data[9*(i0 + (size_t)i1 * var.width)+k];
+    box_mean_1d_9ch(var.height, buf, var.data + 9*i0, 9 * var.width, w);
+  }
+}
+
+// apply guided filter to single-component image img using the 3-components image imgg as a guide
+// the filtering applies a monochrome box filter to a total of 13 image channels:
+//    1 monochrome input image
+//    3 color guide image
+//    3 covariance (R, G, B)
+//    6 variance (R-R, R-G, R-B, G-G, G-B, B-B)
+// for computational efficiency, we'll pack them into a four-channel image and a 9-channel image
+// image instead of running 13 separate box filters: guide+input, R/G/B/R-R/R-G/R-B/G-G/G-B/B-B.
 static void guided_filter_tiling(color_image imgg, gray_image img, gray_image img_out, tile target, const int w,
                                  const float eps, const float guide_weight, const float min, const float max)
 {
@@ -186,141 +297,131 @@ static void guided_filter_tiling(color_image imgg, gray_image img, gray_image im
   const int width = source.right - source.left;
   const int height = source.upper - source.lower;
   size_t size = (size_t)width * (size_t)height;
-  gray_image imgg_mean_r = new_gray_image(width, height);
-  gray_image imgg_mean_g = new_gray_image(width, height);
-  gray_image imgg_mean_b = new_gray_image(width, height);
-  gray_image img_mean = new_gray_image(width, height);
+// since we're packing multiple monochrome planes into a color image, define symbolic constants so that
+// we can keep track of which values we're actually using
+#define INP_MEAN 0
+#define GUIDE_MEAN_R 1
+#define GUIDE_MEAN_G 2
+#define GUIDE_MEAN_B 3
+#define COV_R 0
+#define COV_G 1
+#define COV_B 2
+#define VAR_RR 3
+#define VAR_RG 4
+#define VAR_RB 5
+#define VAR_GG 6
+#define VAR_BB 8
+#define VAR_GB 7
+  color_image mean = new_color_image(width, height, 4);
+  color_image variance = new_color_image(width, height, 9);
+  const size_t img_dimen = max_i(mean.width, mean.height);
+  const size_t img_bak_sz = 13 * img_dimen;
+  float *img_bak = dt_alloc_align(64, dt_get_num_threads() * img_bak_sz * sizeof(float));
+#ifdef _OPENMP
+#pragma omp parallel for schedule(static) default(none) shared(img, imgg, mean, variance, img_bak) \
+  dt_omp_firstprivate(img_bak_sz, img_dimen, w, guide_weight) dt_omp_sharedconst(source)
+#endif
   for(int j_imgg = source.lower; j_imgg < source.upper; j_imgg++)
   {
     int j = j_imgg - source.lower;
+    // put values directly into temp buffer to avoid a memcpy for the horizontal box mean pass
+    float *const meanpx = img_bak + dt_get_thread_num() * img_bak_sz;
+    float *const varpx = meanpx + 4*img_dimen;
     for(int i_imgg = source.left; i_imgg < source.right; i_imgg++)
     {
-      int i = i_imgg - source.left;
-      float *pixel = get_color_pixel(imgg, i_imgg + (size_t)j_imgg * imgg.width);
-      size_t k = i + (size_t)j * width;
-      imgg_mean_r.data[k] = pixel[0] * guide_weight;
-      imgg_mean_g.data[k] = pixel[1] * guide_weight;
-      imgg_mean_b.data[k] = pixel[2] * guide_weight;
-      img_mean.data[k] = img.data[i_imgg + (size_t)j_imgg * img.width];
-    }
-  }
-  box_mean(imgg_mean_r, imgg_mean_r, w);
-  box_mean(imgg_mean_g, imgg_mean_g, w);
-  box_mean(imgg_mean_b, imgg_mean_b, w);
-  box_mean(img_mean, img_mean, w);
-  gray_image cov_imgg_img_r = new_gray_image(width, height);
-  gray_image cov_imgg_img_g = new_gray_image(width, height);
-  gray_image cov_imgg_img_b = new_gray_image(width, height);
-  gray_image var_imgg_rr = new_gray_image(width, height);
-  gray_image var_imgg_gg = new_gray_image(width, height);
-  gray_image var_imgg_bb = new_gray_image(width, height);
-  gray_image var_imgg_rg = new_gray_image(width, height);
-  gray_image var_imgg_rb = new_gray_image(width, height);
-  gray_image var_imgg_gb = new_gray_image(width, height);
-  for(int j_imgg = source.lower; j_imgg < source.upper; j_imgg++)
-  {
-    int j = j_imgg - source.lower;
-    for(int i_imgg = source.left; i_imgg < source.right; i_imgg++)
-    {
-      int i = i_imgg - source.left;
-      float *pixel_ = get_color_pixel(imgg, i_imgg + (size_t)j_imgg * imgg.width);
+      size_t i = i_imgg - source.left;
+      const float *pixel_ = get_color_pixel(imgg, i_imgg + (size_t)j_imgg * imgg.width);
       float pixel[3] = { pixel_[0] * guide_weight, pixel_[1] * guide_weight, pixel_[2] * guide_weight };
-      size_t k = i + (size_t)j * width;
-      cov_imgg_img_r.data[k] = pixel[0] * img.data[i_imgg + (size_t)j_imgg * imgg.width];
-      cov_imgg_img_g.data[k] = pixel[1] * img.data[i_imgg + (size_t)j_imgg * imgg.width];
-      cov_imgg_img_b.data[k] = pixel[2] * img.data[i_imgg + (size_t)j_imgg * imgg.width];
-      var_imgg_rr.data[k] = pixel[0] * pixel[0];
-      var_imgg_rg.data[k] = pixel[0] * pixel[1];
-      var_imgg_rb.data[k] = pixel[0] * pixel[2];
-      var_imgg_gg.data[k] = pixel[1] * pixel[1];
-      var_imgg_gb.data[k] = pixel[1] * pixel[2];
-      var_imgg_bb.data[k] = pixel[2] * pixel[2];
+      const float input = img.data[i_imgg + (size_t)j_imgg * img.width];
+      meanpx[4*i+INP_MEAN] = input;
+      meanpx[4*i+GUIDE_MEAN_R] = pixel[0];
+      meanpx[4*i+GUIDE_MEAN_G] = pixel[1];
+      meanpx[4*i+GUIDE_MEAN_B] = pixel[2];
+      varpx[9*i+COV_R] = pixel[0] * input;
+      varpx[9*i+COV_G] = pixel[1] * input;
+      varpx[9*i+COV_B] = pixel[2] * input;
+      varpx[9*i+VAR_RR] = pixel[0] * pixel[0];
+      varpx[9*i+VAR_RG] = pixel[0] * pixel[1];
+      varpx[9*i+VAR_RB] = pixel[0] * pixel[2];
+      varpx[9*i+VAR_GG] = pixel[1] * pixel[1];
+      varpx[9*i+VAR_GB] = pixel[1] * pixel[2];
+      varpx[9*i+VAR_BB] = pixel[2] * pixel[2];
     }
+    // apply horizontal pass of box mean filter while the cache is still hot
+    box_mean_1d_4ch(mean.width, meanpx, mean.data + 4 * j * mean.width, 4, w);
+    box_mean_1d_9ch(variance.width, varpx, variance.data + 9 * j * variance.width, 9, w);
   }
-  box_mean(cov_imgg_img_r, cov_imgg_img_r, w);
-  box_mean(cov_imgg_img_g, cov_imgg_img_g, w);
-  box_mean(cov_imgg_img_b, cov_imgg_img_b, w);
-  box_mean(var_imgg_rr, var_imgg_rr, w);
-  box_mean(var_imgg_rg, var_imgg_rg, w);
-  box_mean(var_imgg_rb, var_imgg_rb, w);
-  box_mean(var_imgg_gg, var_imgg_gg, w);
-  box_mean(var_imgg_gb, var_imgg_gb, w);
-  box_mean(var_imgg_bb, var_imgg_bb, w);
+  box_means_vert(img_bak, w, mean, variance);
+  dt_free_align(img_bak);
+  // we will recycle memory of 'mean' for the new coefficient arrays a_? and b to reduce memory foot print
+  color_image a_b = mean;
+  #define A_RED 0
+  #define A_GREEN 1
+  #define A_BLUE 2
+  #define B 3
+#ifdef _OPENMP
+#pragma omp parallel for schedule(static) default(none) \
+  dt_omp_firstprivate(size, eps) shared(mean, variance, a_b)
+#endif
   for(size_t i = 0; i < size; i++)
   {
-    cov_imgg_img_r.data[i] -= imgg_mean_r.data[i] * img_mean.data[i];
-    cov_imgg_img_g.data[i] -= imgg_mean_g.data[i] * img_mean.data[i];
-    cov_imgg_img_b.data[i] -= imgg_mean_b.data[i] * img_mean.data[i];
-    var_imgg_rr.data[i] -= imgg_mean_r.data[i] * imgg_mean_r.data[i];
-    var_imgg_rr.data[i] += eps;
-    var_imgg_rg.data[i] -= imgg_mean_r.data[i] * imgg_mean_g.data[i];
-    var_imgg_rb.data[i] -= imgg_mean_r.data[i] * imgg_mean_b.data[i];
-    var_imgg_gg.data[i] -= imgg_mean_g.data[i] * imgg_mean_g.data[i];
-    var_imgg_gg.data[i] += eps;
-    var_imgg_gb.data[i] -= imgg_mean_g.data[i] * imgg_mean_b.data[i];
-    var_imgg_bb.data[i] -= imgg_mean_b.data[i] * imgg_mean_b.data[i];
-    var_imgg_bb.data[i] += eps;
-  }
-  // we will recycle memory of the arrays imgg_mean_? and img_mean for the new coefficient arrays a_? and b to
-  // reduce memory foot print
-  gray_image a_r = imgg_mean_r;
-  gray_image a_g = imgg_mean_g;
-  gray_image a_b = imgg_mean_b;
-  gray_image b = img_mean;
-  for(int i1 = 0; i1 < height; i1++)
-  {
-    size_t i = (size_t)i1 * width;
-    for(int i0 = 0; i0 < width; i0++)
+    const float *meanpx = get_color_pixel(mean, i);
+    const float inp_mean = meanpx[INP_MEAN];
+    const float guide_r = meanpx[GUIDE_MEAN_R];
+    const float guide_g = meanpx[GUIDE_MEAN_G];
+    const float guide_b = meanpx[GUIDE_MEAN_B];
+    float *const varpx = get_color_pixel(variance, i);
+    // solve linear system of equations of size 3x3 via Cramer's rule
+    // symmetric coefficient matrix
+    const float Sigma_0_0 = varpx[VAR_RR] - (guide_r * guide_r) + eps;
+    const float Sigma_0_1 = varpx[VAR_RG] - (guide_r * guide_g);
+    const float Sigma_0_2 = varpx[VAR_RB] - (guide_r * guide_b);
+    const float Sigma_1_1 = varpx[VAR_GG] - (guide_g * guide_g) + eps;;
+    const float Sigma_1_2 = varpx[VAR_GB] - (guide_g * guide_b);
+    const float Sigma_2_2 = varpx[VAR_BB] - (guide_b * guide_b) + eps;
+    const float det0 = Sigma_0_0 * (Sigma_1_1 * Sigma_2_2 - Sigma_1_2 * Sigma_1_2)
+      - Sigma_0_1 * (Sigma_0_1 * Sigma_2_2 - Sigma_0_2 * Sigma_1_2)
+      + Sigma_0_2 * (Sigma_0_1 * Sigma_1_2 - Sigma_0_2 * Sigma_1_1);
+    float a_r_, a_g_, a_b_, b_;
+    if(fabsf(det0) > 4.f * FLT_EPSILON)
     {
-      // solve linear system of equations of size 3x3 via Cramer's rule
-      // symmetric coefficient matrix
-      const float Sigma_0_0 = var_imgg_rr.data[i];
-      const float Sigma_0_1 = var_imgg_rg.data[i];
-      const float Sigma_0_2 = var_imgg_rb.data[i];
-      const float Sigma_1_1 = var_imgg_gg.data[i];
-      const float Sigma_1_2 = var_imgg_gb.data[i];
-      const float Sigma_2_2 = var_imgg_bb.data[i];
-      const float cov_imgg_img[3] = { cov_imgg_img_r.data[i], cov_imgg_img_g.data[i], cov_imgg_img_b.data[i] };
-      const float det0 = Sigma_0_0 * (Sigma_1_1 * Sigma_2_2 - Sigma_1_2 * Sigma_1_2)
-                         - Sigma_0_1 * (Sigma_0_1 * Sigma_2_2 - Sigma_0_2 * Sigma_1_2)
-                         + Sigma_0_2 * (Sigma_0_1 * Sigma_1_2 - Sigma_0_2 * Sigma_1_1);
-      float a_r_, a_g_, a_b_;
-      if(fabsf(det0) > 4.f * FLT_EPSILON)
-      {
-        const float det1 = cov_imgg_img[0] * (Sigma_1_1 * Sigma_2_2 - Sigma_1_2 * Sigma_1_2)
-                           - Sigma_0_1 * (cov_imgg_img[1] * Sigma_2_2 - cov_imgg_img[2] * Sigma_1_2)
-                           + Sigma_0_2 * (cov_imgg_img[1] * Sigma_1_2 - cov_imgg_img[2] * Sigma_1_1);
-        const float det2 = Sigma_0_0 * (cov_imgg_img[1] * Sigma_2_2 - cov_imgg_img[2] * Sigma_1_2)
-                           - cov_imgg_img[0] * (Sigma_0_1 * Sigma_2_2 - Sigma_0_2 * Sigma_1_2)
-                           + Sigma_0_2 * (Sigma_0_1 * cov_imgg_img[2] - Sigma_0_2 * cov_imgg_img[1]);
-        const float det3 = Sigma_0_0 * (Sigma_1_1 * cov_imgg_img[2] - Sigma_1_2 * cov_imgg_img[1])
-                           - Sigma_0_1 * (Sigma_0_1 * cov_imgg_img[2] - Sigma_0_2 * cov_imgg_img[1])
-                           + cov_imgg_img[0] * (Sigma_0_1 * Sigma_1_2 - Sigma_0_2 * Sigma_1_1);
-        a_r_ = det1 / det0;
-        a_g_ = det2 / det0;
-        a_b_ = det3 / det0;
-      }
-      else
-      {
-        // linear system is singular
-        a_r_ = 0.f;
-        a_g_ = 0.f;
-        a_b_ = 0.f;
-      }
-      b.data[i] -= a_r_ * imgg_mean_r.data[i];
-      b.data[i] -= a_g_ * imgg_mean_g.data[i];
-      b.data[i] -= a_b_ * imgg_mean_b.data[i];
-      // now data of imgg_mean_? is no longer needed, we can safely overwrite aliasing arrays
-      a_r.data[i] = a_r_;
-      a_g.data[i] = a_g_;
-      a_b.data[i] = a_b_;
-      ++i;
+      const float cov_r = varpx[COV_R] - guide_r * inp_mean;
+      const float cov_g = varpx[COV_G] - guide_g * inp_mean;
+      const float cov_b = varpx[COV_B] - guide_b * inp_mean;
+      const float det1 = cov_r * (Sigma_1_1 * Sigma_2_2 - Sigma_1_2 * Sigma_1_2)
+        - Sigma_0_1 * (cov_g * Sigma_2_2 - cov_b * Sigma_1_2)
+        + Sigma_0_2 * (cov_g * Sigma_1_2 - cov_b * Sigma_1_1);
+      const float det2 = Sigma_0_0 * (cov_g * Sigma_2_2 - cov_b * Sigma_1_2)
+        - cov_r * (Sigma_0_1 * Sigma_2_2 - Sigma_0_2 * Sigma_1_2)
+        + Sigma_0_2 * (Sigma_0_1 * cov_b - Sigma_0_2 * cov_g);
+      const float det3 = Sigma_0_0 * (Sigma_1_1 * cov_b - Sigma_1_2 * cov_g)
+        - Sigma_0_1 * (Sigma_0_1 * cov_b - Sigma_0_2 * cov_g)
+        + cov_r * (Sigma_0_1 * Sigma_1_2 - Sigma_0_2 * Sigma_1_1);
+      a_r_ = det1 / det0;
+      a_g_ = det2 / det0;
+      a_b_ = det3 / det0;
+      b_ = inp_mean - a_r_ * guide_r - a_g_ * guide_g - a_b_ * guide_b;
     }
+    else
+    {
+      // linear system is singular
+      a_r_ = 0.f;
+      a_g_ = 0.f;
+      a_b_ = 0.f;
+      b_ = get_color_pixel(mean, i)[INP_MEAN];
+    }
+    // now data of imgg_mean_? is no longer needed, we can safely overwrite aliasing arrays
+    a_b.data[4*i+A_RED] = a_r_;
+    a_b.data[4*i+A_GREEN] = a_g_;
+    a_b.data[4*i+A_BLUE] = a_b_;
+    a_b.data[4*i+B] = b_;
   }
-  box_mean(a_r, a_r, w);
-  box_mean(a_g, a_g, w);
-  box_mean(a_b, a_b, w);
-  box_mean(b, b, w);
+  free_color_image(&variance);
+  box_mean_4ch(a_b, w);
+#ifdef _OPENMP
+#pragma omp parallel for schedule(static) default(none) \
+  shared(target, imgg, a_b, img_out) dt_omp_sharedconst(source) dt_omp_firstprivate(min, max, width, guide_weight)
+#endif
   for(int j_imgg = target.lower; j_imgg < target.upper; j_imgg++)
   {
     // index of the left most target pixel in the current row
@@ -331,30 +432,65 @@ static void guided_filter_tiling(color_image imgg, gray_image img, gray_image im
     size_t k = (target.left - source.left) + (size_t)(j_imgg - source.lower) * width;
     for(int i_imgg = target.left; i_imgg < target.right; i_imgg++, k++, l++)
     {
-      float *pixel = get_color_pixel(imgg, l);
-      float res = a_r.data[k] * pixel[0] + a_g.data[k] * pixel[1] + a_b.data[k] * pixel[2];
-      res *= guide_weight;
-      res += b.data[k];
-      if(res < min) res = min;
-      if(res > max) res = max;
-      img_out.data[i_imgg + (size_t)j_imgg * imgg.width] = res;
+      const float *pixel = get_color_pixel(imgg, l);
+      const float *px_ab = get_color_pixel(a_b, k);
+      float res = guide_weight * (px_ab[A_RED] * pixel[0] + px_ab[A_GREEN] * pixel[1] + px_ab[A_BLUE] * pixel[2]);
+      res += px_ab[B];
+      img_out.data[i_imgg + (size_t)j_imgg * imgg.width] = CLAMP(res, min, max);
     }
   }
-  free_gray_image(&var_imgg_rr);
-  free_gray_image(&var_imgg_rg);
-  free_gray_image(&var_imgg_rb);
-  free_gray_image(&var_imgg_gg);
-  free_gray_image(&var_imgg_gb);
-  free_gray_image(&var_imgg_bb);
-  free_gray_image(&cov_imgg_img_r);
-  free_gray_image(&cov_imgg_img_g);
-  free_gray_image(&cov_imgg_img_b);
-  free_gray_image(&img_mean);
-  free_gray_image(&imgg_mean_r);
-  free_gray_image(&imgg_mean_g);
-  free_gray_image(&imgg_mean_b);
+  free_color_image(&mean);
 }
 
+static int compute_tile_height(const int height, const int w)
+{
+  int tile_h = max_i(3 * w, GF_TILE_SIZE);
+#if 0 // enabling the below doesn't make any measureable speed difference, but does cause a handfull of pixels
+      // to round off differently (as does changing GF_TILE_SIZE)
+  if ((height % tile_h) > 0 && (height % tile_h) < GF_TILE_SIZE/3)
+  {
+    // if there's just a sliver left over for the last row of tiles, see whether slicing off a few pixels
+    // gives us a mostly-full tile
+    if (height % (tile_h - 8) >= GF_TILE_SIZE/3)
+      tile_h -= 8;
+    else  if (height % (tile_h - w/4) >= GF_TILE_SIZE/3)
+      tile_h -= (w/4);
+    else  if (height % (tile_h - w/2) >= GF_TILE_SIZE/3)
+      tile_h -= (w/2);
+    // try adding a few pixels
+    else if (height % (tile_h + 8) >= GF_TILE_SIZE/3)
+      tile_h += 8;
+    else if (height % (tile_h + 16) >= GF_TILE_SIZE/3)
+      tile_h += 16;
+  }
+#endif
+  return tile_h;
+}
+
+static int compute_tile_width(const int width, const int w)
+{
+  int tile_w = max_i(3 * w, GF_TILE_SIZE);
+#if 0 // enabling the below doesn't make any measureable speed difference, but does cause a handfull of pixels
+      // to round off differently (as does changing GF_TILE_SIZE)
+  if ((width % tile_w) > 0 && (width % tile_w) < GF_TILE_SIZE/2)
+  {
+    // if there's just a sliver left over for the last column of tiles, see whether slicing off a few pixels
+    // gives us a mostly-full tile
+    if (width % (tile_w - 8) >= GF_TILE_SIZE/3)
+      tile_w -= 8;
+    else  if (width % (tile_w - w/4) >= GF_TILE_SIZE/3)
+      tile_w -= (w/4);
+    else  if (width % (tile_w - w/2) >= GF_TILE_SIZE/3)
+      tile_w -= (w/2);
+    // try adding a few pixels
+    else if (width % (tile_w + 8) >= GF_TILE_SIZE/3)
+      tile_w += 8;
+    else if (width % (tile_w + 16) >= GF_TILE_SIZE/3)
+      tile_w += 16;
+  }
+#endif
+  return tile_w;
+}
 
 void guided_filter(const float *const guide, const float *const in, float *const out, const int width,
                    const int height, const int ch,
@@ -369,17 +505,15 @@ void guided_filter(const float *const guide, const float *const in, float *const
   color_image img_guide = (color_image){ (float *)guide, width, height, ch };
   gray_image img_in = (gray_image){ (float *)in, width, height };
   gray_image img_out = (gray_image){ out, width, height };
-  const int tile_width = max_i(3 * w, 512);
+  const int tile_width = compute_tile_width(width,w);
+  const int tile_height = compute_tile_height(height,w);
   const float eps = sqrt_eps * sqrt_eps; // this is the regularization parameter of the original papers
 
-#ifdef _OPENMP
-#pragma omp parallel for collapse(2)
-#endif
-  for(int j = 0; j < height; j += tile_width)
+  for(int j = 0; j < height; j += tile_height)
   {
     for(int i = 0; i < width; i += tile_width)
     {
-      tile target = { i, min_i(i + tile_width, width), j, min_i(j + tile_width, height) };
+      tile target = { i, min_i(i + tile_width, width), j, min_i(j + tile_height, height) };
       guided_filter_tiling(img_guide, img_in, img_out, target, w, eps, guide_weight, min, max);
     }
   }
