@@ -81,6 +81,12 @@ typedef enum
   DT_IMAGE_HAS_USERCROP = 65536,
   // image is an sraw
   DT_IMAGE_S_RAW = 1 << 17,
+  // image has a monochrome preview tested
+  DT_IMAGE_MONOCHROME_PREVIEW = 1 << 18,
+  // image has been set to monochrome via demosaic module
+  DT_IMAGE_MONOCHROME_BAYER = 1 << 19,
+  // image has a flag set to use the monochrome workflow in the modules supporting it
+  DT_IMAGE_MONOCHROME_WORKFLOW = 1 << 20,
 } dt_image_flags_t;
 
 typedef enum dt_image_colorspace_t
@@ -232,15 +238,21 @@ int dt_image_is_ldr(const dt_image_t *img);
 int dt_image_is_raw(const dt_image_t *img);
 /** returns non-zero if the image contains float data. */
 int dt_image_is_hdr(const dt_image_t *img);
+/** set the monochrome flags if monochrome is TRUE and clear it otherwise */
+void dt_image_set_monochrome_flag(const int32_t imgid, gboolean monochrome);
 /** returns non-zero if this image was taken using a monochrome camera */
 int dt_image_is_monochrome(const dt_image_t *img);
 /** returns non-zero if the image supports a color correction matrix */
 int dt_image_is_matrix_correction_supported(const dt_image_t *img);
 /** returns non-zero if the image supports the rawprepare module */
 int dt_image_is_rawprepare_supported(const dt_image_t *img);
+/** returns the bitmask containing info about monochrome images */
+int dt_image_monochrome_flags(const dt_image_t *img);
+/** returns true if the image has been tested to be monochrome and the image wants monochrome workflow */
+gboolean dt_image_use_monochrome_workflow(const dt_image_t *img);
 /** returns the full path name where the image was imported from. from_cache=TRUE check and return local
  * cached filename if any. */
-void dt_image_full_path(const int imgid, char *pathname, size_t pathname_len, gboolean *from_cache);
+void dt_image_full_path(const int32_t imgid, char *pathname, size_t pathname_len, gboolean *from_cache);
 /** returns the full directory of the associated film roll. */
 void dt_image_film_roll_directory(const dt_image_t *img, char *pathname, size_t pathname_len);
 /** returns the portion of the path used for the film roll name. */
@@ -250,7 +262,7 @@ void dt_image_film_roll(const dt_image_t *img, char *pathname, size_t pathname_l
 /** appends version numbering for duplicated images without querying the db. */
 void dt_image_path_append_version_no_db(int version, char *pathname, size_t pathname_len);
 /** appends version numbering for duplicated images. */
-void dt_image_path_append_version(int imgid, char *pathname, size_t pathname_len);
+void dt_image_path_append_version(const int32_t imgid, char *pathname, size_t pathname_len);
 /** prints a one-line exif information string. */
 void dt_image_print_exif(const dt_image_t *img, char *line, size_t line_len);
 /* set rating to img flags */
@@ -275,7 +287,7 @@ int32_t dt_image_duplicate(const int32_t imgid);
 /** flips the image, clock wise, if given flag. */
 void dt_image_flip(const int32_t imgid, const int32_t cw);
 void dt_image_set_flip(const int32_t imgid, const dt_image_orientation_t user_flip);
-dt_image_orientation_t dt_image_get_orientation(const int imgid);
+dt_image_orientation_t dt_image_get_orientation(const int32_t imgid);
 /** get max width and height of the final processed image with its current hisotry stack */
 gboolean dt_image_get_final_size(const int32_t imgid, int *width, int *height);
 void dt_image_reset_final_size(const int32_t imgid);
@@ -288,9 +300,9 @@ void dt_image_set_locations(const GList *img, const dt_image_geoloc_t *geoloc,
 /** get image location lon/lat/ele */
 void dt_image_get_location(const int32_t imgid, dt_image_geoloc_t *geoloc);
 /** returns TRUE if current hash is not basic nor auto_apply, FALSE otherwise. */
-gboolean dt_image_altered(const uint32_t imgid);
+gboolean dt_image_altered(const int32_t imgid);
 /** returns TRUE if if current has is basic, FALSE otherwise. */
-gboolean dt_image_basic(const uint32_t imgid);
+gboolean dt_image_basic(const int32_t imgid);
 /** set the image final/cropped aspect ratio */
 double dt_image_set_aspect_ratio(const int32_t imgid, gboolean raise);
 /** set the image raw aspect ratio */
@@ -355,13 +367,13 @@ gboolean dt_image_safe_remove(const int32_t imgid);
 /* try to sync .xmp for all local copies */
 void dt_image_local_copy_synch(void);
 // xmp functions:
-void dt_image_write_sidecar_file(int imgid);
+void dt_image_write_sidecar_file(const int32_t imgid);
 void dt_image_synch_xmp(const int selected);
 void dt_image_synch_xmps(const GList *img);
 void dt_image_synch_all_xmp(const gchar *pathname);
 
 // add an offset to the exif_datetime_taken field
-void dt_image_add_time_offset(const int imgid, const long int offset);
+void dt_image_add_time_offset(const int32_t imgid, const long int offset);
 
 /** helper function to get the audio file filename that is accompanying the image. g_free() after use */
 char *dt_image_get_audio_path(const int32_t imgid);
