@@ -80,7 +80,14 @@ dt_imageio_retval_t dt_imageio_open_gm(dt_image_t *img, const char *filename, dt
     goto error;
   }
 
-  fprintf(stderr, "[GraphicsMagick_open] image `%s' loading\n", img->filename);
+  dt_print(DT_DEBUG_IMAGEIO, "[GraphicsMagick_open] image `%s' loading\n", img->filename);
+
+  if(IsCMYKColorspace(image->colorspace))
+  {
+    fprintf(stderr, "[GraphicsMagick_open] error: CMYK images are not supported.\n");
+    err =  DT_IMAGEIO_FILE_CORRUPTED;
+    goto error;
+  }
 
   width = image->columns;
   height = image->rows;
