@@ -187,7 +187,7 @@ void dt_import_session_import(struct dt_import_session_t *self)
   int id = dt_image_import(self->film->id, self->current_filename, TRUE);
   if(id)
   {
-    dt_control_signal_raise(darktable.signals, DT_SIGNAL_VIEWMANAGER_THUMBTABLE_ACTIVATE, id);
+    DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_VIEWMANAGER_THUMBTABLE_ACTIVATE, id);
     dt_control_queue_redraw();
   }
 }
