@@ -20,6 +20,10 @@
 #include <glib.h>
 #include <string.h>
 
+#ifdef DT_HAVE_SIGNAL_TRACE
+#include <execinfo.h>
+#endif
+
 typedef struct dt_control_signal_t
 {
   /* the sinks for the signals */
@@ -52,6 +56,7 @@ static GType image_export_arg[]
     = { G_TYPE_UINT, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_POINTER, G_TYPE_POINTER, G_TYPE_POINTER };
 static GType history_will_change_arg[]
 = { G_TYPE_POINTER, G_TYPE_UINT, G_TYPE_POINTER };
+static GType geotag_arg[] = { G_TYPE_POINTER, G_TYPE_UINT };
 
 // callback for the destructor of DT_SIGNAL_COLLECTION_CHANGED
 static void _collection_changed_destroy_callback(gpointer instance, int query_change, gpointer imgs,
@@ -74,6 +79,16 @@ static void _image_info_changed_destroy_callback(gpointer instance, gpointer img
   }
 }
 
+// callback for the destructor of DT_SIGNAL_GEOTAG_CHANGED
+static void _image_geotag_destroy_callback(gpointer instance, gpointer imgs, const int locid, gpointer user_data)
+{
+  if(imgs && g_list_length(imgs) > 0)
+  {
+    g_list_free(imgs);
+    imgs = NULL;
+  }
+}
+
 static dt_signal_description _signal_description[DT_SIGNAL_COUNT] = {
   /* Global signals */
   { "dt-global-mouse-over-image-change", NULL, NULL, G_TYPE_NONE, g_cclosure_marshal_VOID__VOID, 0, NULL, NULL,
@@ -97,6 +112,8 @@ static dt_signal_description _signal_description[DT_SIGNAL_COUNT] = {
     FALSE }, // DT_SIGNAL_SELECTION_CHANGED
   { "dt-tag-changed", NULL, NULL, G_TYPE_NONE, g_cclosure_marshal_VOID__VOID, 0, NULL, NULL,
     FALSE }, // DT_SIGNAL_TAG_CHANGED
+  { "dt-geotag-changed", NULL, NULL, G_TYPE_NONE,  g_cclosure_marshal_generic, 2, geotag_arg,
+    G_CALLBACK(_image_geotag_destroy_callback), FALSE }, // DT_SIGNAL_GEOTAG_CHANGED
   { "dt-metadata-changed", NULL, NULL, G_TYPE_NONE, g_cclosure_marshal_VOID__UINT, 1, uint_arg, NULL,
     FALSE }, // DT_SIGNAL_METADATA_CHANGED
   { "dt-image-info-changed", NULL, NULL, G_TYPE_NONE, g_cclosure_marshal_generic, 1, pointer_arg,
@@ -240,6 +257,27 @@ gboolean _async_com_callback(gpointer data)
   return FALSE;
 }
 
+static void _print_trace (const char* op)
+{
+#ifdef DT_HAVE_SIGNAL_TRACE
+  if(darktable.unmuted_signal_dbg_acts & DT_DEBUG_SIGNAL_ACT_PRINT_TRACE)
+  {
+    void *array[10];
+    size_t size;
+    char **strings;
+    size_t i;
+
+    size = backtrace (array, 10);
+    strings = backtrace_symbols (array, size);
+
+    for (i = 0; i < size; i++)
+      dt_print(DT_DEBUG_SIGNAL, "[signal-trace-%s]: %s\n", op, strings[i]);
+
+    free (strings);
+  }
+#endif
+}
+
 void dt_control_signal_raise(const dt_control_signal_t *ctlsig, dt_signal_t signal, ...)
 {
   // ignore all signals on shutdown
@@ -257,6 +295,12 @@ void dt_control_signal_raise(const dt_control_signal_t *ctlsig, dt_signal_t sign
     return;
   }
 
+  if(darktable.unmuted_signal_dbg_acts & DT_DEBUG_SIGNAL_ACT_RAISE && darktable.unmuted_signal_dbg[signal])
+  {
+    dt_print(DT_DEBUG_SIGNAL, "[signal] raised: %s\n", signal_description->name);
+    _print_trace("raise");
+  }
+
   // 0th element has to be the instance to call
   g_value_init(instance_and_params, _signal_type);
   g_value_set_object(instance_and_params, ctlsig->sink);
@@ -322,15 +366,24 @@ void dt_control_signal_raise(const dt_control_signal_t *ctlsig, dt_signal_t sign
   }
 }
 
-
 void dt_control_signal_connect(const dt_control_signal_t *ctlsig, dt_signal_t signal, GCallback cb,
                                gpointer user_data)
 {
+  if(darktable.unmuted_signal_dbg_acts & DT_DEBUG_SIGNAL_ACT_CONNECT && darktable.unmuted_signal_dbg[signal])
+  {
+    dt_print(DT_DEBUG_SIGNAL, "[signal] connected: %s\n", _signal_description[signal].name);
+    _print_trace("connect");
+  }
   g_signal_connect(G_OBJECT(ctlsig->sink), _signal_description[signal].name, G_CALLBACK(cb), user_data);
 }
 
 void dt_control_signal_disconnect(const struct dt_control_signal_t *ctlsig, GCallback cb, gpointer user_data)
 {
+  if(darktable.unmuted_signal_dbg_acts & DT_DEBUG_SIGNAL_ACT_DISCONNECT)
+  {
+    dt_print(DT_DEBUG_SIGNAL, "[signal] disconnected\n");
+    _print_trace("disconnect");
+  }
   g_signal_handlers_disconnect_matched(G_OBJECT(ctlsig->sink), G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, 0,
                                        0, NULL, cb, user_data);
 }
