@@ -80,6 +80,12 @@ typedef enum dt_signal_t
   /** \brief This signal is raised when a tag is added/deleted/changed  */
   DT_SIGNAL_TAG_CHANGED,
 
+  /** \brief This signal is raised when a geotag is added/deleted/changed  */
+  // when imgs <> NULL these images have some geotag changes
+  // when imgs == NULL locations have changed
+  // if locid <> 0 it the new selected location on map
+  DT_SIGNAL_GEOTAG_CHANGED,
+
   /** \brief This signal is raised when metadata status (shown/hidden) or value has changed */
   DT_SIGNAL_METADATA_CHANGED,
 
@@ -239,6 +245,15 @@ typedef enum dt_signal_t
   DT_SIGNAL_COUNT
 } dt_signal_t;
 
+typedef enum dt_debug_signal_action_t
+{
+  // powers of two, masking
+  DT_DEBUG_SIGNAL_ACT_RAISE       = 1 << 0,
+  DT_DEBUG_SIGNAL_ACT_CONNECT     = 1 << 1,
+  DT_DEBUG_SIGNAL_ACT_DISCONNECT  = 1 << 2,
+  DT_DEBUG_SIGNAL_ACT_PRINT_TRACE = 1 << 3,
+} dt_debug_signal_action_t;
+
 /* inititialize the signal framework */
 struct dt_control_signal_t *dt_control_signal_init();
 /* raises a signal */
@@ -253,6 +268,37 @@ void dt_control_signal_block_by_func(const struct dt_control_signal_t *ctlsig, G
 /* unblocks a callback */
 void dt_control_signal_unblock_by_func(const struct dt_control_signal_t *ctlsig, GCallback cb, gpointer user_data);
 
+#define DT_DEBUG_CONTROL_SIGNAL_RAISE(ctlsig, signal, ...)                                                                       \
+  do                                                                                                                             \
+  {                                                                                                                              \
+    if((darktable.unmuted_signal_dbg_acts & DT_DEBUG_SIGNAL_ACT_RAISE) && darktable.unmuted_signal_dbg[signal])                 \
+    {                                                                                                                            \
+      dt_print(DT_DEBUG_SIGNAL, "[signal] %s:%d, function %s(): raise signal %s\n", __FILE__, __LINE__, __FUNCTION__, #signal);  \
+    }                                                                                                                            \
+    dt_control_signal_raise(ctlsig, signal, ##__VA_ARGS__);                                                                      \
+  } while (0)
+
+#define DT_DEBUG_CONTROL_SIGNAL_CONNECT(ctlsig, signal, cb, user_data)                                                           \
+  do                                                                                                                             \
+  {                                                                                                                              \
+    if((darktable.unmuted_signal_dbg_acts & DT_DEBUG_SIGNAL_ACT_CONNECT) && darktable.unmuted_signal_dbg[signal])                \
+    {                                                                                                                            \
+      dt_print(DT_DEBUG_SIGNAL, "[signal] %s:%d, function: %s() connect handler %s to signal %s\n", __FILE__, __LINE__,          \
+               __FUNCTION__, #cb, #signal);                                                                                      \
+    }                                                                                                                            \
+    dt_control_signal_connect(ctlsig, signal, cb, user_data);                                                                    \
+  } while (0)
+
+#define DT_DEBUG_CONTROL_SIGNAL_DISCONNECT(ctlsig, cb, user_data)                                                                \
+  do                                                                                                                             \
+  {                                                                                                                              \
+    if(darktable.unmuted_signal_dbg_acts & DT_DEBUG_SIGNAL_ACT_DISCONNECT)                                                       \
+    {                                                                                                                            \
+      dt_print(DT_DEBUG_SIGNAL, "[signal] %s:%d, function: %s() disconnect handler %s\n", __FILE__, __LINE__, __FUNCTION__, #cb);\
+    }                                                                                                                            \
+    dt_control_signal_disconnect(ctlsig, cb, user_data);                                                                         \
+  } while (0)
+
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
