@@ -24,7 +24,6 @@
 struct dt_dev_pixelpipe_iop_t;
 struct dt_dev_pixelpipe_t;
 struct dt_iop_module_t;
-struct dt_iop_order_iccprofile_info_t;
 
 typedef enum dt_iop_buffer_type_t {
   TYPE_UNKNOWN,
@@ -57,11 +56,10 @@ typedef struct dt_iop_buffer_dsc_t
 
   /** sensor saturation, propagated through the operations */
   float processed_maximum[4];
-  
+
   /** colorspace of the image */
   int cst;
-  /** work profile info of the image */
-  struct dt_iop_order_iccprofile_info_t *work_profile_info;
+
 } dt_iop_buffer_dsc_t;
 
 size_t dt_iop_buffer_dsc_to_bpp(const struct dt_iop_buffer_dsc_t *dsc);
