@@ -236,143 +236,6 @@ static inline int fcol(const int row, const int col, const uint32_t filters, con
     return FC(row, col, filters);
 }
 
-
-#ifdef _OPENMP
-#pragma omp declare simd
-#endif
-static inline uint64_t splitmix64(const uint64_t seed)
-{
-  // fast random number generator
-  // reference : http://prng.di.unimi.it/splitmix64.c
-  uint64_t result = (seed ^ (seed >> 30)) * 0xBF58476D1CE4E5B9;
-  result = (result ^ (result >> 27)) * 0x94D049BB133111EB;
-  return result ^ (result >> 31);
-}
-
-
-#ifdef _OPENMP
-#pragma omp declare simd aligned(state:64)
-#endif
-static inline void xoshiro256_init(uint64_t seed, uint64_t state[4])
-{
-  // Init the xoshiro256 random generator
-  uint64_t tmp = splitmix64(seed);
-  state[0] = (uint32_t)tmp;
-  state[1] = (uint32_t)(tmp >> 32);
-
-  tmp = splitmix64(seed + 0x9E3779B97f4A7C15);
-  state[2] = (uint32_t)tmp;
-  state[3] = (uint32_t)(tmp >> 32);
-}
-
-
-#ifdef _OPENMP
-#pragma omp declare simd uniform(k)
-#endif
-static inline uint64_t rol64(const uint64_t x, const uint64_t k)
-{
-  return (x << k) | (x >> (64 - k));
-}
-
-
-#ifdef _OPENMP
-#pragma omp declare simd aligned(state:64)
-#endif
-static inline float xoshiro256ss(uint64_t state[4])
-{
-  // fast random number generator
-  // reference : http://prng.di.unimi.it/
-  const uint64_t result = rol64(state[1] * 5, 7) * 9;
-  const uint64_t t = state[1] << 17;
-
-  state[2] ^= state[0];
-  state[3] ^= state[1];
-  state[1] ^= state[2];
-  state[0] ^= state[3];
-
-  state[2] ^= t;
-  state[3] = rol64(state[3], 45);
-
-  return (float)result / (float)UINT64_MAX;
-}
-
-
-typedef enum dt_noise_distribution_t
-{
-  DT_NOISE_UNIFORM = 0,   // $DESCRIPTION: "uniform"
-  DT_NOISE_GAUSSIAN = 1,  // $DESCRIPTION: "gaussian"
-  DT_NOISE_POISSONIAN = 2 // $DESCRIPTION: "poissonian"
-} dt_noise_distribution_t;
-
-
-#ifdef _OPENMP
-#pragma omp declare simd uniform(sigma)
-#endif
-static inline float gaussian_noise(const float mu, const float sigma, const int flip, uint64_t state[4])
-{
-  // Create gaussian noise centered in mu of standard deviation sigma
-  // state should be initialized with xoshiro256_init() before calling and private in thread
-  // flip needs to be flipped every next iteration
-  // reference : https://en.wikipedia.org/wiki/Box%E2%80%93Muller_transform
-
-  const float u1 = fmaxf(xoshiro256ss(state), FLT_MIN);
-  const float u2 = xoshiro256ss(state);
-
-  float noise;
-
-  if(flip)
-    noise = sqrtf(-2.0f * logf(u1)) * cosf(2.f * M_PI * u2);
-  else
-    noise = sqrtf(-2.0f * logf(u1)) * sinf(2.f * M_PI * u2);
-
-  return noise * sigma + mu;
-}
-
-
-#ifdef _OPENMP
-#pragma omp declare simd uniform(sigma)
-#endif
-static inline float poisson_noise(const float mu, const float sigma, const int flip, uint64_t state[4])
-{
-  // create poisson noise - It's just gaussian noise with Anscombe transform applied
-  const float u1 = fmaxf(xoshiro256ss(state), FLT_MIN);
-  const float u2 = xoshiro256ss(state);
-
-  float noise;
-
-  if(flip)
-    noise = sqrtf(-2.0f * logf(u1)) * cosf(2.f * M_PI * u2);
-  else
-    noise = sqrtf(-2.0f * logf(u1)) * sinf(2.f * M_PI * u2);
-
-  const float r = noise * sigma + 2.0f * sqrtf(fmaxf(mu + 3.f / 8.f, 0.0f));
-  return (r * r - sigma * sigma) / 4.f - 3.f / 8.f;
-}
-
-
-#ifdef _OPENMP
-#pragma omp declare simd uniform(distribution, param)
-#endif
-static inline float dt_noise_generator(const dt_noise_distribution_t distribution,
-                                       const float mu, const float param, const int flip, uint64_t state[4])
-{
-  // param needs to be squared for uniform and poissonian distributions in order to have roughly the
-  // same effect as in gaussian.
-
-  switch(distribution)
-  {
-    case(DT_NOISE_UNIFORM):
-    default:
-      return mu + 2.0f * (xoshiro256ss(state) - 0.5f) * param;
-
-    case(DT_NOISE_GAUSSIAN):
-      return gaussian_noise(mu, param, flip, state);
-
-    case(DT_NOISE_POISSONIAN):
-      return poisson_noise(mu, param, flip, state);
-  }
-}
-
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
