@@ -41,6 +41,7 @@ int dt_dev_pixelpipe_cache_init(dt_dev_pixelpipe_cache_t *cache, int entries, si
 #ifdef _DEBUG
   memset(cache->dsc, 0x2c, sizeof(dt_iop_buffer_dsc_t) * entries);
 #endif
+  cache->basichash = (uint64_t *)calloc(entries, sizeof(uint64_t));
   cache->hash = (uint64_t *)calloc(entries, sizeof(uint64_t));
   cache->used = (int32_t *)calloc(entries, sizeof(int32_t));
   for(int k = 0; k < entries; k++)
@@ -56,6 +57,7 @@ int dt_dev_pixelpipe_cache_init(dt_dev_pixelpipe_cache_t *cache, int entries, si
       ASAN_POISON_MEMORY_REGION(cache->data[k], cache->size[k]);
     }
     else cache->data[k] = 0;
+    cache->basichash[k] = -1;
     cache->hash[k] = -1;
     cache->used[k] = 0;
   }
@@ -82,12 +84,13 @@ void dt_dev_pixelpipe_cache_cleanup(dt_dev_pixelpipe_cache_t *cache)
   for(int k = 0; k < cache->entries; k++) dt_free_align(cache->data[k]);
   free(cache->data);
   free(cache->dsc);
+  free(cache->basichash);
   free(cache->hash);
   free(cache->used);
   free(cache->size);
 }
 
-uint64_t dt_dev_pixelpipe_cache_hash(int imgid, const dt_iop_roi_t *roi, dt_dev_pixelpipe_t *pipe, int module)
+uint64_t dt_dev_pixelpipe_cache_basichash(int imgid, struct dt_dev_pixelpipe_t *pipe, int module)
 {
   // bernstein hash (djb2)
   // the hash is made of imgid and the actual fast-pipe mode if activated
@@ -117,9 +120,47 @@ uint64_t dt_dev_pixelpipe_cache_hash(int imgid, const dt_iop_roi_t *roi, dt_dev_
     }
     pieces = g_list_next(pieces);
   }
+  return hash;
+}
+
+uint64_t dt_dev_pixelpipe_cache_basichash_prior(int imgid, struct dt_dev_pixelpipe_t *pipe,
+                                                const dt_iop_module_t *const module)
+{
+  ;
+  // find the last enabled module prior to the specified one, then get its hash
+  GList *pieces = pipe->nodes;
+  GList *modules = pipe->iop;
+  int last = -1;
+  for(int k = 1; modules && pieces; k++)
+  {
+    dt_dev_pixelpipe_iop_t *piece = (dt_dev_pixelpipe_iop_t *)pieces->data;
+    if (module == (dt_iop_module_t *)modules->data)
+      break;		// we've found the given module, so 'last' now contains the index of the prior active module
+    dt_develop_t *dev = piece->module->dev;
+    if (piece->enabled &&
+        !(dev->gui_module && (dev->gui_module->operation_tags_filter() & piece->module->operation_tags())))
+      last = k;
+    pieces = g_list_next(pieces);
+    modules = g_list_next(modules);
+  }
+  return last>=0 ? dt_dev_pixelpipe_cache_basichash(imgid, pipe, last) : -1;
+}
+
+void dt_dev_pixelpipe_cache_fullhash(int imgid, const dt_iop_roi_t *roi, struct dt_dev_pixelpipe_t *pipe, int module,
+                                     uint64_t *basichash, uint64_t *fullhash)
+{
+  uint64_t hash = *basichash = dt_dev_pixelpipe_cache_basichash(imgid, pipe, module);
   // also add scale, x and y:
   const char *str = (const char *)roi;
-  for(size_t i = 0; i < sizeof(dt_iop_roi_t); i++) hash = ((hash << 5) + hash) ^ str[i];
+  for(size_t i = 0; i < sizeof(dt_iop_roi_t); i++)
+    hash = ((hash << 5) + hash) ^ str[i];
+  *fullhash = hash;
+}
+
+uint64_t dt_dev_pixelpipe_cache_hash(int imgid, const dt_iop_roi_t *roi, dt_dev_pixelpipe_t *pipe, int module)
+{
+  uint64_t basichash, hash;
+  dt_dev_pixelpipe_cache_fullhash(imgid, roi, pipe, module, &basichash, &hash);
   return hash;
 }
 
@@ -131,20 +172,21 @@ int dt_dev_pixelpipe_cache_available(dt_dev_pixelpipe_cache_t *cache, const uint
   return 0;
 }
 
-int dt_dev_pixelpipe_cache_get_important(dt_dev_pixelpipe_cache_t *cache, const uint64_t hash, const size_t size,
+int dt_dev_pixelpipe_cache_get_important(dt_dev_pixelpipe_cache_t *cache, const uint64_t basichash,
+                                         const uint64_t hash, const size_t size,
                                          void **data, dt_iop_buffer_dsc_t **dsc)
 {
-  return dt_dev_pixelpipe_cache_get_weighted(cache, hash, size, data, dsc, -cache->entries);
+  return dt_dev_pixelpipe_cache_get_weighted(cache, basichash, hash, size, data, dsc, -cache->entries);
 }
 
-int dt_dev_pixelpipe_cache_get(dt_dev_pixelpipe_cache_t *cache, const uint64_t hash, const size_t size,
-                               void **data, dt_iop_buffer_dsc_t **dsc)
+int dt_dev_pixelpipe_cache_get(dt_dev_pixelpipe_cache_t *cache, const uint64_t basichash, const uint64_t hash,
+                               const size_t size, void **data, dt_iop_buffer_dsc_t **dsc)
 {
-  return dt_dev_pixelpipe_cache_get_weighted(cache, hash, size, data, dsc, 0);
+  return dt_dev_pixelpipe_cache_get_weighted(cache, basichash, hash, size, data, dsc, 0);
 }
 
-int dt_dev_pixelpipe_cache_get_weighted(dt_dev_pixelpipe_cache_t *cache, const uint64_t hash, const size_t size,
-                                        void **data, dt_iop_buffer_dsc_t **dsc, int weight)
+int dt_dev_pixelpipe_cache_get_weighted(dt_dev_pixelpipe_cache_t *cache, const uint64_t basichash, const uint64_t hash,
+                                        const size_t size, void **data, dt_iop_buffer_dsc_t **dsc, int weight)
 {
   cache->queries++;
   *data = NULL;
@@ -192,6 +234,7 @@ int dt_dev_pixelpipe_cache_get_weighted(dt_dev_pixelpipe_cache_t *cache, const u
     cache->dsc[max] = **dsc;
     *dsc = &cache->dsc[max];
 
+    cache->basichash[max] = basichash;
     cache->hash[max] = hash;
     cache->used[max] = weight;
     cache->misses++;
@@ -205,6 +248,20 @@ void dt_dev_pixelpipe_cache_flush(dt_dev_pixelpipe_cache_t *cache)
 {
   for(int k = 0; k < cache->entries; k++)
   {
+    cache->basichash[k] = -1;
+    cache->hash[k] = -1;
+    cache->used[k] = 0;
+    ASAN_POISON_MEMORY_REGION(cache->data[k], cache->size[k]);
+  }
+}
+
+void dt_dev_pixelpipe_cache_flush_all_but(dt_dev_pixelpipe_cache_t *cache, uint64_t basichash)
+{
+  for(int k = 0; k < cache->entries; k++)
+  {
+    if (cache->basichash[k] == basichash)
+      continue;
+    cache->basichash[k] = -1;
     cache->hash[k] = -1;
     cache->used[k] = 0;
     ASAN_POISON_MEMORY_REGION(cache->data[k], cache->size[k]);
@@ -228,6 +285,7 @@ void dt_dev_pixelpipe_cache_invalidate(dt_dev_pixelpipe_cache_t *cache, void *da
   {
     if(cache->data[k] == data)
     {
+      cache->basichash[k] = -1;
       cache->hash[k] = -1;
       ASAN_POISON_MEMORY_REGION(cache->data[k], cache->size[k]);
     }
@@ -239,7 +297,7 @@ void dt_dev_pixelpipe_cache_print(dt_dev_pixelpipe_cache_t *cache)
   for(int k = 0; k < cache->entries; k++)
   {
     printf("pixelpipe cacheline %d ", k);
-    printf("used %d by %" PRIu64 "", cache->used[k], cache->hash[k]);
+    printf("used %d by %" PRIu64 " (%" PRIu64 ")", cache->used[k], cache->hash[k], cache->basichash[k]);
     printf("\n");
   }
   printf("cache hit rate so far: %.3f\n", (cache->queries - cache->misses) / (float)cache->queries);
