@@ -213,6 +213,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Fujifilm X-H1", { 11434,-4948,-1210,-3746,12042,1903,-666,1479,5235 } },
     { "Fujifilm X-M1", { 10413,-3996,-993,-3721,11640,2361,-733,1540,6011 } },
     { "Fujifilm X-S1", { 13509,-6199,-1254,-4430,12733,1865,-331,1441,5022 } },
+    { "Fujifilm X-S10", { 13426,-6334,-1177,-4244,12136,2371,-580,1303,5980 } },
     { "Fujifilm X-T10", { 8458,-2451,-855,-4597,12447,2407,-1475,2482,6526 } },
     { "Fujifilm X-T20", { 10592,-4262,-1008,-3514,11355,2465,-870,2025,6386 } },
     { "Fujifilm X-T30", { 13426,-6334,-1177,-4244,12136,2371,-580,1303,5980 } },
@@ -387,7 +388,8 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Nikon 1 AW1", { 6588,-1305,-693,-3277,10987,2634,-355,2016,5106 } },
     { "Nikon 1 S1", { 8994,-2667,-865,-4594,12324,2552,-699,1786,6260 } },
     { "Nikon 1 S2", { 6612,-1342,-618,-3338,11055,2623,-174,1792,5075 } },
-    { "Nikon Z 6", { 8210,-2534,-683,-5355,13338,2212,-1143,1929,6464 } },
+    { "Nikon Z 5", { 8695,-2558,-648,-5015,12711,2575,-1279,2215,7514 } },
+    { "Nikon Z 6", { 9943,-3269,-839,-5323,13269,2259,-1198,2083,7557 } },
     { "Nikon Z 7", { 10405,-3755,-1270,-5461,13787,1793,-1040,2015,6785 } },
     { "Nikon Z 50", { 11640,-4829,-1079,-5107,13006,2325,-972,1711,7380 } },
     { "Olympus C5050Z", { 10508,-3124,-1273,-6079,14294,1901,-1653,2306,6237 } },
@@ -431,6 +433,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Olympus E-M10", { 8380,-2630,-639,-2887,10725,2496,-627,1427,5438 } },
     { "Olympus E-M10 Mark II", { 8380,-2630,-639,-2887,10725,2496,-627,1427,5438 } },
     { "Olympus E-M10 Mark III", { 8380,-2630,-639,-2887,10725,2496,-627,1427,5438 } },
+    { "Olympus E-M10 Mark IV", { 9476,-3182,-765,-2613,10958,1893,-449,1315,5268 } },
     { "Olympus E-M1", { 7687,-1984,-606,-4327,11928,2721,-1381,2339,6452 } },
     { "Olympus E-M1X", { 11896,-5110,-1076,-3181,11378,2048,-519,1224,5166 } },
     { "Olympus E-M1MarkII", { 9383,-3170,-763,-2457,10702,2020,-384,1236,5552 } },
@@ -568,6 +571,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Panasonic DC-GH5S", { 6929,-2355,-708,-4192,12534,1828,-1097,1989,5195 } },
     { "Panasonic DC-LX100M2", { 8585,-3127,-833,-4005,12250,1953,-650,1494,4862 } },
     { "Panasonic DC-S1", { 9744,-3905,-779,-4899,12807,2324,-798,1630,5827 } },
+    { "Panasonic DC-S5", { 9744,-3905,-779,-4899,12807,2324,-798,1630,5827 } },
     { "Panasonic DC-S1H", { 9397,-3719,-805,-5425,13326,2309,-972,1715,6034 } },
     { "Panasonic DC-S1R", { 11822,-5321,-1249,-5958,15114,766,-614,1264,7043 } },
     { "Panasonic DC-G9", { 7685,-2375,-634,-3687,11700,2249,-748,1546,5111 } },
@@ -668,6 +672,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Sony ILCE-7R", { 4913,-541,-202,-6130,13513,2906,-1564,2151,7183 } },
     { "Sony ILCE-7M2", { 5271,-712,-347,-6153,13653,2763,-1601,2366,7242 } },
     { "Sony ILCE-7M3", { 7374,-2389,-551,-5435,13162,2519,-1006,1795,6552 } },
+    { "Sony ILCE-7C", { 7374,-2389,-551,-5435,13162,2519,-1006,1795,6552 } },
     { "Sony ILCE-7", { 5271,-712,-347,-6153,13653,2763,-1601,2366,7242 } },
     { "Sony ILCE-3000", { 5991,-1456,-455,-4764,12135,2980,-707,1425,6701 } },
     { "Sony ILCE-3500", { 5991,-1456,-455,-4764,12135,2980,-707,1425,6701 } },
@@ -702,6 +707,7 @@ static void dt_dcraw_adobe_coeff(const char *name, float cam_xyz[1][12])
     { "Sony SLT-A65", { 5491,-1192,-363,-4951,12342,2948,-911,1722,7192 } },
     { "Sony SLT-A77", { 5491,-1192,-363,-4951,12342,2948,-911,1722,7192 } },
     { "Sony SLT-A99", { 6344,-1612,-462,-4863,12477,2681,-865,1786,6899 } },
+    { "Sony ZV-1", { 8280,-2987,-703,-3531,11645,2133,-550,1542,5312 } },
   };
 
   for (size_t i = 0; i < sizeof(table)/sizeof(table_data); i++) {
