@@ -18,9 +18,14 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
+#pragma once
+
 #include "common/RawImage.h"           // for RawImage
-#include "io/ByteStream.h"             // for ByteStream
 #include "metadata/ColorFilterArray.h" // for ColorFilterArray
 
+namespace rawspeed {
+class ByteStream;
+} // namespace rawspeed
+
 rawspeed::RawImage CreateRawImage(rawspeed::ByteStream* bs);
 rawspeed::ColorFilterArray CreateCFA(rawspeed::ByteStream* bs);
