@@ -19,12 +19,13 @@
 */
 
 #include "common/RawImage.h"              // for RawImageDataFloat, TYPE_FL...
-#include "common/Common.h"                // for uint8_t, uint32_t, writeLog
+#include "common/Common.h"                // for writeLog, DEBUG_PRIO_INFO
 #include "common/Point.h"                 // for iPoint2D
 #include "decoders/RawDecoderException.h" // for ThrowRDE
 #include "metadata/BlackArea.h"           // for BlackArea
 #include <algorithm>                      // for max, min
 #include <array>                          // for array
+#include <cstdint>                        // for uint8_t, uint32_t, uint16_t
 #include <memory>                         // for operator==, unique_ptr
 #include <vector>                         // for vector
 
@@ -34,12 +35,12 @@ using std::max;
 namespace rawspeed {
 
 RawImageDataFloat::RawImageDataFloat() {
-  bpp = 4;
+  bpp = sizeof(float);
   dataType = TYPE_FLOAT32;
   }
 
   RawImageDataFloat::RawImageDataFloat(const iPoint2D& _dim, uint32_t _cpp)
-      : RawImageData(_dim, 4, _cpp) {
+      : RawImageData(_dim, sizeof(float), _cpp) {
     dataType = TYPE_FLOAT32;
   }
 
@@ -295,48 +296,50 @@ void RawImageDataFloat::fixBadPixel(uint32_t x, uint32_t y, int component) {
   std::array<float, 4> weight;
 
   uint8_t* bad_line = &mBadPixelMap[y * mBadPixelMapPitch];
+  // We can have cfa or no-cfa for RawImageDataFloat
+  int step = isCFA ? 2 : 1;
 
   // Find pixel to the left
-  int x_find = static_cast<int>(x) - 2;
+  int x_find = static_cast<int>(x) - step;
   int curr = 0;
   while (x_find >= 0 && values[curr] < 0) {
     if (0 == ((bad_line[x_find>>3] >> (x_find&7)) & 1)) {
-      values[curr] = (reinterpret_cast<float*>(getData(x_find, y)))[component];
+      values[curr] = (reinterpret_cast<float*>(getDataUncropped(x_find, y)))[component];
       dist[curr] = static_cast<float>(static_cast<int>(x) - x_find);
     }
-    x_find-=2;
+    x_find -= step;
   }
   // Find pixel to the right
-  x_find = static_cast<int>(x) + 2;
+  x_find = static_cast<int>(x) + step;
   curr = 1;
   while (x_find < uncropped_dim.x && values[curr] < 0) {
     if (0 == ((bad_line[x_find>>3] >> (x_find&7)) & 1)) {
-      values[curr] = (reinterpret_cast<float*>(getData(x_find, y)))[component];
+      values[curr] = (reinterpret_cast<float*>(getDataUncropped(x_find, y)))[component];
       dist[curr] = static_cast<float>(x_find - static_cast<int>(x));
     }
-    x_find+=2;
+    x_find += step;
   }
 
   bad_line = &mBadPixelMap[x>>3];
   // Find pixel upwards
-  int y_find = static_cast<int>(y) - 2;
+  int y_find = static_cast<int>(y) - step;
   curr = 2;
   while (y_find >= 0 && values[curr] < 0) {
     if (0 == ((bad_line[y_find*mBadPixelMapPitch] >> (x&7)) & 1)) {
-      values[curr] = (reinterpret_cast<float*>(getData(x, y_find)))[component];
+      values[curr] = (reinterpret_cast<float*>(getDataUncropped(x, y_find)))[component];
       dist[curr] = static_cast<float>(static_cast<int>(y) - y_find);
     }
-    y_find-=2;
+    y_find -= step;
   }
   // Find pixel downwards
-  y_find = static_cast<int>(y) + 2;
+  y_find = static_cast<int>(y) + step;
   curr = 3;
   while (y_find < uncropped_dim.y && values[curr] < 0) {
     if (0 == ((bad_line[y_find*mBadPixelMapPitch] >> (x&7)) & 1)) {
-      values[curr] = (reinterpret_cast<float*>(getData(x, y_find)))[component];
+      values[curr] = (reinterpret_cast<float*>(getDataUncropped(x, y_find)))[component];
       dist[curr] = static_cast<float>(y_find - static_cast<int>(y));
     }
-    y_find+=2;
+    y_find += step;
   }
   // Find x weights
   float total_dist_x = dist[0] + dist[1];
@@ -360,7 +363,7 @@ void RawImageDataFloat::fixBadPixel(uint32_t x, uint32_t y, int component) {
   float total_pixel = 0;
   for (int i = 0; i < 4; i++)
     if (values[i] >= 0)
-      total_pixel += values[i] * dist[i];
+      total_pixel += values[i] * weight[i];
 
   total_pixel /= total_div;
   auto* pix = reinterpret_cast<float*>(getDataUncropped(x, y));
