@@ -21,15 +21,15 @@
 
 #pragma once
 
-#include "common/Common.h" // for uint16_t
-#include "common/Point.h"  // for iPoint2D
-#include <algorithm>       // for max, min, adjacent_find, for_each
-#include <cassert>         // for assert
-#include <functional>      // for greater_equal
-#include <limits>          // for numeric_limits
-#include <memory>          // for allocator_traits<>::value_type
-#include <type_traits>     // for is_floating_point, enable_if_t, is_arithm...
-#include <vector>          // for vector
+#include "common/Point.h" // for iPoint2D
+#include <algorithm>      // for max, min, adjacent_find, for_each
+#include <cassert>        // for assert
+#include <cstdint>        // for uint16_t
+#include <functional>     // for greater_equal
+#include <limits>         // for numeric_limits
+#include <memory>         // for allocator_traits<>::value_type
+#include <type_traits>    // for is_floating_point, enable_if_t, is_arithmetic
+#include <vector>         // for vector
 
 namespace rawspeed {
 
