@@ -20,9 +20,11 @@
 */
 
 #include "decoders/MefDecoder.h"
-#include "decoders/RawDecoderException.h" // for RawDecoderException (ptr o...
+#include "decoders/RawDecoderException.h"           // for ThrowRDE
 #include "decompressors/UncompressedDecompressor.h" // for UncompressedDeco...
-#include "io/Endianness.h"                          // for Endianness::big
+#include "io/Buffer.h"                              // for Buffer, DataBuffer
+#include "io/ByteStream.h"                          // for ByteStream
+#include "io/Endianness.h"                          // for Endianness, Endi...
 #include <string>                                   // for operator==, string
 
 namespace rawspeed {
