@@ -20,13 +20,14 @@
 */
 
 #include "decoders/NefDecoder.h"
-#include "common/Common.h"                          // for uint32_t, uint8_t
+#include "common/Array2DRef.h"                      // for Array2DRef
+#include "common/Common.h"                          // for clampBits, round...
 #include "common/Point.h"                           // for iPoint2D
 #include "decoders/RawDecoderException.h"           // for ThrowRDE
 #include "decompressors/NikonDecompressor.h"        // for NikonDecompressor
 #include "decompressors/UncompressedDecompressor.h" // for UncompressedDeco...
 #include "io/BitPumpMSB.h"                          // for BitPumpMSB
-#include "io/Buffer.h"                              // for Buffer
+#include "io/Buffer.h"                              // for Buffer, DataBuffer
 #include "io/ByteStream.h"                          // for ByteStream
 #include "io/Endianness.h"                          // for getU16BE, Endian...
 #include "io/IOException.h"                         // for ThrowIOE
@@ -63,7 +64,7 @@ bool NefDecoder::isAppropriateDecoder(const TiffRootIFD* rootIFD,
 }
 
 RawImage NefDecoder::decodeRawInternal() {
-  auto raw = mRootIFD->getIFDWithTag(CFAPATTERN);
+  const auto* raw = mRootIFD->getIFDWithTag(CFAPATTERN);
   auto compression = raw->getEntry(COMPRESSION)->getU32();
 
   TiffEntry *offsets = raw->getEntry(STRIPOFFSETS);
@@ -203,7 +204,7 @@ bool NefDecoder::NEFIsUncompressedRGB(const TiffIFD* raw) {
 }
 
 void NefDecoder::DecodeUncompressed() {
-  auto raw = getIFDWithLargestImage(CFAPATTERN);
+  const auto* raw = getIFDWithLargestImage(CFAPATTERN);
   TiffEntry *offsets = raw->getEntry(STRIPOFFSETS);
   TiffEntry *counts = raw->getEntry(STRIPBYTECOUNTS);
   uint32_t yPerSlice = raw->getEntry(ROWSPERSTRIP)->getU32();
@@ -342,7 +343,7 @@ void NefDecoder::readCoolpixSplitRaw(ByteStream input, const iPoint2D& size,
 }
 
 void NefDecoder::DecodeD100Uncompressed() {
-  auto ifd = mRootIFD->getIFDWithTag(STRIPOFFSETS, 1);
+  const auto* ifd = mRootIFD->getIFDWithTag(STRIPOFFSETS, 1);
 
   uint32_t offset = ifd->getEntry(STRIPOFFSETS)->getU32();
   // Hardcode the sizes as at least the width is not correctly reported
@@ -364,7 +365,7 @@ void NefDecoder::DecodeD100Uncompressed() {
 }
 
 void NefDecoder::DecodeSNefUncompressed() {
-  auto raw = getIFDWithLargestImage(CFAPATTERN);
+  const auto* raw = getIFDWithLargestImage(CFAPATTERN);
   uint32_t offset = raw->getEntry(STRIPOFFSETS)->getU32();
   uint32_t width = raw->getEntry(IMAGEWIDTH)->getU32();
   uint32_t height = raw->getEntry(IMAGELENGTH)->getU32();
@@ -395,7 +396,7 @@ void NefDecoder::checkSupportInternal(const CameraMetaData* meta) {
 
 string NefDecoder::getMode() {
   ostringstream mode;
-  auto raw = getIFDWithLargestImage(CFAPATTERN);
+  const auto* raw = getIFDWithLargestImage(CFAPATTERN);
   int compression = raw->getEntry(COMPRESSION)->getU32();
   uint32_t bitPerPixel = raw->getEntry(BITSPERSAMPLE)->getU32();
 
@@ -413,7 +414,7 @@ string NefDecoder::getMode() {
 string NefDecoder::getExtendedMode(const string &mode) {
   ostringstream extended_mode;
 
-  auto ifd = mRootIFD->getIFDWithTag(CFAPATTERN);
+  const auto* ifd = mRootIFD->getIFDWithTag(CFAPATTERN);
   uint32_t width = ifd->getEntry(IMAGEWIDTH)->getU32();
   uint32_t height = ifd->getEntry(IMAGELENGTH)->getU32();
 
