@@ -20,7 +20,7 @@
 */
 
 #include "decoders/RafDecoder.h"
-#include "common/Common.h"                          // for uint32, ushort16
+#include "common/Common.h"                          // for BitOrder_LSB
 #include "common/Point.h"                           // for iPoint2D, iRecta...
 #include "decoders/RawDecoderException.h"           // for ThrowRDE
 #include "decompressors/FujiDecompressor.h"         // for FujiDecompressor
@@ -38,7 +38,7 @@
 #include "tiff/TiffTag.h"                           // for FUJI_RAWIMAGEFUL...
 #include <array>                                    // for array
 #include <cassert>                                  // for assert
-#include <cstdio>                                   // for size_t
+#include <cstdint>                                  // for uint32_t, uint16_t
 #include <cstring>                                  // for memcmp
 #include <memory>                                   // for unique_ptr
 #include <string>                                   // for string, operator==
@@ -65,9 +65,9 @@ bool RafDecoder::isAppropriateDecoder(const TiffRootIFD* rootIFD,
 }
 
 RawImage RafDecoder::decodeRawInternal() {
-  auto raw = mRootIFD->getIFDWithTag(FUJI_STRIPOFFSETS);
-  uint32 height = 0;
-  uint32 width = 0;
+  const auto* raw = mRootIFD->getIFDWithTag(FUJI_STRIPOFFSETS);
+  uint32_t height = 0;
+  uint32_t width = 0;
 
   if (raw->hasEntry(FUJI_RAWIMAGEFULLHEIGHT)) {
     height = raw->getEntry(FUJI_RAWIMAGEFULLHEIGHT)->getU32();
@@ -79,7 +79,7 @@ RawImage RafDecoder::decodeRawInternal() {
   } else
     ThrowRDE("Unable to locate image size");
 
-  if (width == 0 || height == 0 || width > 9216 || height > 6210)
+  if (width == 0 || height == 0 || width > 11808 || height > 8754)
     ThrowRDE("Unexpected image dimensions found: (%u; %u)", width, height);
 
   if (raw->hasEntry(FUJI_LAYOUT)) {
@@ -146,7 +146,7 @@ RawImage RafDecoder::decodeRawInternal() {
   }
 
   double_width = hints.has("double_width_unpacked");
-  const uint32 real_width = double_width ? 2U * width : width;
+  const uint32_t real_width = double_width ? 2U * width : width;
 
   mRaw->dim = iPoint2D(real_width, height);
   mRaw->createData();
@@ -225,8 +225,8 @@ void RafDecoder::decodeMetaDataInternal(const CameraMetaData* meta) {
   // Rotate 45 degrees - could be multithreaded.
   if (rotate && !this->uncorrectedRawValues) {
     // Calculate the 45 degree rotated size;
-    uint32 rotatedsize;
-    uint32 rotationPos;
+    uint32_t rotatedsize;
+    uint32_t rotationPos;
     if (alt_layout) {
       rotatedsize = new_size.y+new_size.x/2;
       rotationPos = new_size.x/2 - 1;
@@ -243,10 +243,10 @@ void RafDecoder::decodeMetaDataInternal(const CameraMetaData* meta) {
     rotated->metadata.fujiRotationPos = rotationPos;
 
     int dest_pitch = static_cast<int>(rotated->pitch) / 2;
-    auto* dst = reinterpret_cast<ushort16*>(rotated->getData(0, 0));
+    auto* dst = reinterpret_cast<uint16_t*>(rotated->getData(0, 0));
 
     for (int y = 0; y < new_size.y; y++) {
-      auto* src = reinterpret_cast<ushort16*>(
+      auto* src = reinterpret_cast<uint16_t*>(
           mRaw->getData(crop_offset.x, crop_offset.y + y));
       for (int x = 0; x < new_size.x; x++) {
         int h;
@@ -322,9 +322,9 @@ void RafDecoder::decodeMetaDataInternal(const CameraMetaData* meta) {
 }
 
 int RafDecoder::isCompressed() {
-  auto raw = mRootIFD->getIFDWithTag(FUJI_STRIPOFFSETS);
-  uint32 height = 0;
-  uint32 width = 0;
+  const auto* raw = mRootIFD->getIFDWithTag(FUJI_STRIPOFFSETS);
+  uint32_t height = 0;
+  uint32_t width = 0;
 
   if (raw->hasEntry(FUJI_RAWIMAGEFULLHEIGHT)) {
     height = raw->getEntry(FUJI_RAWIMAGEFULLHEIGHT)->getU32();
@@ -336,10 +336,10 @@ int RafDecoder::isCompressed() {
   } else
     ThrowRDE("Unable to locate image size");
 
-  if (width == 0 || height == 0 || width > 9216 || height > 6210)
+  if (width == 0 || height == 0 || width > 11808 || height > 8754)
     ThrowRDE("Unexpected image dimensions found: (%u; %u)", width, height);
 
-  uint32 count = raw->getEntry(FUJI_STRIPBYTECOUNTS)->getU32();
+  uint32_t count = raw->getEntry(FUJI_STRIPBYTECOUNTS)->getU32();
 
   // The uncompressed raf's can be 12/14 bpp, so if it is less than that,
   // then we are likely in compressed raf.
