@@ -22,9 +22,9 @@
 
 #pragma once
 
-#include "common/Common.h"                // for uint32_t
 #include "decoders/AbstractTiffDecoder.h" // for AbstractTiffDecoder
 #include "tiff/TiffIFD.h"                 // for TiffIFD (ptr only), TiffRo...
+#include <cstdint>                        // for uint32_t
 #include <utility>                        // for move
 
 namespace rawspeed {
