@@ -21,19 +21,18 @@
 
 #pragma once
 
-#include "common/Common.h"                      // for uint32_t
+#include "common/Common.h"                      // for roundUpDivision
 #include "common/Point.h"                       // for iPoint2D
 #include "common/RawImage.h"                    // for RawImage
 #include "decompressors/AbstractDecompressor.h" // for AbstractDecompressor
 #include "io/ByteStream.h"                      // for ByteStream
 #include <cassert>                              // for assert
+#include <cstdint>                              // for uint32_t
 #include <utility>                              // for move
 #include <vector>                               // for vector
 
 namespace rawspeed {
 
-class RawImage;
-
 struct DngTilingDescription final {
   // The dimensions of the whole image.
   const iPoint2D& dim;
