@@ -22,13 +22,14 @@
 
 #ifdef HAVE_ZLIB
 
-#include "common/Common.h"                // for uint32_t, uint16_t
+#include "common/Point.h"                 // for iPoint2D
 #include "decoders/RawDecoderException.h" // for ThrowRDE
 #include "decompressors/DeflateDecompressor.h"
-#include "io/Endianness.h" // for getHostEndianness, Endiann...
+#include "io/Endianness.h" // for getHostEndianness, Endianness
 #include <cassert>         // for assert
+#include <cstdint>         // for uint32_t, uint16_t
 #include <cstdio>          // for size_t
-#include <zlib.h>
+#include <zlib.h>          // for uncompress, zError, Z_OK
 
 namespace rawspeed {
 
@@ -218,14 +219,14 @@ void DeflateDecompressor::decode(
     predFactor = 0;
     break;
   }
+  predFactor *= mRaw->getCpp();
 
   int bytesps = bps / 8;
 
   for (auto row = 0; row < dim.y; ++row) {
     unsigned char* src = uBuffer->get() + row * maxDim.x * bytesps;
-    unsigned char* dst =
-        static_cast<unsigned char*>(mRaw->getData()) +
-        ((off.y + row) * mRaw->pitch + off.x * sizeof(float) * mRaw->getCpp());
+    unsigned char* dst = static_cast<unsigned char*>(mRaw->getData()) +
+                         ((off.y + row) * mRaw->pitch + off.x * sizeof(float));
 
     if (predFactor)
       decodeFPDeltaRow(src, dst, dim.x, maxDim.x, bytesps, predFactor);
