@@ -19,7 +19,8 @@
 */
 
 #include "decompressors/PentaxDecompressor.h"
-#include "common/Common.h"                // for uint32_t, uint8_t, uint16_t
+#include "common/Array2DRef.h"            // for Array2DRef
+#include "common/Common.h"                // for extractHighBits, isIntN
 #include "common/Point.h"                 // for iPoint2D
 #include "common/RawImage.h"              // for RawImage, RawImageData
 #include "decoders/RawDecoderException.h" // for ThrowRDE
@@ -28,6 +29,7 @@
 #include "io/Buffer.h"                    // for Buffer
 #include "io/ByteStream.h"                // for ByteStream
 #include <cassert>                        // for assert
+#include <cstdint>                        // for uint8_t, uint32_t, uint16_t
 #include <vector>                         // for vector
 
 namespace rawspeed {
@@ -44,7 +46,7 @@ PentaxDecompressor::PentaxDecompressor(const RawImage& img,
                                        ByteStream* metaData)
     : mRaw(img), ht(SetupHuffmanTable(metaData)) {
   if (mRaw->getCpp() != 1 || mRaw->getDataType() != TYPE_USHORT16 ||
-      mRaw->getBpp() != 2)
+      mRaw->getBpp() != sizeof(uint16_t))
     ThrowRDE("Unexpected component count / data type");
 
   if (!mRaw->dim.x || !mRaw->dim.y || mRaw->dim.x % 2 != 0 ||
@@ -91,7 +93,7 @@ HuffmanTable PentaxDecompressor::SetupHuffmanTable_Modern(ByteStream stream) {
   std::array<uint32_t, 16> v2;
   /* Calculate codes and store bitcounts */
   for (uint32_t c = 0; c < depth; c++) {
-    v2[c] = v0[c] >> (12 - v1[c]);
+    v2[c] = extractHighBits(v0[c], v1[c], /*effectiveBitwidth=*/12);
     nCodesPerLength.at(v1[c])++;
   }
 
@@ -150,7 +152,7 @@ void PentaxDecompressor::decompress(const ByteStream& data) const {
       pred = {out(row - 2, 0), out(row - 2, 1)};
 
     for (int col = 0; col < out.width; col++) {
-      pred[col & 1] += ht.decodeNext(bs);
+      pred[col & 1] += ht.decodeDifference(bs);
       int value = pred[col & 1];
       if (!isIntN(value, 16))
         ThrowRDE("decoded value out of bounds at %d:%d", col, row);
