@@ -20,17 +20,21 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#include "rawspeedconfig.h"
+#include "rawspeedconfig.h" // for HAVE_OPENMP
 #include "decompressors/PhaseOneDecompressor.h"
-#include "common/Common.h"                // for int32_t, uint32_t, uint16_t
+#include "common/Array2DRef.h"            // for Array2DRef
+#include "common/Common.h"                // for rawspeed_get_number_of_pro...
 #include "common/Point.h"                 // for iPoint2D
-#include "common/RawImage.h"              // for RawImage, RawImageData
+#include "common/RawImage.h"              // for RawImageData, RawImage
+#include "common/RawspeedException.h"     // for RawspeedException
 #include "decoders/RawDecoderException.h" // for ThrowRDE
 #include "io/BitPumpMSB32.h"              // for BitPumpMSB32
-#include <algorithm>                      // for for_each
+#include <algorithm>                      // for sort
 #include <array>                          // for array
 #include <cassert>                        // for assert
-#include <cstddef>                        // for size_t
+#include <cstdint>                        // for int32_t, uint16_t
+#include <memory>                         // for allocator_traits<>::value_...
+#include <string>                         // for string
 #include <utility>                        // for move
 #include <vector>                         // for vector, vector<>::size_type
 
@@ -42,7 +46,7 @@ PhaseOneDecompressor::PhaseOneDecompressor(const RawImage& img,
   if (mRaw->getDataType() != TYPE_USHORT16)
     ThrowRDE("Unexpected data type");
 
-  if (!((mRaw->getCpp() == 1 && mRaw->getBpp() == 2)))
+  if (!((mRaw->getCpp() == 1 && mRaw->getBpp() == sizeof(uint16_t))))
     ThrowRDE("Unexpected cpp: %u", mRaw->getCpp());
 
   if (!mRaw->dim.hasPositiveArea() || mRaw->dim.x % 2 != 0 ||
