@@ -1,7 +1,7 @@
 /*
     RawSpeed - RAW file decoder.
 
-    Copyright (C) 2017 Roman Lebedev
+    Copyright (C) 2017-2019 Roman Lebedev
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -20,20 +20,22 @@
 
 #pragma once
 
-#include "decompressors/AbstractParallelizedDecompressor.h" // for Abstract...
-#include "io/ByteStream.h"                                  // for ByteStream
+#include "common/RawImage.h"                    // for RawImage
+#include "decompressors/AbstractDecompressor.h" // for AbstractDecompressor
+#include "io/ByteStream.h"                      // for ByteStream
 
 namespace rawspeed {
 
-class RawImage;
-
-class SonyArw2Decompressor final : public AbstractParallelizedDecompressor {
-  void decompressThreaded(const RawDecompressorThread* t) const final;
+class SonyArw2Decompressor final : public AbstractDecompressor {
+  void decompressRow(int row) const;
+  void decompressThread() const noexcept;
 
+  RawImage mRaw;
   ByteStream input;
 
 public:
   SonyArw2Decompressor(const RawImage& img, const ByteStream& input);
+  void decompress() const;
 };
 
 } // namespace rawspeed
