@@ -20,8 +20,9 @@
 
 #pragma once
 
-#include <array>   // for array
-#include <cstdint> // for uint16_t
+#include "common/Array2DRef.h" // for Array2DRef
+#include <array>               // for array
+#include <cstdint>             // for uint16_t
 
 namespace rawspeed {
 
@@ -29,15 +30,17 @@ class RawImage;
 
 class Cr2sRawInterpolator final {
   const RawImage& mRaw;
+
+  const Array2DRef<const uint16_t> input;
   std::array<int, 3> sraw_coeffs;
   int hue;
 
   struct YCbCr;
 
 public:
-  Cr2sRawInterpolator(const RawImage& mRaw_, std::array<int, 3> sraw_coeffs_,
-                      int hue_)
-      : mRaw(mRaw_), sraw_coeffs(sraw_coeffs_), hue(hue_) {}
+  Cr2sRawInterpolator(const RawImage& mRaw_, Array2DRef<const uint16_t> input_,
+                      std::array<int, 3> sraw_coeffs_, int hue_)
+      : mRaw(mRaw_), input(input_), sraw_coeffs(sraw_coeffs_), hue(hue_) {}
 
   void interpolate(int version);
 
@@ -46,12 +49,11 @@ protected:
 
   static inline void STORE_RGB(uint16_t* X, int r, int g, int b);
 
-  template <int version> inline void interpolate_422_row(uint16_t* data, int w);
-  template <int version> inline void interpolate_422(int w, int h);
+  template <int version> void interpolate_422_row(int row);
+  template <int version> void interpolate_422();
 
-  template <int version>
-  inline void interpolate_420_row(std::array<uint16_t*, 3> line, int w);
-  template <int version> inline void interpolate_420(int w, int h);
+  template <int version> void interpolate_420_row(int row);
+  template <int version> void interpolate_420();
 };
 
 } // namespace rawspeed
