@@ -21,9 +21,8 @@
 
 #pragma once
 
-#include "rawspeedconfig.h"
-#include "common/RawspeedException.h"
-#include <string> // for string
+#include "rawspeedconfig.h"           // for RAWSPEED_NOINLINE, RAWSPEED_UN...
+#include "common/RawspeedException.h" // for ThrowExceptionHelper, Rawspeed...
 
 namespace rawspeed {
 
