@@ -18,9 +18,10 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#include "RawSpeed-API.h"        // for RawDecoder, FileReader, RawImage
+#include "RawSpeed-API.h"        // for RawDecoder, FileReader, HAVE_OPENMP
 #include "common/ChecksumFile.h" // for ChecksumFileEntry, ReadChecksumFile
-#include <benchmark/benchmark.h> // for State, DoNotOptimize, Initialize
+#include <algorithm>             // for max
+#include <benchmark/benchmark.h> // for Counter, Counter::Flags, Counter::k...
 #include <chrono>                // for duration, high_resolution_clock
 #include <ctime>                 // for clock, clock_t
 #include <memory>                // for unique_ptr
@@ -100,7 +101,7 @@ static inline void BM_RawSpeed(benchmark::State& state, const char* fileName,
   currThreadCount = threads;
 
 #ifdef HAVE_PUGIXML
-  static const CameraMetaData metadata(CMAKE_SOURCE_DIR "/data/cameras.xml");
+  static const CameraMetaData metadata(RAWSPEED_SOURCE_DIR "/data/cameras.xml");
 #else
   static const CameraMetaData metadata{};
 #endif
@@ -134,15 +135,25 @@ static inline void BM_RawSpeed(benchmark::State& state, const char* fileName,
 
   // For each iteration:
   state.counters.insert({
-      {"CPUTime,s", CPUTime / state.iterations()},
-      {"WallTime,s", WallTime / state.iterations()},
+      {"CPUTime,s",
+       benchmark::Counter(CPUTime, benchmark::Counter::Flags::kAvgIterations)},
+      {"WallTime,s",
+       benchmark::Counter(WallTime, benchmark::Counter::Flags::kAvgIterations)},
       {"CPUTime/WallTime", CPUTime / WallTime}, // 'Threading factor'
       {"Pixels", pixels},
-      {"Pixels/CPUTime", (state.iterations() * pixels) / CPUTime},
-      {"Pixels/WallTime", (state.iterations() * pixels) / WallTime},
+      {"Pixels/CPUTime",
+       benchmark::Counter(pixels / CPUTime,
+                          benchmark::Counter::Flags::kIsIterationInvariant)},
+      {"Pixels/WallTime",
+       benchmark::Counter(pixels / WallTime,
+                          benchmark::Counter::Flags::kIsIterationInvariant)},
       /* {"Raws", 1}, */
-      {"Raws/CPUTime", state.iterations() / CPUTime},
-      {"Raws/WallTime", state.iterations() / WallTime},
+      {"Raws/CPUTime",
+       benchmark::Counter(1.0 / CPUTime,
+                          benchmark::Counter::Flags::kIsIterationInvariant)},
+      {"Raws/WallTime",
+       benchmark::Counter(1.0 / WallTime,
+                          benchmark::Counter::Flags::kIsIterationInvariant)},
   });
   // Could also have counters wrt. the filesize,
   // but i'm not sure they are interesting.
@@ -155,6 +166,7 @@ static void addBench(const char* fName, std::string tName, int threads) {
       benchmark::RegisterBenchmark(tName.c_str(), &BM_RawSpeed, fName, threads);
   b->Unit(benchmark::kMillisecond);
   b->UseRealTime();
+  b->MeasureProcessCPUTime();
 }
 
 int main(int argc, char** argv) {
