@@ -20,32 +20,30 @@
 
 #include "RawSpeed-API.h"
 
-#include "md5.h"       // for md5_state, md5_hash, hash_to_string, md5_init
-#include <array>       // for array
-#include <cassert>     // for assert
-#include <chrono>      // for milliseconds, steady_clock, duration_cast
-#include <cstdarg>     // for va_end, va_list, va_start
-#include <cstdint>     // for uint8_t
-#include <cstdio>      // for fprintf, fclose, size_t, fopen, ftell, fwrite
-#include <cstdlib>     // for system
-#include <fstream>     // IWYU pragma: keep
-#include <iostream>    // for cout, left, cerr, internal
-#include <map>         // for map
-#include <memory>      // for allocator, unique_ptr
-#include <sstream>     // IWYU pragma: keep
-#include <string>      // for string, operator+, operator<<, char_traits
-#include <utility>     // for pair
-#include <vector>      // for vector
+#include "md5.h"     // for md5_state, md5_hash, hash_to_string, md5_init
+#include <algorithm> // for fill, max
+#include <array>     // for array
+#include <cassert>   // for assert
+#include <chrono>    // for milliseconds, steady_clock, duration_cast
+#include <cstdarg>   // for va_end, va_list, va_start
+#include <cstdint>   // for uint16_t, uint32_t, uint8_t
+#include <cstdio>    // for fprintf, fclose, fopen, ftell, fwrite, size_t
+#include <cstdlib>   // for system
+#include <fstream>   // IWYU pragma: keep
+#include <iostream>  // for cout, cerr
+#include <iterator>  // for istreambuf_iterator, operator!=
+#include <map>       // for map
+#include <memory>    // for allocator, unique_ptr
+#include <sstream>   // IWYU pragma: keep
+#include <string>    // for string, operator+, operator<<, char_traits
+#include <utility>   // for pair
+#include <vector>    // for vector
 // IWYU pragma: no_include <ext/alloc_traits.h>
 
 #if !defined(__has_feature) || !__has_feature(thread_sanitizer)
 #include <iomanip> // for operator<<, setw
 #endif
 
-#ifdef HAVE_OPENMP
-#include <omp.h>
-#endif
-
 using std::chrono::steady_clock;
 using std::string;
 using std::ostringstream;
