@@ -20,8 +20,8 @@
 
 #pragma once
 
-#include "common/Common.h" // for int32_t, int16_t, uint32_t, uint64_t, uint16_t
-#include <gmock/gmock.h>   // for make_tuple, tuple
+#include <cstdint> // for uint16_t, uint32_t, uint64_t
+#include <tuple>   // for tuple
 
 namespace rawspeed_test {
 
@@ -105,7 +105,7 @@ static const uint32Type uint32Values[] = {
     {0x017c2230, 0x30227c01}, {0x03b26f3a, 0x3a6fb203},
     {0x03e67a66, 0x667ae603}, {0x073bac8d, 0x8dac3b07},
     {0x083548ff, 0xff483508}, {0x086d9c35, 0x359c6d08},
-    {0x0a0c7e14, 0x147e0c0a}, {0x0a43bd7f, 0x7fbd430a},
+    {0x0a0c7e14, 0x147e0c0a}, {0x0a43bd7e, 0x7ebd430a},
     {0x0a4aa774, 0x74a74a0a}, {0x0a4ca277, 0x77a24c0a},
     {0x0a4f5e46, 0x465e4f0a}, {0x0a56b2bd, 0xbdb2560a},
     {0x0a585294, 0x9452580a}, {0x0a7b114f, 0x4f117b0a},
