@@ -22,12 +22,13 @@
 #include "metadata/CameraSensorInfo.h" // for CameraSensorInfo
 #include <algorithm>                   // for generate
 #include <cstdlib>                     // for rand, srand
-#include <gmock/gmock.h>               // for get, InitGoogleMock, tuple
-#include <gtest/gtest.h> // for IsNullLiteralHelper, AssertionResult
-#include <iostream>      // for operator<<, basic_ostream::operator<<
-#include <limits>        // for numeric_limits
-#include <memory>        // for unique_ptr
-#include <vector>        // for vector, allocator
+#include <gmock/gmock.h>               // for InitGoogleMock
+#include <gtest/gtest.h>               // for ParamIteratorInterface, Message
+#include <iostream>                    // for operator<<, basic_ostream::op...
+#include <limits>                      // for numeric_limits
+#include <memory>                      // for unique_ptr
+#include <tuple>                       // for get, tuple
+#include <vector>                      // for vector
 
 using rawspeed::CameraSensorInfo;
 using std::unique_ptr;
@@ -37,7 +38,7 @@ namespace rawspeed_test {
 std::vector<int> ISOList(6);
 
 class CameraSensorInfoTestDumb
-    : public ::testing::TestWithParam<std::tr1::tuple<int, int>> {
+    : public ::testing::TestWithParam<std::tuple<int, int>> {
 protected:
   CameraSensorInfoTestDumb()
       : mBlackLevel(std::rand()), // NOLINT do not need crypto-level randomness
@@ -49,8 +50,8 @@ protected:
             std::rand()  // NOLINT do not need crypto-level randomness
         }) {}
   virtual void SetUp() override {
-    mMinIso = std::tr1::get<0>(GetParam());
-    mMaxIso = std::tr1::get<1>(GetParam());
+    mMinIso = std::get<0>(GetParam());
+    mMaxIso = std::get<1>(GetParam());
   }
 
   void checkHelper(const CameraSensorInfo& csi) {
