@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 henrik andersson.
+    Copyright (C) 2012-2020 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,19 +18,12 @@
 
 #pragma once
 
-typedef struct dt_gui_hist_dialog_t
-{
-  GList *selops;
-  GtkTreeView *items;
-  int copied_imageid;
-} dt_gui_hist_dialog_t;
-
 /** shows a dialog for creating a new style, w if not null is a widget to
     change the sensitive state depending on the dialog response.  */
-int dt_gui_hist_dialog_new(dt_gui_hist_dialog_t *d, int imgid, gboolean iscopy);
+int dt_gui_hist_dialog_new(dt_history_copy_item_t *d, int imgid, gboolean iscopy);
 
 /** must be called to initialize the structure. */
-void dt_gui_hist_dialog_init(dt_gui_hist_dialog_t *d);
+void dt_gui_hist_dialog_init(dt_history_copy_item_t *d);
 
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
