@@ -642,7 +642,7 @@ void gui_init(dt_imageio_module_format_t *self)
   const int quality_last = dt_conf_get_int("plugins/imageio/format/j2k/quality");
 
   gui->format = dt_bauhaus_combobox_new(NULL);
-  dt_bauhaus_widget_set_label(gui->format, NULL, _("format"));
+  dt_bauhaus_widget_set_label(gui->format, NULL, N_("format"));
   dt_bauhaus_combobox_add(gui->format, _("J2K"));
   dt_bauhaus_combobox_add(gui->format, _("jp2"));
   dt_bauhaus_combobox_set(gui->format, format_last);
@@ -650,14 +650,14 @@ void gui_init(dt_imageio_module_format_t *self)
   g_signal_connect(G_OBJECT(gui->format), "value-changed", G_CALLBACK(format_changed), NULL);
 
   gui->quality = dt_bauhaus_slider_new_with_range(NULL, 5, 100, 1, 95, 0);
-  dt_bauhaus_widget_set_label(gui->quality, NULL, _("quality"));
+  dt_bauhaus_widget_set_label(gui->quality, NULL, N_("quality"));
   dt_bauhaus_slider_set_default(gui->quality, 95);
   if(quality_last > 0 && quality_last <= 100) dt_bauhaus_slider_set(gui->quality, quality_last);
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(gui->quality), TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(gui->quality), "value-changed", G_CALLBACK(quality_changed), NULL);
 
   gui->preset = dt_bauhaus_combobox_new(NULL);
-  dt_bauhaus_widget_set_label(gui->preset, NULL, _("DCP mode"));
+  dt_bauhaus_widget_set_label(gui->preset, NULL, N_("DCP mode"));
   dt_bauhaus_combobox_add(gui->preset, _("off"));
   dt_bauhaus_combobox_add(gui->preset, _("Cinema2K, 24FPS"));
   dt_bauhaus_combobox_add(gui->preset, _("Cinema2K, 48FPS"));
