@@ -745,7 +745,7 @@ void gui_init(dt_imageio_module_storage_t *self)
 
   // account
   ui->account_list = dt_bauhaus_combobox_new(NULL);
-  dt_bauhaus_widget_set_label(ui->account_list, NULL, _("accounts"));
+  dt_bauhaus_widget_set_label(ui->account_list, NULL, N_("accounts"));
   GList *a = ui->accounts;
   int account_index = -1, index=0;
   while(a)
@@ -822,7 +822,7 @@ void gui_init(dt_imageio_module_storage_t *self)
 
   // permissions list
   ui->permission_list = dt_bauhaus_combobox_new(NULL);
-  dt_bauhaus_widget_set_label(ui->permission_list, NULL, _("visible to"));
+  dt_bauhaus_widget_set_label(ui->permission_list, NULL, N_("visible to"));
   dt_bauhaus_combobox_add(ui->permission_list, _("everyone"));
   dt_bauhaus_combobox_add(ui->permission_list, _("contacts"));
   dt_bauhaus_combobox_add(ui->permission_list, _("friends"));
@@ -835,7 +835,7 @@ void gui_init(dt_imageio_module_storage_t *self)
   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
   ui->album_list = dt_bauhaus_combobox_new(NULL); // Available albums
-  dt_bauhaus_widget_set_label(ui->album_list, NULL, _("album"));
+  dt_bauhaus_widget_set_label(ui->album_list, NULL, N_("album"));
   g_signal_connect(G_OBJECT(ui->album_list), "value-changed", G_CALLBACK(_piwigo_album_changed), (gpointer)ui);
   gtk_widget_set_sensitive(ui->album_list, FALSE);
   gtk_box_pack_start(GTK_BOX(hbox), ui->album_list, TRUE, TRUE, 0);
@@ -868,7 +868,7 @@ void gui_init(dt_imageio_module_storage_t *self)
 
   // parent album list
   ui->parent_album_list = dt_bauhaus_combobox_new(NULL); // Available albums
-  dt_bauhaus_widget_set_label(ui->parent_album_list, NULL, _("parent album"));
+  dt_bauhaus_widget_set_label(ui->parent_album_list, NULL, N_("parent album"));
   gtk_widget_set_sensitive(ui->parent_album_list, TRUE);
   gtk_box_pack_start(ui->create_box, ui->parent_album_list, TRUE, TRUE, 0);
 
