@@ -29,6 +29,7 @@
 #include "develop/imageop_gui.h"
 #include "develop/tiling.h"
 #include "gui/gtk.h"
+#include "gui/presets.h"
 #include "iop/iop_api.h"
 
 #include <gtk/gtk.h>
@@ -93,13 +94,13 @@ const char *name()
   return _("local contrast");
 }
 
-const char *description()
+const char *description(struct dt_iop_module_t *self)
 {
-  return _("manipulate local and global contrast separately,\n"
-           "for corrective and creative purposes.\n"
-           "works in Lab,\n"
-           "takes any RGB input,\n"
-           "outputs non-linear RGB.");
+  return dt_iop_set_description(self, _("manipulate local and global contrast separately"),
+                                      _("creative"),
+                                      _("non-linear, Lab, display-referred"),
+                                      _("non-linear, Lab"),
+                                      _("non-linear, Lab, display-referred"));
 }
 
 // some additional flags (self explanatory i think):
@@ -111,7 +112,7 @@ int flags()
 // where does it appear in the gui?
 int default_group()
 {
-  return IOP_GROUP_TONE;
+  return IOP_GROUP_TONE | IOP_GROUP_EFFECTS;
 }
 
 int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
@@ -119,30 +120,6 @@ int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_p
   return iop_cs_Lab;
 }
 
-void init_key_accels(dt_iop_module_so_t *self)
-{
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "detail"));
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "coarseness"));
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "contrast"));
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "highlights"));
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "shadows"));
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "midtone range"));
-  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "mode"));
-}
-
-void connect_key_accels(dt_iop_module_t *self)
-{
-  dt_iop_bilat_gui_data_t *g = (dt_iop_bilat_gui_data_t *)self->gui_data;
-
-  dt_accel_connect_slider_iop(self, "detail", GTK_WIDGET(g->detail));
-  dt_accel_connect_slider_iop(self, "coarseness", GTK_WIDGET(g->spatial));
-  dt_accel_connect_slider_iop(self, "contrast", GTK_WIDGET(g->range));
-  dt_accel_connect_slider_iop(self, "highlights", GTK_WIDGET(g->highlights));
-  dt_accel_connect_slider_iop(self, "shadows", GTK_WIDGET(g->shadows));
-  dt_accel_connect_slider_iop(self, "midtone range", GTK_WIDGET(g->midtone));
-  dt_accel_connect_combobox_iop(self, "mode", GTK_WIDGET(g->mode));
-}
-
 int legacy_params(
     dt_iop_module_t *self, const void *const old_params, const int old_version,
     void *new_params, const int new_version)
@@ -172,6 +149,31 @@ int legacy_params(
   return 1;
 }
 
+void init_presets(dt_iop_module_so_t *self)
+{
+  dt_iop_bilat_params_t p;
+  memset(&p, 0, sizeof(p));
+
+  p.mode = s_mode_local_laplacian;
+  p.sigma_r = 0.f;
+  p.sigma_s = 0.f;
+  p.detail = 0.33f;
+  p.midtone = 0.5f;
+
+  dt_gui_presets_add_generic(_("clarity"), self->op,
+                             self->version(), &p, sizeof(p), 1, DEVELOP_BLEND_CS_RGB_SCENE);
+
+  p.mode = s_mode_local_laplacian;
+  p.sigma_r = 0.f;
+  p.sigma_s = 0.f;
+  p.detail = 1.f;
+  p.midtone = 0.25f;
+
+  dt_gui_presets_add_generic(_("HDR local tone-mapping"), self->op,
+                             self->version(), &p, sizeof(p), 1, DEVELOP_BLEND_CS_RGB_SCENE);
+}
+
+
 #ifdef HAVE_OPENCL
 int process_cl(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_in, cl_mem dev_out,
                const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
@@ -285,7 +287,6 @@ void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pix
 void init_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
   piece->data = calloc(1, sizeof(dt_iop_bilat_data_t));
-  self->commit_params(self, self->default_params, pipe, piece);
 }
 
 
@@ -390,10 +391,8 @@ void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
   }
 }
 
-/** gui callbacks, these are needed. */
 void gui_update(dt_iop_module_t *self)
 {
-  // let gui slider match current parameters:
   dt_iop_bilat_gui_data_t *g = (dt_iop_bilat_gui_data_t *)self->gui_data;
   dt_iop_bilat_params_t *p = (dt_iop_bilat_params_t *)self->params;
   dt_bauhaus_slider_set(g->detail, p->detail);
@@ -422,8 +421,7 @@ void gui_update(dt_iop_module_t *self)
 void gui_init(dt_iop_module_t *self)
 {
   // init the slider (more sophisticated layouts are possible with gtk tables and boxes):
-  self->gui_data = malloc(sizeof(dt_iop_bilat_gui_data_t));
-  dt_iop_bilat_gui_data_t *g = (dt_iop_bilat_gui_data_t *)self->gui_data;
+  dt_iop_bilat_gui_data_t *g = IOP_GUI_ALLOC(bilat);
 
   g->mode = dt_bauhaus_combobox_from_params(self, N_("mode"));
   gtk_widget_set_tooltip_text(g->mode, _("the filter used for local contrast enhancement. bilateral is faster but can lead to artifacts around edges for extreme settings."));
@@ -435,28 +433,31 @@ void gui_init(dt_iop_module_t *self)
   dt_bauhaus_slider_set_format(g->detail, "%.0f%%");
   gtk_widget_set_tooltip_text(g->detail, _("changes the local contrast"));
 
+  ++darktable.bauhaus->skip_accel;
   g->spatial = dt_bauhaus_slider_from_params(self, "sigma_s");
+  g->range = dt_bauhaus_slider_from_params(self, "sigma_r");
+  g->highlights = dt_bauhaus_slider_from_params(self, "sigma_r");
+  g->shadows = dt_bauhaus_slider_from_params(self, "sigma_s");
+  --darktable.bauhaus->skip_accel;
+
   dt_bauhaus_slider_set_default(g->spatial, 50.0);
   dt_bauhaus_slider_set_digits(g->spatial, 0);
-  dt_bauhaus_widget_set_label(g->spatial, NULL, _("coarseness"));
+  dt_bauhaus_widget_set_label(g->spatial, NULL, N_("coarseness"));
   gtk_widget_set_tooltip_text(g->spatial, _("feature size of local details (spatial sigma of bilateral filter)"));
 
-  g->range = dt_bauhaus_slider_from_params(self, "sigma_r");
   dt_bauhaus_slider_set_default(g->range, 20.0);
   dt_bauhaus_slider_set_digits(g->range, 0);
-  dt_bauhaus_widget_set_label(g->range, NULL, _("contrast"));
+  dt_bauhaus_widget_set_label(g->range, NULL, N_("contrast"));
   gtk_widget_set_tooltip_text(g->range, _("L difference to detect edges (range sigma of bilateral filter)"));
 
-  g->highlights = dt_bauhaus_slider_from_params(self, "sigma_r");
   dt_bauhaus_slider_set_step(g->highlights, 0.01);
-  dt_bauhaus_widget_set_label(g->highlights, NULL, _("highlights"));
+  dt_bauhaus_widget_set_label(g->highlights, NULL, N_("highlights"));
   dt_bauhaus_slider_set_factor(g->highlights, 100);
   dt_bauhaus_slider_set_format(g->highlights, "%.0f%%");
   gtk_widget_set_tooltip_text(g->highlights, _("changes the local contrast of highlights"));
 
-  g->shadows = dt_bauhaus_slider_from_params(self, "sigma_s");
   dt_bauhaus_slider_set_step(g->shadows, 0.01);
-  dt_bauhaus_widget_set_label(g->shadows, NULL, _("shadows"));
+  dt_bauhaus_widget_set_label(g->shadows, NULL, N_("shadows"));
   dt_bauhaus_slider_set_factor(g->shadows, 100);
   dt_bauhaus_slider_set_format(g->shadows, "%.0f%%");
   gtk_widget_set_tooltip_text(g->shadows, _("changes the local contrast of shadows"));
