@@ -46,8 +46,6 @@ typedef struct dt_iop_cacorrect_gui_data_t
 {
 } dt_iop_cacorrect_gui_data_t;
 
-dt_iop_cacorrect_gui_data_t dummy;
-
 // this returns a translatable name
 const char *name()
 {
@@ -55,9 +53,19 @@ const char *name()
   return _("chromatic aberrations");
 }
 
+const char *description(struct dt_iop_module_t *self)
+{
+  return dt_iop_set_description(self, _("correct chromatic aberrations for Bayer sensors"),
+                                      _("corrective"),
+                                      _("linear, raw, scene-referred"),
+                                      _("linear, raw"),
+                                      _("linear, raw, scene-referred"));
+}
+
+
 int default_group()
 {
-  return IOP_GROUP_CORRECT;
+  return IOP_GROUP_CORRECT | IOP_GROUP_TECHNICAL;
 }
 
 int flags()
@@ -1495,15 +1503,27 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
 
 void reload_defaults(dt_iop_module_t *module)
 {
-  // we might be called from presets update infrastructure => there is no image
-  if(!module->dev) return;
-
   dt_image_t *img = &module->dev->image_storage;
   // can't be switched on for non-raw or x-trans images:
-  if(dt_image_is_raw(img) && (img->buf_dsc.filters != 9u) && !dt_image_is_monochrome(img))
+  if(dt_image_is_raw(img) && (img->buf_dsc.filters != 9u) &&
+     !(dt_image_monochrome_flags(img) & (DT_IMAGE_MONOCHROME | DT_IMAGE_MONOCHROME_BAYER)))
     module->hide_enable_button = 0;
   else
     module->hide_enable_button = 1;
+
+  if(module->widget)
+  {
+    if(dt_image_is_raw(&module->dev->image_storage))
+      if(module->dev->image_storage.buf_dsc.filters != 9u &&
+        !(dt_image_monochrome_flags(&module->dev->image_storage) & (DT_IMAGE_MONOCHROME | DT_IMAGE_MONOCHROME_BAYER)))
+        gtk_label_set_text(GTK_LABEL(module->widget), _("automatic chromatic aberration correction"));
+      else
+        gtk_label_set_text(GTK_LABEL(module->widget),
+                          _("automatic chromatic aberration correction\ndisabled for non-Bayer sensors"));
+    else
+      gtk_label_set_text(GTK_LABEL(module->widget),
+                        _("automatic chromatic aberration correction\nonly works for raw images."));
+  }
 }
 
 /** commit is the synch point between core and gui, so it copies params to pipe data. */
@@ -1511,7 +1531,8 @@ void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *params, dt_dev
                    dt_dev_pixelpipe_iop_t *piece)
 {
   dt_image_t *img = &pipe->image;
-  if(!dt_image_is_raw(img) || dt_image_is_monochrome(img)) piece->enabled = 0;
+  if(!dt_image_is_raw(img) || (dt_image_monochrome_flags(img) & (DT_IMAGE_MONOCHROME | DT_IMAGE_MONOCHROME_BAYER)))
+    piece->enabled = 0;
 }
 
 void init_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
@@ -1524,29 +1545,11 @@ void cleanup_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev
   piece->data = NULL;
 }
 
-void gui_update(dt_iop_module_t *self)
-{
-  if(dt_image_is_raw(&self->dev->image_storage))
-    if(self->dev->image_storage.buf_dsc.filters != 9u && !dt_image_is_monochrome(&self->dev->image_storage))
-      gtk_label_set_text(GTK_LABEL(self->widget), _("automatic chromatic aberration correction"));
-    else
-      gtk_label_set_text(GTK_LABEL(self->widget),
-                         _("automatic chromatic aberration correction\ndisabled for non-Bayer sensors"));
-  else
-    gtk_label_set_text(GTK_LABEL(self->widget),
-                       _("automatic chromatic aberration correction\nonly works for raw images."));
-}
-
 void gui_init(dt_iop_module_t *self)
 {
-  self->widget = gtk_label_new("");
-  gtk_widget_set_halign(self->widget, GTK_ALIGN_START);
-  self->gui_data = &dummy;
-}
+  IOP_GUI_ALLOC(cacorrect);
 
-void gui_cleanup(dt_iop_module_t *self)
-{
-  self->gui_data = NULL;
+  self->widget = dt_ui_label_new("");
 }
 
 /** additional, optional callbacks to capture darkroom center events. */
