@@ -66,6 +66,7 @@ typedef enum dt_iop_colormapping_flags_t
   NEUTRAL = 0,
   HAS_SOURCE = 1 << 0,
   HAS_TARGET = 1 << 1,
+  HAS_SOURCE_TARGET = HAS_SOURCE | HAS_TARGET,
   ACQUIRE = 1 << 2,
   GET_SOURCE = 1 << 3,
   GET_TARGET = 1 << 4
@@ -145,9 +146,18 @@ const char *name()
   return _("color mapping");
 }
 
+const char *description(struct dt_iop_module_t *self)
+{
+  return dt_iop_set_description(self, _("transfer a color palette and tonal repartition from one image to another"),
+                                      _("creative"),
+                                      _("linear or non-linear, Lab, display-referred"),
+                                      _("non-linear, Lab"),
+                                      _("non-linear, Lab, display-referred"));
+}
+
 int default_group()
 {
-  return IOP_GROUP_EFFECT;
+  return IOP_GROUP_EFFECT | IOP_GROUP_EFFECTS;
 }
 
 int flags()
@@ -160,29 +170,16 @@ int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_p
   return iop_cs_Lab;
 }
 
-
-void init_key_accels(dt_iop_module_so_t *self)
+//TODO: refactor by moving into a common header and consolidating with copy in color_picker.c
+static void *dt_alloc_perthread(const size_t n, const size_t objsize, size_t* padded_size)
 {
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "number of clusters"));
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "color dominance"));
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "histogram equalization"));
-
-  dt_accel_register_iop(self, FALSE, NC_("accel", "acquire as source"), 0, 0);
-  dt_accel_register_iop(self, FALSE, NC_("accel", "acquire as target"), 0, 0);
+  const size_t alloc_size = n * objsize;
+  const size_t cache_lines = (alloc_size+63)/64;
+  *padded_size = 64 * cache_lines / objsize;
+  return dt_alloc_align(64, 64 * cache_lines * dt_get_num_threads());
 }
-
-void connect_key_accels(dt_iop_module_t *self)
-{
-  dt_iop_colormapping_gui_data_t *g = (dt_iop_colormapping_gui_data_t *)self->gui_data;
-
-  dt_accel_connect_slider_iop(self, "number of clusters", GTK_WIDGET(g->clusters));
-  dt_accel_connect_slider_iop(self, "color dominance", GTK_WIDGET(g->dominance));
-  dt_accel_connect_slider_iop(self, "histogram equalization", GTK_WIDGET(g->equalization));
-
-  dt_accel_connect_button_iop(self, "acquire as source", g->acquire_source_button);
-  dt_accel_connect_button_iop(self, "acquire as target", g->acquire_target_button);
-}
-
+#define dt_get_perthread(buf, padsize) ((buf) + ((padsize) * dt_get_thread_num()))
+#define dt_get_bythread(buf, padsize, tnum) ((buf) + ((padsize) * (tnum)))
 
 static void capture_histogram(const float *col, const int width, const int height, int *hist)
 {
@@ -540,17 +537,18 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
       dt_bilateral_free(b);
     }
 
-    float *const weight_buf = malloc(data->n * dt_get_num_threads() * sizeof(float));
+    size_t allocsize;
+    float *const weight_buf = dt_alloc_perthread(data->n, sizeof(float), &allocsize);
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-    dt_omp_firstprivate(ch, height, mapio, var_ratio, weight_buf, width) \
+    dt_omp_firstprivate(ch, height, mapio, var_ratio, weight_buf, width, allocsize) \
     shared(data, in, out, equalization) \
     schedule(static)
 #endif
     for(int k = 0; k < height; k++)
     {
-      float *weight = weight_buf + data->n * dt_get_thread_num();
+      float *weight = dt_get_perthread(weight_buf,allocsize);
       size_t j = (size_t)ch * width * k;
       for(int i = 0; i < width; i++)
       {
@@ -800,17 +798,21 @@ void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
   dt_iop_colormapping_params_t *p = (dt_iop_colormapping_params_t *)self->params;
   dt_iop_colormapping_gui_data_t *g = (dt_iop_colormapping_gui_data_t *)self->gui_data;
 
-  memset(p->source_ihist, 0, sizeof(float) * HISTN);
-  memset(p->source_mean, 0, sizeof(float) * MAXN * 2);
-  memset(p->source_var, 0, sizeof(float) * MAXN * 2);
-  memset(p->source_weight, 0, sizeof(float) * MAXN);
-  memset(p->target_hist, 0, sizeof(int) * HISTN);
-  memset(p->target_mean, 0, sizeof(float) * MAXN * 2);
-  memset(p->target_var, 0, sizeof(float) * MAXN * 2);
-  memset(p->target_weight, 0, sizeof(float) * MAXN);
-  p->flag = NEUTRAL;
-  dt_control_queue_redraw_widget(g->source_area);
-  dt_control_queue_redraw_widget(g->target_area);
+  if(w == g->clusters)
+  {
+    // only reset source/target when changing number of clusters
+    memset(p->source_ihist, 0, sizeof(float) * HISTN);
+    memset(p->source_mean, 0, sizeof(float) * MAXN * 2);
+    memset(p->source_var, 0, sizeof(float) * MAXN * 2);
+    memset(p->source_weight, 0, sizeof(float) * MAXN);
+    memset(p->target_hist, 0, sizeof(int) * HISTN);
+    memset(p->target_mean, 0, sizeof(float) * MAXN * 2);
+    memset(p->target_var, 0, sizeof(float) * MAXN * 2);
+    memset(p->target_weight, 0, sizeof(float) * MAXN);
+    p->flag = NEUTRAL;
+    dt_control_queue_redraw_widget(g->source_area);
+    dt_control_queue_redraw_widget(g->target_area);
+  }
 }
 
 static void acquire_source_button_pressed(GtkButton *button, dt_iop_module_t *self)
@@ -838,7 +840,6 @@ static void acquire_target_button_pressed(GtkButton *button, dt_iop_module_t *se
 void init_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
   piece->data = malloc(sizeof(dt_iop_colormapping_data_t));
-  self->commit_params(self, self->default_params, pipe, piece);
 }
 
 void cleanup_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
@@ -880,22 +881,16 @@ void reload_defaults(dt_iop_module_t *module)
 {
   dt_iop_colormapping_params_t *d = module->default_params;
 
-  // we might be called from presets update infrastructure => there is no image
-  if(module->dev)
-  { 
-    dt_iop_colormapping_gui_data_t *g = (dt_iop_colormapping_gui_data_t *)module->gui_data;
-    if(module->dev->gui_attached && g && g->flowback_set)
-    {
-      memcpy(d->source_ihist, g->flowback.hist, sizeof(float) * HISTN);
-      memcpy(d->source_mean, g->flowback.mean, sizeof(float) * MAXN * 2);
-      memcpy(d->source_var, g->flowback.var, sizeof(float) * MAXN * 2);
-      memcpy(d->source_weight, g->flowback.weight, sizeof(float) * MAXN);
-      d->n = g->flowback.n;
-      d->flag = HAS_SOURCE;
-    }
+  dt_iop_colormapping_gui_data_t *g = (dt_iop_colormapping_gui_data_t *)module->gui_data;
+  if(module->dev->gui_attached && g && g->flowback_set)
+  {
+    memcpy(d->source_ihist, g->flowback.hist, sizeof(float) * HISTN);
+    memcpy(d->source_mean, g->flowback.mean, sizeof(float) * MAXN * 2);
+    memcpy(d->source_var, g->flowback.var, sizeof(float) * MAXN * 2);
+    memcpy(d->source_weight, g->flowback.weight, sizeof(float) * MAXN);
+    d->n = g->flowback.n;
+    d->flag = HAS_SOURCE;
   }
-
-  memcpy(module->params, module->default_params, sizeof(dt_iop_colormapping_params_t));
 }
 
 
@@ -1050,8 +1045,7 @@ static void process_clusters(gpointer instance, gpointer user_data)
 
 void gui_init(struct dt_iop_module_t *self)
 {
-  self->gui_data = malloc(sizeof(dt_iop_colormapping_gui_data_t));
-  dt_iop_colormapping_gui_data_t *g = (dt_iop_colormapping_gui_data_t *)self->gui_data;
+  dt_iop_colormapping_gui_data_t *g = IOP_GUI_ALLOC(colormapping);
 
   g->flag = NEUTRAL;
   g->flowback_set = 0;
@@ -1064,40 +1058,32 @@ void gui_init(struct dt_iop_module_t *self)
 
   self->widget = GTK_WIDGET(gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE));
 
-  GtkBox *hbox1 = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
-  GtkWidget *source = gtk_label_new(_("source clusters:"));
-  gtk_box_pack_start(GTK_BOX(hbox1), GTK_WIDGET(source), FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(hbox1), TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget), dt_ui_label_new(_("source clusters:")), TRUE, TRUE, 0);
 
   g->source_area = dtgtk_drawing_area_new_with_aspect_ratio(1.0 / 3.0);
   gtk_box_pack_start(GTK_BOX(self->widget), g->source_area, TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(g->source_area), "draw", G_CALLBACK(cluster_preview_draw), self);
 
-  GtkBox *hbox2 = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
-  GtkWidget *target = gtk_label_new(_("target clusters:"));
-  gtk_box_pack_start(GTK_BOX(hbox2), GTK_WIDGET(target), FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(hbox2), TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget), dt_ui_label_new(_("target clusters:")), TRUE, TRUE, 0);
 
   g->target_area = dtgtk_drawing_area_new_with_aspect_ratio(1.0 / 3.0);
   gtk_box_pack_start(GTK_BOX(self->widget), g->target_area, TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(g->target_area), "draw", G_CALLBACK(cluster_preview_draw), self);
 
+  GtkWidget *box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
+  gtk_box_pack_start(GTK_BOX(self->widget), box, TRUE, TRUE, 0);
 
-  GtkBox *box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5));
-  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(box), TRUE, TRUE, 0);
-  GtkWidget *button;
+  g->acquire_source_button = dt_iop_button_new(self, N_("acquire as source"),
+                                               G_CALLBACK(acquire_source_button_pressed), FALSE, 0, 0,
+                                               NULL, 0, box);
+  gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->acquire_source_button))), PANGO_ELLIPSIZE_START);
+  gtk_widget_set_tooltip_text(g->acquire_source_button, _("analyze this image as a source image"));
 
-  button = gtk_button_new_with_label(_("acquire as source"));
-  g->acquire_source_button = button;
-  gtk_widget_set_tooltip_text(button, _("analyze this image as a source image"));
-  gtk_box_pack_start(box, button, TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(acquire_source_button_pressed), (gpointer)self);
-
-  button = gtk_button_new_with_label(_("acquire as target"));
-  g->acquire_target_button = button;
-  gtk_widget_set_tooltip_text(button, _("analyze this image as a target image"));
-  gtk_box_pack_start(box, button, TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(acquire_target_button_pressed), (gpointer)self);
+  g->acquire_target_button = dt_iop_button_new(self, N_("acquire as target"),
+                                               G_CALLBACK(acquire_target_button_pressed), FALSE, 0, 0,
+                                               NULL, 0, box);
+  gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->acquire_target_button))), PANGO_ELLIPSIZE_START);
+  gtk_widget_set_tooltip_text(g->acquire_target_button, _("analyze this image as a target image"));
 
   g->clusters = dt_bauhaus_slider_from_params(self, "n");
   gtk_widget_set_tooltip_text(g->clusters, _("number of clusters to find in image. value change resets all clusters"));
@@ -1112,10 +1098,9 @@ void gui_init(struct dt_iop_module_t *self)
   dt_bauhaus_slider_set_format(g->equalization, "%.02f%%");
 
   /* add signal handler for preview pipe finished: process clusters if requested */
-  dt_control_signal_connect(darktable.signals, DT_SIGNAL_DEVELOP_PREVIEW_PIPE_FINISHED,
+  DT_DEBUG_CONTROL_SIGNAL_CONNECT(darktable.signals, DT_SIGNAL_DEVELOP_PREVIEW_PIPE_FINISHED,
                             G_CALLBACK(process_clusters), self);
 
-
   FILE *f = g_fopen("/tmp/dt_colormapping_loaded", "rb");
   if(f)
   {
@@ -1128,13 +1113,13 @@ void gui_cleanup(struct dt_iop_module_t *self)
 {
   dt_iop_colormapping_gui_data_t *g = (dt_iop_colormapping_gui_data_t *)self->gui_data;
 
-  dt_control_signal_disconnect(darktable.signals, G_CALLBACK(process_clusters), self);
+  DT_DEBUG_CONTROL_SIGNAL_DISCONNECT(darktable.signals, G_CALLBACK(process_clusters), self);
 
   cmsDeleteTransform(g->xform);
   dt_pthread_mutex_destroy(&g->lock);
   free(g->buffer);
-  free(self->gui_data);
-  self->gui_data = NULL;
+
+  IOP_GUI_FREE;
 }
 
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
