@@ -21,6 +21,7 @@
 #include "bauhaus/bauhaus.h"
 #include "common/imageio.h"
 #include "common/opencl.h"
+#include "common/tea.h"
 #include "control/control.h"
 #include "develop/develop.h"
 #include "develop/imageop.h"
@@ -44,7 +45,6 @@
 #endif
 
 #define CLIP(x) ((x < 0) ? 0.0 : (x > 1.0) ? 1.0 : x)
-#define TEA_ROUNDS 8
 
 DT_MODULE_INTROSPECTION(1, dt_iop_dither_params_t)
 
@@ -104,9 +104,18 @@ const char *name()
   return _("dithering");
 }
 
+const char *description(struct dt_iop_module_t *self)
+{
+  return dt_iop_set_description(self, _("reduce banding and posterization effects in output JPEGs by adding random noise"),
+                                      _("corrective"),
+                                      _("non-linear, RGB, display-referred"),
+                                      _("non-linear, RGB"),
+                                      _("non-linear, RGB, display-referred"));
+}
+
 int default_group()
 {
-  return IOP_GROUP_CORRECT;
+  return IOP_GROUP_CORRECT | IOP_GROUP_TECHNICAL;
 }
 
 int flags()
@@ -119,18 +128,6 @@ int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_p
   return iop_cs_rgb;
 }
 
-void init_key_accels(dt_iop_module_so_t *self)
-{
-  dt_accel_register_combobox_iop(self, FALSE, NC_("accel", "method"));
-}
-
-void connect_key_accels(dt_iop_module_t *self)
-{
-  dt_iop_dither_gui_data_t *g = (dt_iop_dither_gui_data_t *)self->gui_data;
-
-  dt_accel_connect_combobox_iop(self, "method", GTK_WIDGET(g->dither_type));
-}
-
 void init_presets(dt_iop_module_so_t *self)
 {
   DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db), "BEGIN", NULL, NULL, NULL);
@@ -138,7 +135,9 @@ void init_presets(dt_iop_module_so_t *self)
   dt_iop_dither_params_t tmp
       = (dt_iop_dither_params_t){ DITHER_FSAUTO, 0, { 0.0f, { 0.0f, 0.0f, 1.0f, 1.0f }, -200.0f } };
   // add the preset.
-  dt_gui_presets_add_generic(_("dither"), self->op, self->version(), &tmp, sizeof(dt_iop_dither_params_t), 1);
+  dt_gui_presets_add_generic(_("dither"), self->op,
+                             self->version(), &tmp, sizeof(dt_iop_dither_params_t), 1,
+                             DEVELOP_BLEND_CS_NONE);
   // make it auto-apply for all images:
   // dt_gui_presets_update_autoapply(_("dither"), self->op, self->version(), 1);
 
@@ -151,10 +150,10 @@ static void _find_nearest_color_n_levels_gray(float *val, float *err, const floa
 {
   const float in = 0.30f * val[0] + 0.59f * val[1] + 0.11f * val[2]; // RGB -> GRAY
 
-  float tmp = in * f;
-  int itmp = floorf(tmp);
+  const float tmp = in * f;
+  const int itmp = floorf(tmp);
 
-  float new = (tmp - itmp > 0.5f ? (float)(itmp + 1) : (float)itmp) * rf;
+  const float new = (tmp - itmp > 0.5f ? (float)(itmp + 1) : (float)itmp) * rf;
 
   for(int c = 0; c < 4; c++)
   {
@@ -172,8 +171,8 @@ static __m128 _find_nearest_color_n_levels_gray_sse(float *val, const float f, c
 
   const float in = 0.30f * val[0] + 0.59f * val[1] + 0.11f * val[2]; // RGB -> GRAY
 
-  float tmp = in * f;
-  int itmp = floorf(tmp);
+  const float tmp = in * f;
+  const int itmp = floorf(tmp);
 
   new = _mm_set1_ps(tmp - itmp > 0.5f ? (float)(itmp + 1) * rf : (float)itmp * rf);
   err = _mm_sub_ps(_mm_load_ps(val), new);
@@ -582,70 +581,44 @@ static void process_floyd_steinberg_sse2(struct dt_iop_module_t *self, dt_dev_pi
 }
 #endif
 
-
-static void encrypt_tea(unsigned int *arg)
-{
-  const unsigned int key[] = { 0xa341316c, 0xc8013ea4, 0xad90777d, 0x7e95761e };
-  unsigned int v0 = arg[0], v1 = arg[1];
-  unsigned int sum = 0;
-  unsigned int delta = 0x9e3779b9;
-  for(int i = 0; i < TEA_ROUNDS; i++)
-  {
-    sum += delta;
-    v0 += ((v1 << 4) + key[0]) ^ (v1 + sum) ^ ((v1 >> 5) + key[1]);
-    v1 += ((v0 << 4) + key[2]) ^ (v0 + sum) ^ ((v0 >> 5) + key[3]);
-  }
-  arg[0] = v0;
-  arg[1] = v1;
-}
-
-
-static float tpdf(unsigned int urandom)
-{
-  float frandom = (float)urandom / (float)0xFFFFFFFFu;
-
-  return (frandom < 0.5f ? (sqrtf(2.0f * frandom) - 1.0f) : (1.0f - sqrtf(2.0f * (1.0f - frandom))));
-}
-
-
 static void process_random(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece,
                            const void *const ivoid, void *const ovoid, const dt_iop_roi_t *const roi_in,
                            const dt_iop_roi_t *const roi_out)
 {
-  dt_iop_dither_data_t *data = (dt_iop_dither_data_t *)piece->data;
+  const dt_iop_dither_data_t *const data = (dt_iop_dither_data_t *)piece->data;
 
   const int width = roi_in->width;
   const int height = roi_in->height;
-  const int ch = piece->colors;
+  assert(piece->colors == 4);
 
   const float dither = powf(2.0f, data->random.damping / 10.0f);
 
-  unsigned int *const tea_states = calloc(2 * dt_get_num_threads(), sizeof(unsigned int));
+  unsigned int *const tea_states = alloc_tea_states(dt_get_num_threads());
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-  dt_omp_firstprivate(ch, dither, height, ivoid, ovoid, tea_states, width) \
+  dt_omp_firstprivate(dither, height, ivoid, ovoid, tea_states, width) \
   schedule(static)
 #endif
   for(int j = 0; j < height; j++)
   {
-    const size_t k = (size_t)ch * width * j;
-    const float *in = (const float *)ivoid + k;
-    float *out = (float *)ovoid + k;
-    unsigned int *tea_state = tea_states + 2 * dt_get_thread_num();
-    tea_state[0] = j * height + dt_get_thread_num();
-    for(int i = 0; i < width; i++, in += ch, out += ch)
+    const size_t k = (size_t)4 * width * j;
+    const float *const in = (const float *)ivoid + k;
+    float *const out = (float *)ovoid + k;
+    unsigned int *tea_state = get_tea_state(tea_states,dt_get_thread_num());
+    tea_state[0] = j * height; /* + dt_get_thread_num() -- do not include, makes results unreproducible */
+    for(int i = 0; i < width; i++)
     {
       encrypt_tea(tea_state);
       float dith = dither * tpdf(tea_state[0]);
 
-      out[0] = CLIP(in[0] + dith);
-      out[1] = CLIP(in[1] + dith);
-      out[2] = CLIP(in[2] + dith);
+      out[4*i+0] = CLIP(in[4*i+0] + dith);
+      out[4*i+1] = CLIP(in[4*i+1] + dith);
+      out[4*i+2] = CLIP(in[4*i+2] + dith);
     }
   }
 
-  free(tea_states);
+  free_tea_states(tea_states);
 
   if(piece->pipe->mask_display & DT_DEV_PIXELPIPE_DISPLAY_MASK) dt_iop_alpha_copy(ivoid, ovoid, width, height);
 }
@@ -728,7 +701,6 @@ void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pix
 void init_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
   piece->data = malloc(sizeof(dt_iop_dither_data_t));
-  self->commit_params(self, self->default_params, pipe, piece);
 }
 
 void cleanup_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
@@ -740,9 +712,8 @@ void cleanup_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev
 
 void gui_update(struct dt_iop_module_t *self)
 {
-  dt_iop_module_t *module = (dt_iop_module_t *)self;
   dt_iop_dither_gui_data_t *g = (dt_iop_dither_gui_data_t *)self->gui_data;
-  dt_iop_dither_params_t *p = (dt_iop_dither_params_t *)module->params;
+  dt_iop_dither_params_t *p = (dt_iop_dither_params_t *)self->params;
   dt_bauhaus_combobox_set(g->dither_type, p->dither_type);
 #if 0
   dt_bauhaus_slider_set(g->radius, p->random.radius);
@@ -756,19 +727,18 @@ void gui_update(struct dt_iop_module_t *self)
   dt_bauhaus_slider_set(g->damping, p->random.damping);
 
   gtk_widget_set_visible(g->random, p->dither_type == DITHER_RANDOM);
-} 
+}
 
 void gui_init(struct dt_iop_module_t *self)
 {
-  self->gui_data = malloc(sizeof(dt_iop_dither_gui_data_t));
-  dt_iop_dither_gui_data_t *g = (dt_iop_dither_gui_data_t *)self->gui_data;
+  dt_iop_dither_gui_data_t *g = IOP_GUI_ALLOC(dither);
 
   g->random = self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
 
 #if 0
   g->radius = dt_bauhaus_slider_new_with_range(self, 0.0, 200.0, 0.1, p->random.radius, 2);
   gtk_widget_set_tooltip_text(g->radius, _("radius for blurring step"));
-  dt_bauhaus_widget_set_label(g->radius, NULL, _("radius"));
+  dt_bauhaus_widget_set_label(g->radius, NULL, N_("radius"));
 
   g->range = dtgtk_gradient_slider_multivalue_new(4);
   dtgtk_gradient_slider_multivalue_set_marker(DTGTK_GRADIENT_SLIDER(g->range), GRADIENT_SLIDER_MARKER_LOWER_OPEN_BIG, 0);
