@@ -36,6 +36,7 @@
 #endif
 
 #include "bauhaus/bauhaus.h"
+#include "common/box_filters.h"
 #include "common/darktable.h"
 #include "common/guided_filter.h"
 #include "develop/imageop.h"
@@ -100,6 +101,20 @@ const char *name()
 }
 
 
+const char *aliases()
+{
+  return _("dehaze|defog|smoke|smog");
+}
+
+const char *description(struct dt_iop_module_t *self)
+{
+  return dt_iop_set_description(self, _("remove fog and atmospheric hazing from pictures"),
+                                      _("corrective"),
+                                      _("linear, RGB, scene-referred"),
+                                      _("frequential, RGB"),
+                                      _("linear, RGB, scene-referred"));
+}
+
 int flags()
 {
   return IOP_FLAGS_INCLUDE_IN_STYLES | IOP_FLAGS_SUPPORTS_BLENDING;
@@ -108,7 +123,7 @@ int flags()
 
 int default_group()
 {
-  return IOP_GROUP_CORRECT;
+  return IOP_GROUP_CORRECT | IOP_GROUP_TECHNICAL;
 }
 
 
@@ -117,24 +132,10 @@ int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_p
   return iop_cs_rgb;
 }
 
-void init_key_accels(dt_iop_module_so_t *self)
-{
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "strength"));
-  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "distance"));
-}
-
-void connect_key_accels(dt_iop_module_t *self)
-{
-  dt_iop_hazeremoval_gui_data_t *g = (dt_iop_hazeremoval_gui_data_t *)self->gui_data;
-
-  dt_accel_connect_slider_iop(self, "strength", GTK_WIDGET(g->strength));
-  dt_accel_connect_slider_iop(self, "distance", GTK_WIDGET(g->distance));
-}
 
 void init_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
   piece->data = calloc(1, sizeof(dt_iop_hazeremoval_data_t));
-  self->commit_params(self, self->default_params, pipe, piece);
 }
 
 
@@ -192,8 +193,7 @@ void gui_update(struct dt_iop_module_t *self)
 
 void gui_init(dt_iop_module_t *self)
 {
-  self->gui_data = malloc(sizeof(dt_iop_hazeremoval_gui_data_t));
-  dt_iop_hazeremoval_gui_data_t *g = (dt_iop_hazeremoval_gui_data_t *)self->gui_data;
+  dt_iop_hazeremoval_gui_data_t *g = IOP_GUI_ALLOC(hazeremoval);
 
   dt_pthread_mutex_init(&g->lock, NULL);
   g->distance_max = NAN;
@@ -216,8 +216,8 @@ void gui_cleanup(dt_iop_module_t *self)
 {
   dt_iop_hazeremoval_gui_data_t *g = (dt_iop_hazeremoval_gui_data_t *)self->gui_data;
   dt_pthread_mutex_destroy(&g->lock);
-  free(self->gui_data);
-  self->gui_data = NULL;
+
+  IOP_GUI_FREE;
 }
 
 //----------------------------------------------------------------------
@@ -244,47 +244,6 @@ typedef struct const_rgb_image
 } const_rgb_image;
 
 
-typedef struct gray_image
-{
-  float *data;
-  int width, height;
-} gray_image;
-
-
-// allocate space for 1-component image of size width x height
-static inline gray_image new_gray_image(int width, int height)
-{
-  return (gray_image){ dt_alloc_align(64, sizeof(float) * width * height), width, height };
-}
-
-
-// free space for 1-component image
-static inline void free_gray_image(gray_image *img_p)
-{
-  dt_free_align(img_p->data);
-  img_p->data = NULL;
-}
-
-
-// copy 1-component image img1 to img2
-static inline void copy_gray_image(gray_image img1, gray_image img2)
-{
-  memcpy(img2.data, img1.data, sizeof(float) * img1.width * img1.height);
-}
-
-
-// minimum of two integers
-static inline int min_i(int a, int b)
-{
-  return a < b ? a : b;
-}
-
-
-// maximum of two integers
-static inline int max_i(int a, int b)
-{
-  return a > b ? a : b;
-}
 
 
 // swap the two floats that the pointers point to
@@ -296,164 +255,6 @@ static inline void pointer_swap_f(float *a, float *b)
 }
 
 
-// calculate the one-dimensional moving maximum over a window of size 2*w+1
-// input array x has stride 1, output array y has stride stride_y
-static inline void box_max_1d(int N, const float *x, float *y, size_t stride_y, int w)
-{
-  float m = -(INFINITY);
-  for(int i = 0, i_end = min_i(w + 1, N); i < i_end; i++) m = fmaxf(x[i], m);
-  for(int i = 0; i < N; i++)
-  {
-    y[i * stride_y] = m;
-    if(i - w >= 0 && x[i - w] == m)
-    {
-      m = -(INFINITY);
-      for(int j = max_i(i - w + 1, 0), j_end = min_i(i + w + 2, N); j < j_end; j++) m = fmaxf(x[j], m);
-    }
-    if(i + w + 1 < N) m = fmaxf(x[i + w + 1], m);
-  }
-}
-
-
-// calculate the two-dimensional moving maximum over a box of size (2*w+1) x (2*w+1)
-// does the calculation in-place if input and output images are identical
-static void box_max(const gray_image img1, const gray_image img2, const int w)
-{
-  gray_image img2_bak;
-  if(img1.data == img2.data)
-  {
-#ifdef _OPENMP
-#pragma omp parallel default(none) \
-    dt_omp_firstprivate(img1, img2, w) \
-    private(img2_bak)
-#endif
-    {
-      img2_bak = new_gray_image(img2.width, 1);
-#ifdef _OPENMP
-#pragma omp for
-#endif
-      for(int i1 = 0; i1 < img2.height; i1++)
-      {
-        memcpy(img2_bak.data, img2.data + (size_t)i1 * img2.width, sizeof(float) * img2.width);
-        box_max_1d(img2.width, img2_bak.data, img2.data + (size_t)i1 * img2.width, 1, w);
-      }
-      free_gray_image(&img2_bak);
-    }
-  }
-  else
-  {
-#ifdef _OPENMP
-#pragma omp parallel default(none) \
-    dt_omp_firstprivate(img1, img2, w) \
-    private(img2_bak)
-#endif
-    {
-#ifdef _OPENMP
-#pragma omp for
-#endif
-      for(int i1 = 0; i1 < img1.height; i1++)
-        box_max_1d(img1.width, img1.data + (size_t)i1 * img1.width, img2.data + (size_t)i1 * img2.width, 1, w);
-    }
-  }
-#ifdef _OPENMP
-#pragma omp parallel default(none) \
-  dt_omp_firstprivate(img1, img2, w) \
-  private(img2_bak)
-#endif
-  {
-    img2_bak = new_gray_image(1, img2.height);
-#ifdef _OPENMP
-#pragma omp for
-#endif
-    for(int i0 = 0; i0 < img1.width; i0++)
-    {
-      for(int i1 = 0; i1 < img1.height; i1++) img2_bak.data[i1] = img2.data[i0 + (size_t)i1 * img2.width];
-      box_max_1d(img1.height, img2_bak.data, img2.data + i0, img1.width, w);
-    }
-    free_gray_image(&img2_bak);
-  }
-}
-
-
-// calculate the one-dimensional moving minimum over a window of size 2*w+1
-// input array x has stride 1, output array y has stride stride_y
-static inline void box_min_1d(int N, const float *x, float *y, size_t stride_y, int w)
-{
-  float m = INFINITY;
-  for(int i = 0, i_end = min_i(w + 1, N); i < i_end; i++) m = fminf(x[i], m);
-  for(int i = 0; i < N; i++)
-  {
-    y[i * stride_y] = m;
-    if(i - w >= 0 && x[i - w] == m)
-    {
-      m = INFINITY;
-      for(int j = max_i(i - w + 1, 0), j_end = min_i(i + w + 2, N); j < j_end; j++) m = fminf(x[j], m);
-    }
-    if(i + w + 1 < N) m = fminf(x[i + w + 1], m);
-  }
-}
-
-
-// calculate the two-dimensional moving minimum over a box of size (2*w+1) x (2*w+1)
-// does the calculation in-place if input and output images are identical
-static void box_min(const gray_image img1, const gray_image img2, const int w)
-{
-  gray_image img2_bak;
-  if(img1.data == img2.data)
-  {
-#ifdef _OPENMP
-#pragma omp parallel default(none) \
-    dt_omp_firstprivate(img1, img2, w) \
-    private(img2_bak)
-#endif
-    {
-      img2_bak = new_gray_image(img2.width, 1);
-#ifdef _OPENMP
-#pragma omp for
-#endif
-      for(int i1 = 0; i1 < img2.height; i1++)
-      {
-        memcpy(img2_bak.data, img2.data + (size_t)i1 * img2.width, sizeof(float) * img2.width);
-        box_min_1d(img2.width, img2_bak.data, img2.data + (size_t)i1 * img2.width, 1, w);
-      }
-      free_gray_image(&img2_bak);
-    }
-  }
-  else
-  {
-#ifdef _OPENMP
-#pragma omp parallel default(none) \
-    dt_omp_firstprivate(img1, img2, w) \
-    private(img2_bak)
-#endif
-    {
-#ifdef _OPENMP
-#pragma omp for
-#endif
-      for(int i1 = 0; i1 < img1.height; i1++)
-        box_min_1d(img1.width, img1.data + (size_t)i1 * img1.width, img2.data + (size_t)i1 * img2.width, 1, w);
-    }
-  }
-#ifdef _OPENMP
-#pragma omp parallel default(none) \
-  dt_omp_firstprivate(img1, img2, w) \
-  private(img2_bak)
-#endif
-  {
-    img2_bak = new_gray_image(1, img2.height);
-#ifdef _OPENMP
-#pragma omp for schedule(static)
-#endif
-    for(int i0 = 0; i0 < img1.width; i0++)
-    {
-      for(int i1 = 0; i1 < img1.height; i1++) img2_bak.data[i1] = img2.data[i0 + (size_t)i1 * img2.width];
-      box_min_1d(img1.height, img2_bak.data, img2.data + i0, img1.width, w);
-    }
-    free_gray_image(&img2_bak);
-  }
-}
-
-
 // calculate the dark channel (minimal color component over a box of size (2*w+1) x (2*w+1) )
 static void dark_channel(const const_rgb_image img1, const gray_image img2, const int w)
 {
@@ -471,7 +272,7 @@ static void dark_channel(const const_rgb_image img1, const gray_image img2, cons
     m = fminf(pixel[2], m);
     img2.data[i] = m;
   }
-  box_min(img2, img2, w);
+  dt_box_min(img2.data, img2.height, img2.width, 1, w);
 }
 
 
@@ -493,7 +294,7 @@ static void transition_map(const const_rgb_image img1, const gray_image img2, co
     m = fminf(pixel[2] / A0[2], m);
     img2.data[i] = 1.f - m * strength;
   }
-  box_max(img2, img2, w);
+  dt_box_max(img2.data, img2.height, img2.width, 1, w);
 }
 
 
@@ -702,7 +503,7 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
   transition_map(img_in, trans_map, w1, A0, strength);
 
   // refine the transition map
-  box_min(trans_map, trans_map, w1);
+  dt_box_min(trans_map.data, trans_map.height, trans_map.width, 1, w1);
   gray_image trans_map_filtered = new_gray_image(width, height);
   // apply guided filter with no clipping
   guided_filter(img_in.data, trans_map.data, trans_map_filtered.data, width, height, ch, w2, eps, 1.f, -FLT_MAX,
