@@ -36,7 +36,7 @@ typedef struct dt_iop_scalepixels_params_t
   // Aspect ratio of the pixels, usually 1 but some cameras need scaling
   // <1 means the image needs to be stretched vertically, (0.5 means 2x)
   // >1 means the image needs to be stretched horizontally (2 mean 2x)
-  float pixel_aspect_ratio;
+  float pixel_aspect_ratio; // $DEFAULT: 1.0f
 } dt_iop_scalepixels_params_t;
 
 typedef struct dt_iop_scalepixels_gui_data_t
@@ -49,8 +49,6 @@ typedef struct dt_iop_scalepixels_data_t {
   float y_scale;
 } dt_iop_scalepixels_data_t;
 
-static dt_iop_scalepixels_gui_data_t dummy;
-
 const char *name()
 {
   return C_("modulename", "scale pixels");
@@ -58,12 +56,13 @@ const char *name()
 
 int flags()
 {
-  return IOP_FLAGS_ALLOW_TILING | IOP_FLAGS_TILING_FULL_ROI | IOP_FLAGS_ONE_INSTANCE;
+  return IOP_FLAGS_ALLOW_TILING | IOP_FLAGS_TILING_FULL_ROI | IOP_FLAGS_ONE_INSTANCE
+    | IOP_FLAGS_UNSAFE_COPY;
 }
 
 int default_group()
 {
-  return IOP_GROUP_CORRECT;
+  return IOP_GROUP_CORRECT | IOP_GROUP_TECHNICAL;
 }
 
 int operation_tags()
@@ -76,6 +75,12 @@ int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_p
   return iop_cs_rgb;
 }
 
+const char *description(struct dt_iop_module_t *self)
+{
+  return g_strdup(_("internal module to setup technical specificities of raw sensor.\n\n"
+                    "you should not touch values here !"));
+}
+
 static void transform(const dt_dev_pixelpipe_iop_t *const piece, float *p)
 {
   dt_iop_scalepixels_data_t *d = piece->data;
@@ -235,7 +240,6 @@ void commit_params(dt_iop_module_t *self, dt_iop_params_t *params, dt_dev_pixelp
 void init_pipe(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
   piece->data = calloc(1, sizeof(dt_iop_scalepixels_data_t));
-  self->commit_params(self, self->default_params, pipe, piece);
 }
 
 void cleanup_pipe(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
@@ -246,65 +250,36 @@ void cleanup_pipe(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelp
 
 void reload_defaults(dt_iop_module_t *self)
 {
-  dt_iop_scalepixels_params_t tmp = (dt_iop_scalepixels_params_t){ .pixel_aspect_ratio = 1.0f };
-
-  // we might be called from presets update infrastructure => there is no image
-  if(!self->dev) goto end;
+  dt_iop_scalepixels_params_t *d = self->default_params;
 
   const dt_image_t *const image = &(self->dev->image_storage);
 
-  tmp.pixel_aspect_ratio = image->pixel_aspect_ratio;
+  d->pixel_aspect_ratio = image->pixel_aspect_ratio;
 
-  self->default_enabled
-      = (!isnan(tmp.pixel_aspect_ratio) && tmp.pixel_aspect_ratio > 0.0f && tmp.pixel_aspect_ratio != 1.0f);
+  self->default_enabled = (!isnan(d->pixel_aspect_ratio) &&
+                           d->pixel_aspect_ratio > 0.0f &&
+                           d->pixel_aspect_ratio != 1.0f);
 
   // FIXME: does not work.
   self->hide_enable_button = !self->default_enabled;
 
-end:
-  memcpy(self->params, &tmp, sizeof(dt_iop_scalepixels_params_t));
-  memcpy(self->default_params, &tmp, sizeof(dt_iop_scalepixels_params_t));
+  if(self->widget)
+    gtk_label_set_text(GTK_LABEL(self->widget), self->default_enabled
+                       ? _("automatic pixel scaling")
+                       :_("automatic pixel scaling\nonly works for the sensors that need it."));
 }
 
 void gui_update(dt_iop_module_t *self)
 {
-  if(!self->widget) return;
-  if(self->default_enabled)
-    gtk_label_set_text(GTK_LABEL(self->widget), _("automatic pixel scaling"));
-  else
-    gtk_label_set_text(GTK_LABEL(self->widget), _("automatic pixel scaling only works for the sensors that need it."));
-}
-
-void init(dt_iop_module_t *self)
-{
-  const dt_image_t *const image = &(self->dev->image_storage);
-
-  self->params = calloc(1, sizeof(dt_iop_scalepixels_params_t));
-  self->default_params = calloc(1, sizeof(dt_iop_scalepixels_params_t));
-  self->default_enabled = (!isnan(image->pixel_aspect_ratio) && image->pixel_aspect_ratio > 0.0f
-                           && image->pixel_aspect_ratio != 1.0f);
-  self->params_size = sizeof(dt_iop_scalepixels_params_t);
-  self->gui_data = &dummy;
-}
-
-void cleanup(dt_iop_module_t *self)
-{
-  free(self->params);
-  self->params = NULL;
-  free(self->default_params);
-  self->default_params = NULL;
 }
 
 void gui_init(dt_iop_module_t *self)
 {
-  self->widget = gtk_label_new("");
+  IOP_GUI_ALLOC(scalepixels);
+
+  self->widget = dt_ui_label_new("");
   gtk_label_set_line_wrap(GTK_LABEL(self->widget), TRUE);
-  gtk_widget_set_halign(self->widget, GTK_ALIGN_START);
-}
 
-void gui_cleanup(dt_iop_module_t *self)
-{
-  self->gui_data = NULL;
 }
 
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
