@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2019 philippe weyland.
+    Copyright (C) 2019-2020 darktable developers.
 
 
     darktable is free software: you can redistribute it and/or modify
@@ -56,7 +56,7 @@ typedef struct dt_lib_export_metadata_t
   GList *taglist;
 } dt_lib_export_metadata_t;
 
-GList *dt_get_exiv2_taglist();
+GList *dt_exif_get_exiv2_taglist();
 
 // find a string on the list
 static gboolean find_metadata_iter_per_text(GtkTreeModel *model, GtkTreeIter *iter, gint col, const char *text)
@@ -373,7 +373,7 @@ char *dt_lib_export_metadata_configuration_dialog(char *metadata_presets, const
   gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(liststore), DT_LIB_EXPORT_METADATA_COL_XMP, GTK_SORT_ASCENDING);
   gtk_tree_view_set_model(view, GTK_TREE_MODEL(liststore));
   g_object_unref(liststore);
-  d->taglist = dt_get_exiv2_taglist();
+  d->taglist = (GList *)dt_exif_get_exiv2_taglist();
   GList *list = dt_util_str_to_glist("\1", metadata_presets);
   int32_t flags = 0;
   if (list)
@@ -414,12 +414,12 @@ char *dt_lib_export_metadata_configuration_dialog(char *metadata_presets, const
   box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, TRUE, 0);
 
-  GtkWidget *button = dtgtk_button_new(dtgtk_cairo_paint_plus_simple, CPF_STYLE_FLAT | CPF_DO_NOT_USE_BORDER, NULL);
+  GtkWidget *button = dtgtk_button_new(dtgtk_cairo_paint_plus_simple, CPF_STYLE_FLAT, NULL);
   gtk_widget_set_tooltip_text(button, _("add an output metadata tag"));
   gtk_box_pack_end(GTK_BOX(box), button, FALSE, TRUE, 0);
   g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(add_tag_button_clicked), (gpointer)d);
 
-  button = dtgtk_button_new(dtgtk_cairo_paint_minus_simple, CPF_STYLE_FLAT | CPF_DO_NOT_USE_BORDER, NULL);
+  button = dtgtk_button_new(dtgtk_cairo_paint_minus_simple, CPF_STYLE_FLAT, NULL);
   gtk_widget_set_tooltip_text(button, _("delete metadata tag"));
   gtk_box_pack_end(GTK_BOX(box), button, FALSE, TRUE, 0);
   g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(delete_tag_button_clicked), (gpointer)d);
@@ -462,7 +462,6 @@ char *dt_lib_export_metadata_configuration_dialog(char *metadata_presets, const
     g_free(metadata_presets);
     dt_lib_export_metadata_set_conf(newlist);
   }
-  g_list_free_full(d->taglist, g_free);
   gtk_widget_destroy(dialog);
   free(d);
   return newlist;
