@@ -30,6 +30,11 @@
 #include "libs/lib.h"
 #include "libs/lib_api.h"
 #include "common/history.h"
+#include <complex.h>
+
+#ifdef GDK_WINDOWING_QUARTZ
+#include "osx/osx.h"
+#endif
 
 DT_MODULE(1)
 
@@ -48,6 +53,9 @@ typedef struct dt_lib_history_t
   GtkWidget *create_button;
   GtkWidget *compress_button;
   gboolean record_undo;
+  int record_history_level; // set to +1 in signal DT_SIGNAL_DEVELOP_HISTORY_WILL_CHANGE
+                            // and back to -1 in DT_SIGNAL_DEVELOP_HISTORY_CHANGE. We want
+                            // to avoid multiple will-change before a change cb.
   // previous_* below store values sent by signal DT_SIGNAL_DEVELOP_HISTORY_WILL_CHANGE
   GList *previous_snapshot;
   int previous_history_end;
@@ -60,7 +68,11 @@ typedef struct dt_lib_history_t
 #define HIST_WIDGET_STATUS 2
 
 /* compress history stack */
-static void _lib_history_compress_clicked_callback(GtkWidget *widget, gpointer user_data);
+static gboolean _lib_compress_stack_accel(GtkAccelGroup *accel_group, GObject *acceleratable, guint keyval,
+                                          GdkModifierType modifier, gpointer data);
+static gboolean _lib_truncate_stack_accel(GtkAccelGroup *accel_group, GObject *acceleratable, guint keyval,
+                                          GdkModifierType modifier, gpointer data);
+static void _lib_history_compress_clicked_callback(GtkWidget *widget, GdkEventButton *e, gpointer user_data);
 static void _lib_history_button_clicked_callback(GtkWidget *widget, gpointer user_data);
 static void _lib_history_create_style_button_clicked_callback(GtkWidget *widget, gpointer user_data);
 /* signal callback for history change */
@@ -95,6 +107,7 @@ void init_key_accels(dt_lib_module_t *self)
   dt_accel_register_lib(self, NC_("accel", "create style from history"), 0, 0);
 //   dt_accel_register_lib(self, NC_("accel", "apply style from popup menu"), 0, 0);
   dt_accel_register_lib(self, NC_("accel", "compress history stack"), 0, 0);
+  dt_accel_register_lib(self, NC_("accel", "truncate history stack"), 0, 0);
 }
 
 void connect_key_accels(dt_lib_module_t *self)
@@ -103,10 +116,14 @@ void connect_key_accels(dt_lib_module_t *self)
 
   dt_accel_connect_button_lib(self, "create style from history", d->create_button);
 //   dt_accel_connect_button_lib(self, "apply style from popup menu", d->apply_button);
-  dt_accel_connect_button_lib(self, "compress history stack", d->compress_button);
+  GClosure *closure;
+  closure = g_cclosure_new(G_CALLBACK(_lib_compress_stack_accel), (gpointer)self, NULL);
+  dt_accel_connect_lib(self, "compress history stack", closure);
+
+  closure = g_cclosure_new(G_CALLBACK(_lib_truncate_stack_accel), (gpointer)self, NULL);
+  dt_accel_connect_lib(self, "truncate history stack", closure);
 }
 
-#define ellipsize_button(button) gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(button))), PANGO_ELLIPSIZE_END);
 void gui_init(dt_lib_module_t *self)
 {
   /* initialize ui widgets */
@@ -114,6 +131,7 @@ void gui_init(dt_lib_module_t *self)
   self->data = (void *)d;
 
   d->record_undo = TRUE;
+  d->record_history_level = 0;
   d->previous_snapshot = NULL;
   d->previous_history_end = 0;
   d->previous_iop_order_list = NULL;
@@ -121,14 +139,15 @@ void gui_init(dt_lib_module_t *self)
   self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
   dt_gui_add_help_link(self->widget, dt_get_help_url(self->plugin_name));
   gtk_widget_set_name(self->widget, "history-ui");
+
   d->history_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
   GtkWidget *hhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
-  d->compress_button = gtk_button_new_with_label(_("compress history stack"));
-  ellipsize_button(d->compress_button);
-  gtk_widget_set_tooltip_text(d->compress_button, _("create a minimal history stack which produces the same image"));
-  g_signal_connect(G_OBJECT(d->compress_button), "clicked", G_CALLBACK(_lib_history_compress_clicked_callback), self);
+  d->compress_button = dt_ui_button_new(_("compress history stack"),
+                                        _("create a minimal history stack which produces the same image\n"
+                                          "ctrl-click to truncate history to the selected item"), NULL);
+  g_signal_connect(G_OBJECT(d->compress_button), "button-press-event", G_CALLBACK(_lib_history_compress_clicked_callback), self);
 
   /* add toolbar button for creating style */
   d->create_button = dtgtk_button_new(dtgtk_cairo_paint_styles, CPF_NONE, NULL);
@@ -142,25 +161,25 @@ void gui_init(dt_lib_module_t *self)
   gtk_box_pack_start(GTK_BOX(hhbox), d->create_button, FALSE, FALSE, 0);
 
   /* add history list and buttonbox to widget */
-  gtk_box_pack_start(GTK_BOX(self->widget), d->history_box, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget),
+                     dt_ui_scroll_wrap(d->history_box, 1, "plugins/darkroom/history/windowheight"), FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(self->widget), hhbox, FALSE, FALSE, 0);
 
-
   gtk_widget_show_all(self->widget);
 
   /* connect to history change signal for updating the history view */
-  dt_control_signal_connect(darktable.signals, DT_SIGNAL_DEVELOP_HISTORY_WILL_CHANGE,
+  DT_DEBUG_CONTROL_SIGNAL_CONNECT(darktable.signals, DT_SIGNAL_DEVELOP_HISTORY_WILL_CHANGE,
                             G_CALLBACK(_lib_history_will_change_callback), self);
-  dt_control_signal_connect(darktable.signals, DT_SIGNAL_DEVELOP_HISTORY_CHANGE,
+  DT_DEBUG_CONTROL_SIGNAL_CONNECT(darktable.signals, DT_SIGNAL_DEVELOP_HISTORY_CHANGE,
                             G_CALLBACK(_lib_history_change_callback), self);
-  dt_control_signal_connect(darktable.signals, DT_SIGNAL_DEVELOP_MODULE_REMOVE,
+  DT_DEBUG_CONTROL_SIGNAL_CONNECT(darktable.signals, DT_SIGNAL_DEVELOP_MODULE_REMOVE,
                             G_CALLBACK(_lib_history_module_remove_callback), self);
 }
 
 void gui_cleanup(dt_lib_module_t *self)
 {
-  dt_control_signal_disconnect(darktable.signals, G_CALLBACK(_lib_history_change_callback), self);
-  dt_control_signal_disconnect(darktable.signals, G_CALLBACK(_lib_history_module_remove_callback), self);
+  DT_DEBUG_CONTROL_SIGNAL_DISCONNECT(darktable.signals, G_CALLBACK(_lib_history_change_callback), self);
+  DT_DEBUG_CONTROL_SIGNAL_DISCONNECT(darktable.signals, G_CALLBACK(_lib_history_module_remove_callback), self);
   g_free(self->data);
   self->data = NULL;
 }
@@ -180,9 +199,10 @@ static GtkWidget *_lib_history_create_button(dt_lib_module_t *self, int num, con
 
   /* create toggle button */
   GtkWidget *widget = gtk_toggle_button_new_with_label(label);
-  gtk_widget_set_halign(gtk_bin_get_child(GTK_BIN(widget)), GTK_ALIGN_START);
-  ellipsize_button(widget);
-
+  GtkWidget *lab = gtk_bin_get_child(GTK_BIN(widget));
+  gtk_widget_set_halign(lab, GTK_ALIGN_START);
+  gtk_label_set_xalign(GTK_LABEL(lab), 0);
+  gtk_label_set_ellipsize(GTK_LABEL(lab), PANGO_ELLIPSIZE_END);
   if(always_on)
   {
     onoff = dtgtk_button_new(dtgtk_cairo_paint_switch_on, CPF_STYLE_FLAT | CPF_BG_TRANSPARENT, NULL);
@@ -234,7 +254,6 @@ static GtkWidget *_lib_history_create_button(dt_lib_module_t *self, int num, con
 
   return hbox;
 }
-#undef ellipsize_button
 
 static void _reset_module_instance(GList *hist, dt_iop_module_t *module, int multi_priority)
 {
@@ -400,9 +419,8 @@ static int _check_deleted_instances(dt_develop_t *dev, GList **_iop_list, GList
       dt_undo_iterate_internal(darktable.undo, DT_UNDO_HISTORY, mod, &_history_invalidate_cb);
 
       // we cleanup the module
-      dt_accel_disconnect_list(&mod->accel_closures);
-      dt_accel_cleanup_locals_iop(mod);
-      mod->accel_closures = NULL;
+      dt_accel_cleanup_closures_iop(mod);
+
       // don't delete the module, a pipe may still need it
       dev->alliop = g_list_append(dev->alliop, mod);
 
@@ -653,7 +671,7 @@ static void _lib_history_will_change_callback(gpointer instance, GList *history,
   dt_lib_module_t *self = (dt_lib_module_t *)user_data;
   dt_lib_history_t *lib = (dt_lib_history_t *)self->data;
 
-  if(lib->record_undo)
+  if(lib->record_undo && (lib->record_history_level == 0))
   {
     // history is about to change, we want here ot record a snapshot of the history for the undo
     // record previous history
@@ -663,6 +681,333 @@ static void _lib_history_will_change_callback(gpointer instance, GList *history,
     lib->previous_history_end = history_end;
     lib->previous_iop_order_list = iop_order_list;
   }
+
+  lib->record_history_level += 1;
+}
+
+static gchar *_lib_history_change_text(dt_introspection_field_t *field, const char *d, dt_iop_params_t *params, dt_iop_params_t *oldpar)
+{
+  dt_iop_params_t *p = params + field->header.offset;
+  dt_iop_params_t *o = oldpar + field->header.offset;
+
+  switch(field->header.type)
+  {
+  case DT_INTROSPECTION_TYPE_STRUCT:
+  case DT_INTROSPECTION_TYPE_UNION:
+    {
+      gchar **change_parts = g_malloc0_n(field->Struct.entries + 1, sizeof(char*));
+      int num_parts = 0;
+
+      for(int i = 0; i < field->Struct.entries; i++)
+      {
+        dt_introspection_field_t *entry = field->Struct.fields[i];
+
+        gchar *description = _(*entry->header.description ?
+                                entry->header.description :
+                                entry->header.field_name);
+
+        if(d) description = g_strdup_printf("%s.%s", d, description);
+
+        if((change_parts[num_parts] = _lib_history_change_text(entry, description, params, oldpar)))
+          num_parts++;
+
+        if(d) g_free(description);
+      }
+
+      gchar *struct_text = num_parts ? g_strjoinv("\n", change_parts) : NULL;
+      g_strfreev(change_parts);
+
+      return struct_text;
+    }
+    break;
+  case DT_INTROSPECTION_TYPE_ARRAY:
+    if(field->Array.type == DT_INTROSPECTION_TYPE_CHAR)
+    {
+      if(strncmp((char*)o, (char*)p, field->Array.count))
+        return g_strdup_printf("%s\t\"%s\"\t\u2192\t\"%s\"", d, (char*)o, (char*)p);
+    }
+    else
+    {
+      const int max_elements = 4;
+      gchar **change_parts = g_malloc0_n(max_elements + 1, sizeof(char*));
+      int num_parts = 0;
+
+      for(int i = 0, item_offset = 0; i < field->Array.count; i++, item_offset += field->Array.field->header.size)
+      {
+        char *description = g_strdup_printf("%s[%d]", d, i);
+        char *element_text = _lib_history_change_text(field->Array.field, description, params + item_offset, oldpar + item_offset);
+        g_free(description);
+
+        if(element_text && ++num_parts <= max_elements)
+          change_parts[num_parts - 1] = element_text;
+        else
+          g_free(element_text);
+      }
+
+      gchar *array_text = NULL;
+      if(num_parts > max_elements)
+        array_text = g_strdup_printf("%s\t%d changes", d, num_parts);
+      else if(num_parts > 0)
+        array_text = g_strjoinv("\n", change_parts);
+
+      g_strfreev(change_parts);
+
+      return array_text;
+    }
+    break;
+  case DT_INTROSPECTION_TYPE_FLOAT:
+    if(*(float*)o != *(float*)p && (isfinite(*(float*)o) || isfinite(*(float*)p)))
+      return g_strdup_printf("%s\t%.4f\t\u2192\t%.4f", d, *(float*)o, *(float*)p);
+    break;
+  case DT_INTROSPECTION_TYPE_INT:
+    if(*(int*)o != *(int*)p)
+      return g_strdup_printf("%s\t%d\t\u2192\t%d", d, *(int*)o, *(int*)p);
+    break;
+  case DT_INTROSPECTION_TYPE_UINT:
+    if(*(unsigned int*)o != *(unsigned int*)p)
+      return g_strdup_printf("%s\t%u\t\u2192\t%u", d, *(unsigned int*)o, *(unsigned int*)p);
+    break;
+  case DT_INTROSPECTION_TYPE_USHORT:
+    if(*(unsigned short int*)o != *(unsigned short int*)p)
+      return g_strdup_printf("%s\t%hu\t\u2192\t%hu", d, *(unsigned short int*)o, *(unsigned short int*)p);
+    break;
+  case DT_INTROSPECTION_TYPE_INT8:
+    if(*(uint8_t*)o != *(uint8_t*)p)
+      return g_strdup_printf("%s\t%d\t\u2192\t%d", d, *(uint8_t*)o, *(uint8_t*)p);
+    break;
+  case DT_INTROSPECTION_TYPE_CHAR:
+    if(*(char*)o != *(char*)p)
+      return g_strdup_printf("%s\t'%c'\t\u2192\t'%c'", d, *(char *)o, *(char *)p);
+    break;
+  case DT_INTROSPECTION_TYPE_FLOATCOMPLEX:
+    if(*(float complex*)o != *(float complex*)p)
+      return g_strdup_printf("%s\t%.4f + %.4fi\t\u2192\t%.4f + %.4fi", d,
+                             creal(*(float complex*)o), cimag(*(float complex*)o),
+                             creal(*(float complex*)p), cimag(*(float complex*)p));
+    break;
+  case DT_INTROSPECTION_TYPE_ENUM:
+    if(*(int*)o != *(int*)p)
+    {
+      const char *old_str = N_("unknown"), *new_str = N_("unknown");
+      for(dt_introspection_type_enum_tuple_t *i = field->Enum.values; i && i->name; i++)
+      {
+        if(i->value == *(int*)o)
+        {
+          old_str = i->description;
+          if(!*old_str) old_str = i->name;
+        }
+        if(i->value == *(int*)p)
+        {
+          new_str = i->description;
+          if(!*new_str) new_str = i->name;
+        }
+      }
+
+      return g_strdup_printf("%s\t%s\t\u2192\t%s", d, _(old_str), _(new_str));
+    }
+    break;
+  case DT_INTROSPECTION_TYPE_BOOL:
+    if(*(gboolean*)o != *(gboolean*)p)
+    {
+      char *old_str = *(gboolean*)o ? "on" : "off";
+      char *new_str = *(gboolean*)p ? "on" : "off";
+      return g_strdup_printf("%s\t%s\t\u2192\t%s", d, _(old_str), _(new_str));
+    }
+    break;
+  case DT_INTROSPECTION_TYPE_OPAQUE:
+    {
+      // TODO: special case float2
+    }
+    break;
+  default:
+    fprintf(stderr, "unsupported introspection type \"%s\" encountered in _lib_history_change_text (field %s)\n", field->header.type_name, field->header.field_name);
+    break;
+  }
+
+  return NULL;
+}
+
+static gboolean _changes_tooltip_callback(GtkWidget *widget, gint x, gint y, gboolean keyboard_mode,
+                                          GtkTooltip *tooltip, const dt_dev_history_item_t *hitem)
+{
+  dt_iop_params_t *old_params = hitem->module->default_params;
+  dt_develop_blend_params_t *old_blend = hitem->module->default_blendop_params;
+
+  GList *find_old = g_list_first(darktable.develop->history);
+  while(find_old && find_old->data != hitem)
+  {
+    const dt_dev_history_item_t *hiprev = (dt_dev_history_item_t *)(find_old->data);
+
+    if(hiprev->module == hitem->module)
+    {
+      old_params = hiprev->params;
+      old_blend = hiprev->blend_params;
+    }
+
+    find_old = g_list_next(find_old);
+  }
+
+  gchar **change_parts = g_malloc0_n(sizeof(dt_develop_blend_params_t) / (sizeof(float)) + 10, sizeof(char*));
+
+  if(hitem->module->have_introspection)
+    change_parts[0] = _lib_history_change_text(hitem->module->get_introspection()->field, NULL,
+                                                hitem->params, old_params);
+  int num_parts = change_parts[0] ? 1 : 0;
+
+  if(hitem->module->flags() & IOP_FLAGS_SUPPORTS_BLENDING)
+  {
+    #define add_blend_history_change(field, format, label)                                       \
+      if((hitem->blend_params->field) != (old_blend->field))                                     \
+      {                                                                                          \
+        gchar *full_format = g_strconcat("%s\t", format, "\t\u2192\t", format, NULL);            \
+        change_parts[num_parts++] = g_strdup_printf(full_format, label,                          \
+                                    (old_blend->field), (hitem->blend_params->field));           \
+        g_free(full_format);                                                                     \
+      }
+
+    #define add_blend_history_change_enum(field, label, list)                                    \
+      if((hitem->blend_params->field) != (old_blend->field))                                     \
+      {                                                                                          \
+        const char *old_str = NULL, *new_str = NULL;                                             \
+        for(const dt_develop_name_value_t *i = list; *i->name; i++)                              \
+        {                                                                                        \
+          if(i->value == (old_blend->field)) old_str = i->name;                                  \
+          if(i->value == (hitem->blend_params->field)) new_str = i->name;                        \
+        }                                                                                        \
+                                                                                                 \
+        change_parts[num_parts++] = (!old_str || !new_str)                                       \
+                                  ? g_strdup_printf("%s\t%d\t\u2192\t%d", label,                 \
+                                                    old_blend->field, hitem->blend_params->field)\
+                                  : g_strdup_printf("%s\t%s\t\u2192\t%s", label,                 \
+                                                    _(g_dpgettext2(NULL, "blendmode", old_str)), \
+                                                    _(g_dpgettext2(NULL, "blendmode", new_str)));\
+      }
+
+    add_blend_history_change_enum(blend_cst, _("colorspace"), dt_develop_blend_colorspace_names);
+    add_blend_history_change_enum(mask_mode, _("mask mode"), dt_develop_mask_mode_names);
+    add_blend_history_change_enum(blend_mode, _("blend mode"), dt_develop_blend_mode_names);
+    add_blend_history_change(blend_parameter, _("%.2f EV"), _("blend fulcrum"));
+    add_blend_history_change(opacity, "%.4f", _("mask opacity"));
+    add_blend_history_change_enum(mask_combine & (DEVELOP_COMBINE_INV | DEVELOP_COMBINE_INCL), _("combine masks"), dt_develop_combine_masks_names);
+    add_blend_history_change(feathering_radius, "%.4f", _("feathering radius"));
+    add_blend_history_change_enum(feathering_guide, _("feathering guide"), dt_develop_feathering_guide_names);
+    add_blend_history_change(blur_radius, "%.4f", _("mask blur"));
+    add_blend_history_change(contrast, "%.4f", _("mask contrast"));
+    add_blend_history_change(brightness, "%.4f", _("brightness"));
+    add_blend_history_change(raster_mask_instance, "%d", _("raster mask instance"));
+    add_blend_history_change(raster_mask_id, "%d", _("raster mask id"));
+    add_blend_history_change_enum(raster_mask_invert, _("invert mask"), dt_develop_invert_mask_names);
+
+    add_blend_history_change(mask_combine & DEVELOP_COMBINE_MASKS_POS ? '-' : '+', "%c", _("drawn mask polarity"));
+
+    if(hitem->blend_params->mask_id != old_blend->mask_id)
+      change_parts[num_parts++] = old_blend->mask_id == 0
+                                ? g_strdup_printf(_("a drawn mask was added"))
+                                : hitem->blend_params->mask_id == 0
+                                ? g_strdup_printf(_("the drawn mask was removed"))
+                                : g_strdup_printf(_("the drawn mask was changed"));
+
+    dt_iop_gui_blend_data_t *bd = hitem->module->blend_data;
+
+    for(int in_out = 1; in_out >= 0; in_out--)
+    {
+      gboolean first = TRUE;
+
+      for(const dt_iop_gui_blendif_channel_t *b = bd ? bd->channel : NULL;
+          b && b->label != NULL;
+          b++)
+      {
+        const dt_develop_blendif_channels_t ch = b->param_channels[in_out];
+
+        const int oactive = old_blend->blendif & (1 << ch);
+        const int nactive = hitem->blend_params->blendif & (1 << ch);
+
+        const int opolarity = old_blend->blendif & (1 << (ch + 16));
+        const int npolarity = hitem->blend_params->blendif & (1 << (ch + 16));
+
+        float *of = &old_blend->blendif_parameters[4 * ch];
+        float *nf = &hitem->blend_params->blendif_parameters[4 * ch];
+
+        const float oboost = exp2f(old_blend->blendif_boost_factors[ch]);
+        const float nboost = exp2f(hitem->blend_params->blendif_boost_factors[ch]);
+
+        if((oactive || nactive) && (memcmp(of, nf, 4 * sizeof(float)) || opolarity != npolarity))
+        {
+          if(first)
+          {
+            change_parts[num_parts++] = g_strdup(in_out ? _("parametric output mask:") : _("parametric input mask:"));
+            first = FALSE;
+          }
+          char s[4][2][25];
+          for(int k = 0; k < 4; k++)
+          {
+            b->scale_print(of[k], oboost, s[k][0], sizeof(s[k][0]));
+            b->scale_print(nf[k], nboost, s[k][1], sizeof(s[k][1]));
+          }
+
+          char *opol = !oactive ? "" : (opolarity ? "(-)" : "(+)");
+          char *npol = !nactive ? "" : (npolarity ? "(-)" : "(+)");
+
+          change_parts[num_parts++] = g_strdup_printf("%s\t%s| %s- %s| %s%s\t\u2192\t%s| %s- %s| %s%s", _(b->name),
+                                                      s[0][0], s[1][0], s[2][0], s[3][0], opol,
+                                                      s[0][1], s[1][1], s[2][1], s[3][1], npol);
+        }
+      }
+    }
+  }
+
+  gchar *tooltip_text = g_strjoinv("\n", change_parts);
+  g_strfreev(change_parts);
+
+  gboolean show_tooltip = *tooltip_text;
+
+  if(show_tooltip)
+  {
+    static GtkWidget *view = NULL;
+    if(!view)
+    {
+      view = gtk_text_view_new();
+      gtk_widget_set_name(view, "history-tooltip");
+      g_signal_connect(G_OBJECT(view), "destroy", G_CALLBACK(gtk_widget_destroyed), &view);
+    }
+
+    GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+    gtk_text_buffer_set_text(buffer, tooltip_text, -1);
+    gtk_tooltip_set_custom(tooltip, view);
+
+    int count_column1 = 0, count_column2 = 0;
+    for(gchar *line = tooltip_text; *line; )
+    {
+      gchar *endline = g_strstr_len(line, -1, "\n");
+      if(!endline) endline = line + strlen(line);
+
+      gchar *found_tab1 = g_strstr_len(line, endline - line, "\t");
+      if(found_tab1)
+      {
+        if(found_tab1 - line >= count_column1) count_column1 = found_tab1 - line + 1;
+
+        gchar *found_tab2 = g_strstr_len(found_tab1 + 1, endline - found_tab1 - 1, "\t");
+        if(found_tab2 - found_tab1 > count_column2) count_column2 = found_tab2 - found_tab1;
+      }
+
+      line = endline;
+      if(*line) line++;
+    }
+
+    PangoLayout *layout = gtk_widget_create_pango_layout(view, " ");
+    int char_width;
+    pango_layout_get_size(layout, &char_width, NULL);
+    g_object_unref(layout);
+    PangoTabArray *tabs = pango_tab_array_new_with_positions(3, FALSE, PANGO_TAB_LEFT, (count_column1) * char_width,
+                                                                       PANGO_TAB_LEFT, (count_column1 + count_column2) * char_width,
+                                                                       PANGO_TAB_LEFT, (count_column1 + count_column2 + 2) * char_width);
+    gtk_text_view_set_tabs(GTK_TEXT_VIEW(view), tabs);
+    pango_tab_array_free(tabs);
+  }
+
+  g_free(tooltip_text);
+
+  return show_tooltip;
 }
 
 static void _lib_history_change_callback(gpointer instance, gpointer user_data)
@@ -677,10 +1022,12 @@ static void _lib_history_change_callback(gpointer instance, gpointer user_data)
   int num = -1;
   GtkWidget *widget =
     _lib_history_create_button(self, num, _("original"), FALSE, FALSE, TRUE, darktable.develop->history_end == 0, FALSE);
-  gtk_box_pack_start(GTK_BOX(d->history_box), widget, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(d->history_box), widget, FALSE, FALSE, 0);
   num++;
 
-  if (d->record_undo == TRUE)
+  d->record_history_level -= 1;
+
+  if (d->record_undo == TRUE && (d->record_history_level == 0))
   {
     /* record undo/redo history snapshot */
     dt_undo_history_t *hist = malloc(sizeof(dt_undo_history_t));
@@ -720,7 +1067,10 @@ static void _lib_history_change_callback(gpointer instance, gpointer user_data)
 
     g_free(label);
 
-    gtk_box_pack_start(GTK_BOX(d->history_box), widget, TRUE, TRUE, 0);
+    gtk_widget_set_has_tooltip(widget, TRUE);
+    g_signal_connect(G_OBJECT(widget), "query-tooltip", G_CALLBACK(_changes_tooltip_callback), (void *)hitem);
+
+    gtk_box_pack_start(GTK_BOX(d->history_box), widget, FALSE, FALSE, 0);
     gtk_box_reorder_child(GTK_BOX(d->history_box), widget, 0);
     num++;
 
@@ -733,20 +1083,21 @@ static void _lib_history_change_callback(gpointer instance, gpointer user_data)
   dt_pthread_mutex_unlock(&darktable.develop->history_mutex);
 }
 
-static void _lib_history_compress_clicked_callback(GtkWidget *widget, gpointer user_data)
+static void _lib_history_truncate(gboolean compress)
 {
   const int32_t imgid = darktable.develop->image_storage.id;
   if(!imgid) return;
 
-  dt_control_signal_raise(darktable.signals, DT_SIGNAL_DEVELOP_HISTORY_WILL_CHANGE,
-                          dt_history_duplicate(darktable.develop->history), darktable.develop->history_end,
-                          dt_ioppr_iop_order_copy_deep(darktable.develop->iop_order_list));
+  dt_dev_undo_start_record(darktable.develop);
 
   // As dt_history_compress_on_image does *not* use the history stack data at all
   // make sure the current stack is in the database
   dt_dev_write_history(darktable.develop);
 
-  dt_history_compress_on_image(imgid);
+  if(compress)
+    dt_history_compress_on_image(imgid);
+  else
+    dt_history_truncate_on_image(imgid, darktable.develop->history_end);
 
   sqlite3_stmt *stmt;
 
@@ -757,8 +1108,10 @@ static void _lib_history_compress_clicked_callback(GtkWidget *widget, gpointer u
 
   // then we can get the item to select in the new clean-up history retrieve the position of the module
   // corresponding to the history end.
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT IFNULL(MAX(num)+1, 0) FROM main.history "
-                                                             "WHERE imgid=?1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "SELECT IFNULL(MAX(num)+1, 0)"
+                              " FROM main.history"
+                              " WHERE imgid=?1", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, imgid);
 
   if (sqlite3_step(stmt) == SQLITE_ROW)
@@ -766,18 +1119,41 @@ static void _lib_history_compress_clicked_callback(GtkWidget *widget, gpointer u
   sqlite3_finalize(stmt);
 
   // select the new history end corresponding to the one before the history compression
-  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "UPDATE main.images SET history_end=?2 WHERE id=?1",
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),
+                              "UPDATE main.images SET history_end=?2 WHERE id=?1",
                               -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, imgid);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, darktable.develop->history_end);
   sqlite3_step(stmt);
   sqlite3_finalize(stmt);
 
+  darktable.develop->proxy.chroma_adaptation = NULL;
   dt_dev_reload_history_items(darktable.develop);
-  dt_control_signal_raise(darktable.signals, DT_SIGNAL_DEVELOP_HISTORY_CHANGE);
+  dt_dev_undo_end_record(darktable.develop);
+
   dt_dev_modulegroups_set(darktable.develop, dt_dev_modulegroups_get(darktable.develop));
 }
 
+static gboolean _lib_compress_stack_accel(GtkAccelGroup *accel_group, GObject *acceleratable, guint keyval,
+                                          GdkModifierType modifier, gpointer data)
+{
+  _lib_history_truncate(TRUE);
+  return TRUE;
+}
+
+static gboolean _lib_truncate_stack_accel(GtkAccelGroup *accel_group, GObject *acceleratable, guint keyval,
+                                          GdkModifierType modifier, gpointer data)
+{
+  _lib_history_truncate(FALSE);
+  return TRUE;
+}
+
+static void _lib_history_compress_clicked_callback(GtkWidget *widget, GdkEventButton *e, gpointer user_data)
+{
+  const gboolean compress = !(e->state & GDK_CONTROL_MASK);
+  _lib_history_truncate(compress);
+}
+
 static void _lib_history_button_clicked_callback(GtkWidget *widget, gpointer user_data)
 {
   static int reset = 0;
@@ -801,19 +1177,19 @@ static void _lib_history_button_clicked_callback(GtkWidget *widget, gpointer use
   reset = 0;
   if(darktable.gui->reset) return;
 
-  dt_control_signal_raise(darktable.signals, DT_SIGNAL_DEVELOP_HISTORY_WILL_CHANGE,
-                          dt_history_duplicate(darktable.develop->history), darktable.develop->history_end,
-                          dt_ioppr_iop_order_copy_deep(darktable.develop->iop_order_list));
+  dt_dev_undo_start_record(darktable.develop);
 
   /* revert to given history item. */
   const int num = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "history-number"));
   dt_dev_pop_history_items(darktable.develop, num);
   // set the module list order
   dt_dev_reorder_gui_module_list(darktable.develop);
+
   /* signal history changed */
-  dt_control_signal_raise(darktable.signals, DT_SIGNAL_DEVELOP_HISTORY_CHANGE);
+  dt_dev_undo_end_record(darktable.develop);
+
   dt_dev_modulegroups_set(darktable.develop, dt_dev_modulegroups_get(darktable.develop));
-  dt_iop_connect_accels_all() ;
+  dt_iop_connect_accels_all();
 }
 
 static void _lib_history_create_style_button_clicked_callback(GtkWidget *widget, gpointer user_data)
@@ -825,6 +1201,43 @@ static void _lib_history_create_style_button_clicked_callback(GtkWidget *widget,
   }
 }
 
+void gui_reset(dt_lib_module_t *self)
+{
+  const int32_t imgid = darktable.develop->image_storage.id;
+  if(!imgid) return;
+
+  gint res = GTK_RESPONSE_YES;
+
+  if(dt_conf_get_bool("ask_before_discard"))
+  {
+    const GtkWidget *win = dt_ui_main_window(darktable.gui->ui);
+
+    GtkWidget *dialog = gtk_message_dialog_new(
+        GTK_WINDOW(win), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
+        _("do you really want to clear history of current image?"));
+#ifdef GDK_WINDOWING_QUARTZ
+    dt_osx_disallow_fullscreen(dialog);
+#endif
+
+    gtk_window_set_title(GTK_WINDOW(dialog), _("delete image's history?"));
+    res = gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+  }
+
+  if(res == GTK_RESPONSE_YES)
+  {
+    dt_dev_undo_start_record(darktable.develop);
+
+    dt_history_delete_on_image_ext(imgid, FALSE);
+
+    dt_dev_undo_end_record(darktable.develop);
+
+    dt_dev_modulegroups_set(darktable.develop, dt_dev_modulegroups_get(darktable.develop));
+
+    dt_control_queue_redraw_center();
+  }
+}
+
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
