@@ -19,6 +19,7 @@
 #pragma once
 
 #include "common/darktable.h"
+#include "common/colorspaces.h"
 #include "views/view.h"
 #include <gmodule.h>
 #include <gtk/gtk.h>
@@ -64,6 +65,16 @@ typedef struct dt_lib_t
       void (*set_sample_point)(struct dt_lib_module_t *self, float x, float y);
     } colorpicker;
 
+    /** Histogram processing hooks */
+    struct
+    {
+      struct dt_lib_module_t *module;
+      void (*process)(struct dt_lib_module_t *self, const float *const input,
+                      int width, int height,
+                      dt_colorspaces_color_profile_type_t icc_type, const gchar *icc_filename);
+      // FIXME: now that PR #5532 is merged, define this as dt_atomic_int and include "common/atomic.h" and use dt_atomic_set_int() and dt_atomic_get_int()
+      gboolean is_linear;
+    } histogram;
   } proxy;
 } dt_lib_t;
 
@@ -73,8 +84,6 @@ typedef struct dt_lib_module_t
 
   /** opened module. */
   GModule *module;
-  /** reference for dlopened libs. */
-  darktable_t *dt;
   /** other stuff that may be needed by the module, not only in gui mode. */
   void *data;
   /** string identifying this operation. */
@@ -83,6 +92,10 @@ typedef struct dt_lib_module_t
   GtkWidget *widget;
   /** expander containing the widget. */
   GtkWidget *expander;
+  /** callback for delayed update after user interaction */
+  void (*_postponed_update)(struct dt_lib_module_t *self);
+  /** ID of timer for delayed callback */
+  guint timeout_handle;
 
   /** version */
   int (*version)(void);
@@ -131,6 +144,7 @@ typedef struct dt_lib_module_t
   void *(*get_params)(struct dt_lib_module_t *self, int *size);
   int (*set_params)(struct dt_lib_module_t *self, const void *params, int size);
   void (*init_presets)(struct dt_lib_module_t *self);
+  void (*manage_presets)(struct dt_lib_module_t *self);
   /** Optional callbacks for keyboard accelerators */
   void (*init_key_accels)(struct dt_lib_module_t *self);
   void (*connect_key_accels)(struct dt_lib_module_t *self);
@@ -170,7 +184,23 @@ gchar *dt_lib_get_localized_name(const gchar *plugin_name);
 
 /** add or replace a preset for this operation. */
 void dt_lib_presets_add(const char *name, const char *plugin_name, const int32_t version, const void *params,
-                        const int32_t params_size);
+                        const int32_t params_size, gboolean readonly);
+
+/** queue a delayed call of update function after user interaction */
+void dt_lib_queue_postponed_update(dt_lib_module_t *mod, void (*update_fn)(dt_lib_module_t *self));
+/** cancel any previously-queued callback */
+void dt_lib_cancel_postponed_update(dt_lib_module_t *mod);
+
+// apply a preset to the given module
+gboolean dt_lib_presets_apply(const gchar *preset, gchar *module_name, int module_version);
+// duplicate a preset
+gchar *dt_lib_presets_duplicate(const gchar *preset, gchar *module_name, int module_version);
+// remove a preset
+void dt_lib_presets_remove(const gchar *preset, gchar *module_name, int module_version);
+// update a preset
+void dt_lib_presets_update(const gchar *preset, gchar *module_name, int module_version, const gchar *newname,
+                           const gchar *desc, const void *params, const int32_t params_size);
+
 
 /*
  * Proxy functions
