@@ -15,6 +15,7 @@
     You should have received a copy of the GNU General Public License
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
+#include "bauhaus/bauhaus.h"
 #include "common/collection.h"
 #include "common/darktable.h"
 #include "common/debug.h"
@@ -50,7 +51,8 @@ uint32_t container(dt_lib_module_t *self)
 
 typedef struct dt_lib_map_settings_t
 {
-  GtkWidget *show_osd_checkbutton, *map_source_dropdown;
+  GtkWidget *show_osd_checkbutton, *filtered_images_checkbutton, *map_source_dropdown;
+  GtkWidget *epsilon_factor, *min_images;
 } dt_lib_map_settings_t;
 
 int position()
@@ -63,6 +65,18 @@ static void _show_osd_toggled(GtkToggleButton *button, gpointer data)
   dt_view_map_show_osd(darktable.view_manager, gtk_toggle_button_get_active(button));
 }
 
+static void _filtered_images_toggled(GtkToggleButton *button, gpointer data)
+{
+  dt_conf_set_bool("plugins/map/filter_images_drawn", gtk_toggle_button_get_active(button));
+  if(darktable.view_manager->proxy.map.view)
+  {
+    if(dt_conf_get_bool("plugins/map/filter_images_drawn"))
+      darktable.view_manager->proxy.map.display_selected(darktable.view_manager->proxy.map.view);
+    else
+      darktable.view_manager->proxy.map.redraw(darktable.view_manager->proxy.map.view);
+  }
+}
+
 static void _map_source_changed(GtkWidget *widget, gpointer data)
 {
   GtkTreeIter iter;
@@ -82,6 +96,26 @@ static void _map_source_changed(GtkWidget *widget, gpointer data)
   }
 }
 
+static void _epsilon_factor_callback(GtkWidget *slider, gpointer data)
+{
+  const float epsilon = dt_bauhaus_slider_get(slider);
+  dt_conf_set_int("plugins/map/epsilon_factor", epsilon);
+  if(darktable.view_manager->proxy.map.view)
+  {
+    darktable.view_manager->proxy.map.redraw(darktable.view_manager->proxy.map.view);
+  }
+}
+
+static void _min_images_callback(GtkWidget *slider, gpointer data)
+{
+  const int min_images = dt_bauhaus_slider_get(slider);
+  dt_conf_set_int("plugins/map/min_images_per_group", min_images);
+  if(darktable.view_manager->proxy.map.view)
+  {
+    darktable.view_manager->proxy.map.redraw(darktable.view_manager->proxy.map.view);
+  }
+}
+
 void gui_init(dt_lib_module_t *self)
 {
   dt_lib_map_settings_t *d = (dt_lib_map_settings_t *)malloc(sizeof(dt_lib_map_settings_t));
@@ -91,13 +125,24 @@ void gui_init(dt_lib_module_t *self)
   GtkBox *hbox;
   GtkWidget *label;
 
+  hbox = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
+
   d->show_osd_checkbutton = gtk_check_button_new_with_label(_("show OSD"));
   gtk_widget_set_tooltip_text(d->show_osd_checkbutton, _("toggle the visibility of the map overlays"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->show_osd_checkbutton),
                                dt_conf_get_bool("plugins/map/show_map_osd"));
-  gtk_box_pack_start(GTK_BOX(self->widget), d->show_osd_checkbutton, TRUE, TRUE, 0);
+  gtk_box_pack_start(hbox, d->show_osd_checkbutton, TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(d->show_osd_checkbutton), "toggled", G_CALLBACK(_show_osd_toggled), NULL);
 
+  d->filtered_images_checkbutton = gtk_check_button_new_with_label(_("filtered images"));
+  gtk_widget_set_tooltip_text(d->filtered_images_checkbutton, _("when set limit the images drawn to the current filmstrip"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->filtered_images_checkbutton),
+                               dt_conf_get_bool("plugins/map/filter_images_drawn"));
+  gtk_box_pack_start(hbox, d->filtered_images_checkbutton, TRUE, TRUE, 0);
+  g_signal_connect(G_OBJECT(d->filtered_images_checkbutton), "toggled", G_CALLBACK(_filtered_images_toggled), NULL);
+
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(hbox), TRUE, TRUE, 0);
+
   hbox = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
 
   label = gtk_label_new(_("map source"));
@@ -131,6 +176,19 @@ void gui_init(dt_lib_module_t *self)
 
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(hbox), TRUE, TRUE, 0);
 
+  d->epsilon_factor = dt_bauhaus_slider_new_with_range(NULL, 10.0, 100.0, 1.0,
+                            dt_conf_get_int("plugins/map/epsilon_factor"), 0);
+  gtk_widget_set_tooltip_text(d->epsilon_factor, _("modify the spatial size of an images group on the map"));
+  dt_bauhaus_widget_set_label(d->epsilon_factor, NULL, N_("group size factor"));
+  g_signal_connect(G_OBJECT(d->epsilon_factor), "value-changed", G_CALLBACK(_epsilon_factor_callback), self);
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(d->epsilon_factor), TRUE, TRUE, 0);
+  d->min_images = dt_bauhaus_slider_new_with_range(NULL, 1.0, 10.0, 1.0,
+                            dt_conf_get_int("plugins/map/min_images_per_group"), 0);
+  gtk_widget_set_tooltip_text(d->min_images, _("minimum of images to make a group"));
+  dt_bauhaus_widget_set_label(d->min_images, NULL, N_("min images per group"));
+  g_signal_connect(G_OBJECT(d->min_images), "value-changed", G_CALLBACK(_min_images_callback), self);
+  gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(d->min_images), TRUE, TRUE, 0);
+
   g_object_unref(model);
   g_free(map_source);
 }
