@@ -1,6 +1,6 @@
 /*
    This file is part of darktable,
-   copyright (c) 2012 Jeremy Rosen
+   Copyright (C) 2014-2020 darktable developers.
 
    darktable is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -157,12 +157,14 @@ static int write_image(lua_State *L)
 
 
   dt_lua_unlock();
+  // TODO: expose these to the user!
   gboolean high_quality = dt_conf_get_bool("plugins/lighttable/export/high_quality_processing");
+  gboolean export_masks = dt_conf_get_bool("plugins/lighttable/export/export_masks");
   // TODO: expose icc overwrites to the user!
   dt_colorspaces_color_profile_type_t icc_type = dt_conf_get_int("plugins/lighttable/export/icctype");
   gchar *icc_filename = dt_conf_get_string("plugins/lighttable/export/iccprofile");
-  gboolean result = dt_imageio_export(imgid, filename, format, fdata, high_quality, upscale, FALSE, icc_type,
-                                      icc_filename, DT_INTENT_LAST, NULL, NULL, 1, 1, NULL);
+  gboolean result = dt_imageio_export(imgid, filename, format, fdata, high_quality, upscale, FALSE, export_masks,
+                                      icc_type, icc_filename, DT_INTENT_LAST, NULL, NULL, 1, 1, NULL);
   g_free(icc_filename);
   dt_lua_lock();
   lua_pushboolean(L, result);
